@@ -5,6 +5,7 @@ if sys.version_info < (3,):
 else:
     unicode = str
 
+import os
 from matplotlib import rc
 from matplotlib import rcParams
 
@@ -40,9 +41,21 @@ rcParams["text.latex.preamble"] = [#"\usepackage{times}",
                                    r"\usepackage{amssymb}",
                                    r"\usepackage{amsmath}"]
 
-from numpy import *
 import scipy
 import scipy.stats
-from math import *
 import numpy as np
+from pylab import *
+from numpy import *
 import graph_tool.all as gt
+import random as prandom
+
+figure()
+
+try:
+    gt.openmp_set_num_threads(1)
+except RuntimeError:
+    pass
+
+prandom.seed(42)
+np.random.seed(42)
+gt.seed_rng(42)
