@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -34,25 +34,19 @@ using namespace graph_tool;
 boost::python::tuple global_clustering(GraphInterface& g)
 {
     double c, c_err;
-    bool directed = g.get_directed();
-    g.set_directed(false);
     run_action<graph_tool::detail::never_directed>()
         (g, std::bind(get_global_clustering(), std::placeholders::_1,
                       std::ref(c), std::ref(c_err)))();
-    g.set_directed(directed);
     return boost::python::make_tuple(c, c_err);
 }
 
 void local_clustering(GraphInterface& g, boost::any prop)
 {
-    bool directed = g.get_directed();
-    g.set_directed(false);
-    run_action<graph_tool::detail::never_directed>()
+    run_action<>()
         (g, std::bind(set_clustering_to_property(),
                       std::placeholders::_1,
                       std::placeholders::_2),
          writable_vertex_scalar_properties())(prop);
-    g.set_directed(directed);
 }
 
 using namespace boost::python;
@@ -64,6 +58,7 @@ void get_motifs(GraphInterface& g, size_t k, boost::python::list subgraph_list,
 
 BOOST_PYTHON_MODULE(libgraph_tool_clustering)
 {
+    docstring_options dopt(true, false);
     def("global_clustering", &global_clustering);
     def("local_clustering", &local_clustering);
     def("extended_clustering", &extended_clustering);
