@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -58,7 +58,7 @@ struct prop_vector
                             any_cast<Property>(props[i]).get_unchecked(size);
                     prop_vec = vec;
                 }
-                catch (bad_any_cast){}
+                catch (bad_any_cast&){}
             }
         }
     };
