@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -71,7 +71,7 @@ get_vertex_correlation_histogram(GraphInterface& gi,
 
     try
     {
-        run_action<>()(gi, get_correlation_histogram<GetNeighboursPairs>
+        run_action<>()(gi, get_correlation_histogram<GetNeighborsPairs>
                        (hist, bins, ret_bins),
                        scalar_selectors(), scalar_selectors(),
                        boost::mpl::vector<cweight_map_t>())
