@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -37,7 +37,7 @@ void graph_correlations_imp1(GraphInterface& g, boost::python::object& hist,
 {
     typedef DynamicPropertyMapWrap<long double, GraphInterface::edge_t>
         wrapped_weight_t;
-    run_action<>()(g, get_correlation_histogram<GetNeighboursPairs>
+    run_action<>()(g, get_correlation_histogram<GetNeighborsPairs>
                    (hist, bins, ret_bins),
                    scalar_selectors(), scalar_selectors(),
                    boost::mpl::vector<wrapped_weight_t>())
