@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -29,9 +29,12 @@
 
 #include <iostream>
 #include <array>
-#include <chrono>
 
 #include "hash_map_wrap.hh"
+#include "demangle.hh"
+
+#include "coroutine.hh"
+#include "graph_python_interface.hh"
 
 #include <cairommconfig.h>
 #include <cairomm/context.h>
@@ -40,6 +43,8 @@
 
 #include <boost/mpl/map/map50.hpp>
 
+#include <chrono>
+
 using namespace std;
 using namespace boost;
 using namespace graph_tool;
@@ -108,7 +113,8 @@ enum vertex_shape_t {
     SHAPE_DOUBLE_HEXAGON,
     SHAPE_DOUBLE_HEPTAGON,
     SHAPE_DOUBLE_OCTAGON,
-    SHAPE_PIE
+    SHAPE_PIE,
+    SHAPE_NONE
 };
 
 enum edge_marker_t {
@@ -234,14 +240,14 @@ struct Converter
         }
         catch (bad_lexical_cast&)
         {
-            string name1 = python::detail::gcc_demangle(typeid(Type1).name());
-            string name2 = python::detail::gcc_demangle(typeid(Type2).name());
+            string name1 = name_demangle(typeid(Type1).name());
+            string name2 = name_demangle(typeid(Type2).name());
             string val_name;
             try
             {
                 val_name = lexical_cast<string>(v);
             }
-            catch (bad_lexical_cast)
+            catch (bad_lexical_cast&)
             {
                 val_name = "<no lexical cast available>";
             }
@@ -431,10 +437,8 @@ public:
         catch (bad_any_cast&)
         {
             throw ValueException("Error getting attribute " + lexical_cast<string>(k) +
-                                 ", wanted: " +
-                                 boost::python::detail::gcc_demangle(typeid(Value).name()) +
-                                 ", got: " +
-                                 boost::python::detail::gcc_demangle(_defaults[k].type().name()));
+                                 ", wanted: " + name_demangle(typeid(Value).name()) +
+                                 ", got: " + name_demangle(_defaults[k].type().name()));
         }
     }
 
@@ -627,7 +631,7 @@ public:
         size = get_user_dist(cr, size);
 
         string text = _attrs.template get<string>(VERTEX_TEXT);
-        if (text != "")
+        if (!text.empty())
         {
             double text_pos = _attrs.template get<double>(VERTEX_TEXT_POSITION);
             if (text_pos == -1)
@@ -701,6 +705,8 @@ public:
         case SHAPE_PIE:
             dr = r;
             break;
+        case SHAPE_NONE:
+            break;
         default:
             throw ValueException("Invalid vertex shape: " +
                                  lexical_cast<string>(int(_attrs.template get<vertex_shape_t>(VERTEX_SHAPE))));
@@ -818,119 +824,120 @@ public:
         }
 
         boost::python::object osrc = _attrs.template get<boost::python::object>(VERTEX_SURFACE);
-        if (osrc == boost::python::object() || outline)
+
+        pw =_attrs.template get<double>(VERTEX_PENWIDTH);
+        pw = get_user_dist(cr, pw);
+        cr.set_line_width(pw);
+
+        if (!outline)
         {
-            pw =_attrs.template get<double>(VERTEX_PENWIDTH);
-            pw = get_user_dist(cr, pw);
-            cr.set_line_width(pw);
-
-            if (!outline)
-            {
-                color = _attrs.template get<color_t>(VERTEX_COLOR);
-                cr.set_source_rgba(get<0>(color), get<1>(color), get<2>(color),
-                                   get<3>(color));
-            }
-
-            size_t nsides = 0;
-            vertex_shape_t shape = _attrs.template get<vertex_shape_t>(VERTEX_SHAPE);
-            switch (shape)
-            {
-            case SHAPE_CIRCLE:
-            case SHAPE_DOUBLE_CIRCLE:
-                if (aspect != 1.)
-                {
-                    cr.save();
-                    cr.scale(aspect, 1.0);
-                }
-                cr.arc(0, 0, size / 2., 0, 2 * M_PI);
-                cr.close_path();
-                if (aspect != 1.)
-                    cr.restore();
-                if (shape == SHAPE_DOUBLE_CIRCLE && !outline)
-                {
-                    cr.stroke();
-                    if (aspect != 1.)
-                    {
-                        cr.save();
-                        cr.scale(aspect, 1.0);
-                    }
-                    cr.arc(0, 0, min(size / 2 - 2 * pw,
-                                     size * 0.8 / 2),
-                           0, 2 * M_PI);
-                    if (aspect != 1.)
-                        cr.restore();
-                }
-                break;
-            case SHAPE_PIE:
-                {
-                    if (!outline)
-                    {
-                        vector<double> f = _attrs.template get<vector<double> >(VERTEX_PIE_FRACTIONS);
-                        vector<color_t> fcolors = _attrs.template get<vector<color_t> >(VERTEX_PIE_COLORS);
-                        draw_pie(size / 2 + pw / 2, f, fcolors, cr);
-                    }
-                    else
-                    {
-                        cr.arc(0, 0, size / 2., 0, 2 * M_PI);
-                        cr.close_path();
-                    }
-                }
-                break;
-            case SHAPE_TRIANGLE:
-            case SHAPE_SQUARE:
-            case SHAPE_PENTAGON:
-            case SHAPE_HEXAGON:
-            case SHAPE_HEPTAGON:
-            case SHAPE_OCTAGON:
-            case SHAPE_DOUBLE_TRIANGLE:
-            case SHAPE_DOUBLE_SQUARE:
-            case SHAPE_DOUBLE_PENTAGON:
-            case SHAPE_DOUBLE_HEXAGON:
-            case SHAPE_DOUBLE_HEPTAGON:
-            case SHAPE_DOUBLE_OCTAGON:
-                nsides = shape - SHAPE_TRIANGLE + 3;
-                if (nsides > 8)
-                    nsides -= 7;
-                if (aspect != 1.)
-                {
-                    cr.save();
-                    cr.scale(aspect, 1.0);
-                }
-                draw_polygon(nsides, size / 2, cr);
-                if (aspect != 1.)
-                    cr.restore();
-                if (shape >= SHAPE_DOUBLE_TRIANGLE && !outline)
-                {
-                    cr.stroke();
-                    if (aspect != 1.)
-                    {
-                        cr.save();
-                        cr.scale(aspect, 1.0);
-                    }
-                    draw_polygon(nsides, min(size / 2 - 2 * pw,
-                                             size * 0.8 / 2), cr);
-                    if (aspect != 1.)
-                        cr.restore();
-                }
-                break;
-            default:
-                throw ValueException("Invalid vertex shape: " +
-                                     lexical_cast<string>(int(_attrs.template get<vertex_shape_t>(VERTEX_SHAPE))));
-            }
-
-            if (!outline && shape != SHAPE_PIE)
-            {
-                fillcolor = _attrs.template get<color_t>(VERTEX_FILL_COLOR);
-                cr.set_source_rgba(get<0>(fillcolor), get<1>(fillcolor),
-                                   get<2>(fillcolor), get<3>(fillcolor));
-                cr.fill_preserve();
-
-                cr.set_source_rgba(get<0>(color), get<1>(color), get<2>(color),
-                                   get<3>(color));
-                cr.stroke();
-            }
+            color = _attrs.template get<color_t>(VERTEX_COLOR);
+            cr.set_source_rgba(get<0>(color), get<1>(color), get<2>(color),
+                               get<3>(color));
         }
-        else
+
+        size_t nsides = 0;
+        vertex_shape_t shape = _attrs.template get<vertex_shape_t>(VERTEX_SHAPE);
+        switch (shape)
+        {
+        case SHAPE_CIRCLE:
+        case SHAPE_DOUBLE_CIRCLE:
+            if (aspect != 1.)
+            {
+                cr.save();
+                cr.scale(aspect, 1.0);
+            }
+            cr.arc(0, 0, size / 2., 0, 2 * M_PI);
+            cr.close_path();
+            if (aspect != 1.)
+                cr.restore();
+            if (shape == SHAPE_DOUBLE_CIRCLE && !outline)
+            {
+                cr.stroke();
+                if (aspect != 1.)
+                {
+                    cr.save();
+                    cr.scale(aspect, 1.0);
+                }
+                cr.arc(0, 0, min(size / 2 - 2 * pw,
+                                 size * 0.8 / 2),
+                       0, 2 * M_PI);
+                if (aspect != 1.)
+                    cr.restore();
+            }
+            break;
+        case SHAPE_PIE:
+            {
+                if (!outline)
+                {
+                    vector<double> f = _attrs.template get<vector<double> >(VERTEX_PIE_FRACTIONS);
+                    vector<color_t> fcolors = _attrs.template get<vector<color_t> >(VERTEX_PIE_COLORS);
+                    draw_pie(size / 2 + pw / 2, f, fcolors, cr);
+                }
+                else
+                {
+                    cr.arc(0, 0, size / 2., 0, 2 * M_PI);
+                    cr.close_path();
+                }
+            }
+            break;
+        case SHAPE_TRIANGLE:
+        case SHAPE_SQUARE:
+        case SHAPE_PENTAGON:
+        case SHAPE_HEXAGON:
+        case SHAPE_HEPTAGON:
+        case SHAPE_OCTAGON:
+        case SHAPE_DOUBLE_TRIANGLE:
+        case SHAPE_DOUBLE_SQUARE:
+        case SHAPE_DOUBLE_PENTAGON:
+        case SHAPE_DOUBLE_HEXAGON:
+        case SHAPE_DOUBLE_HEPTAGON:
+        case SHAPE_DOUBLE_OCTAGON:
+            nsides = shape - SHAPE_TRIANGLE + 3;
+            if (nsides > 8)
+                nsides -= 7;
+            if (aspect != 1.)
+            {
+                cr.save();
+                cr.scale(aspect, 1.0);
+            }
+            draw_polygon(nsides, size / 2, cr);
+            if (aspect != 1.)
+                cr.restore();
+            if (shape >= SHAPE_DOUBLE_TRIANGLE && !outline)
+            {
+                cr.stroke();
+                if (aspect != 1.)
+                {
+                    cr.save();
+                    cr.scale(aspect, 1.0);
+                }
+                draw_polygon(nsides, min(size / 2 - 2 * pw,
+                                         size * 0.8 / 2), cr);
+                if (aspect != 1.)
+                    cr.restore();
+            }
+            break;
+        case SHAPE_NONE:
+            break;
+        default:
+            throw ValueException("Invalid vertex shape: " +
+                                 lexical_cast<string>(int(_attrs.template get<vertex_shape_t>(VERTEX_SHAPE))));
+        }
+
+        if (!outline && shape != SHAPE_PIE)
+        {
+            fillcolor = _attrs.template get<color_t>(VERTEX_FILL_COLOR);
+            cr.set_source_rgba(get<0>(fillcolor), get<1>(fillcolor),
+                               get<2>(fillcolor), get<3>(fillcolor));
+            cr.fill_preserve();
+
+            cr.set_source_rgba(get<0>(color), get<1>(color), get<2>(color),
+                               get<3>(color));
+            cr.stroke();
+        }
+
+        if (osrc != boost::python::object() && !outline)
         {
             double swidth, sheight;
             PycairoSurface* src = (PycairoSurface*) osrc.ptr();
@@ -955,7 +962,7 @@ public:
         if (!outline)
         {
             string text = _attrs.template get<string>(VERTEX_TEXT);
-            if (text != "")
+            if (!text.empty())
             {
                 cr.save();
 
@@ -1303,7 +1310,7 @@ public:
         }
 
         string text = _attrs.template get<string>(EDGE_TEXT);
-        if (text != "")
+        if (!text.empty())
         {
             cr.save();
             cr.select_font_face(_attrs.template get<string>(EDGE_FONT_FAMILY),
@@ -1315,9 +1322,6 @@ public:
 
             bool text_parallel = _attrs.template get<uint8_t>(EDGE_TEXT_PARALLEL);
 
-            Cairo::TextExtents extents;
-            cr.get_text_extents(text, extents);
-
             pos_t origin;
             if (controls.size() < 8)
             {
@@ -1352,6 +1356,9 @@ public:
                     angle += M_PI;
                 cr.rotate(angle);
                 cr.translate(0, -text_dist);
+
+                Cairo::TextExtents extents;
+                cr.get_text_extents(text, extents);
                 cr.translate(-extents.width / 2, 0);
             }
             else
@@ -1537,22 +1544,14 @@ private:
     AttrDict<Descriptor> _attrs;
 };
 
-template <class Graph, class VertexIterator, class PosMap, class Time>
+template <class Graph, class VertexIterator, class PosMap, class Time,
+          class Yield>
 void draw_vertices(Graph&, pair<VertexIterator, VertexIterator> v_range,
                    PosMap pos_map, attrs_t& attrs, attrs_t& defaults,
-                   size_t offset, Time max_time, size_t& count,
-                   Cairo::Context& cr)
+                   Time max_time, int64_t dt, size_t& count, Cairo::Context& cr,
+                   Yield&& yield)
 {
     typedef typename graph_traits<Graph>::vertex_descriptor vertex_t;
-
-    if (offset > count)
-    {
-        size_t dist = std::distance(v_range.first, v_range.second);
-        size_t skip = std::min(offset - count, dist);
-        std::advance(v_range.first, skip);
-        count += skip;
-    }
-
     for(VertexIterator v = v_range.first; v != v_range.second; ++v)
     {
         pos_t pos;
@@ -1565,34 +1564,24 @@ void draw_vertices(Graph&, pair<VertexIterator, VertexIterator> v_range,
         vs.draw(cr);
         count++;
 
-        if (chrono::high_resolution_clock::now() > max_time)
-            break;
+        if (std::chrono::high_resolution_clock::now() > max_time)
+        {
+            yield(boost::python::object(count));
+            max_time = std::chrono::high_resolution_clock::now() +
+                std::chrono::milliseconds(dt);
+        }
     }
 }
 
-template <class Graph, class EdgeIterator, class PosMap, class Time>
+template <class Graph, class EdgeIterator, class PosMap, class Time,
+          class Yield>
 void draw_edges(Graph& g, pair<EdgeIterator, EdgeIterator> e_range,
                 PosMap pos_map, attrs_t& eattrs, attrs_t& edefaults,
-                attrs_t& vattrs, attrs_t& vdefaults, double res, size_t offset,
-                Time max_time, size_t& count, Cairo::Context& cr)
+                attrs_t& vattrs, attrs_t& vdefaults, double res, Time max_time,
+                int64_t dt, size_t& count, Cairo::Context& cr, Yield&& yield)
 {
     typedef typename graph_traits<Graph>::vertex_descriptor vertex_t;
     typedef typename graph_traits<Graph>::edge_descriptor edge_t;
-    if (offset > count)
-    {
-        size_t E = num_edges(g);
-        if (offset - count >= E)
-        {
-            count += E;
-            return;
-        }
-
-        size_t dist = std::distance(e_range.first, e_range.second);
-        size_t skip = std::min(offset - count, dist);
-        std::advance(e_range.first, skip);
-        count += skip;
-    }
-
     for(EdgeIterator e = e_range.first; e != e_range.second; ++e)
     {
         vertex_t s, t;
@@ -1624,10 +1613,13 @@ void draw_edges(Graph& g, pair<EdgeIterator, EdgeIterator> e_range,
                                                     AttrDict<edge_t>(*e, eattrs,
                                                                      edefaults));
         es.draw(cr, res);
-        count++;
 
-        if (chrono::high_resolution_clock::now() > max_time)
-            break;
+        if (std::chrono::high_resolution_clock::now() > max_time)
+        {
+            yield(boost::python::object(count));
+            max_time = std::chrono::high_resolution_clock::now() +
+                std::chrono::milliseconds(dt);
+        }
     }
 }
 
@@ -1676,37 +1668,6 @@ struct ordered_range
     vector<val_t> _ordered;
 };
 
-
-struct do_cairo_draw_edges
-{
-    template <class Graph, class PosMap, class EdgeOrder, class Time>
-    void operator()(Graph& g, PosMap pos, EdgeOrder edge_order, attrs_t& vattrs,
-                    attrs_t& eattrs, attrs_t& vdefaults, attrs_t& edefaults,
-                    double res, size_t offset, Time max_time, size_t& count,
-                    Cairo::Context& cr) const
-    {
-        ordered_range<typename graph_traits<Graph>::edge_iterator>
-            edge_range(edges(g));
-        draw_edges(g, edge_range.get_range(edge_order), pos, eattrs, edefaults,
-                   vattrs, vdefaults, res, offset, max_time, count, cr);
-    }
-};
-
-struct do_cairo_draw_vertices
-{
-    template <class Graph, class PosMap, class VertexOrder, class Time>
-    void operator()(Graph& g, PosMap pos, VertexOrder vertex_order,
-                    attrs_t& vattrs, attrs_t&, attrs_t& vdefaults, attrs_t&,
-                    size_t offset, Time max_time, size_t& count,
-                    Cairo::Context& cr) const
-    {
-        ordered_range<typename graph_traits<Graph>::vertex_iterator>
-            vertex_range(vertices(g));
-        draw_vertices(g, vertex_range.get_range(vertex_order), pos, vattrs,
-                      vdefaults, offset, max_time, count, cr);
-    }
-};
-
 template <class Descriptor, class PropMaps>
 struct get_pmap
 {
@@ -1789,90 +1750,122 @@ struct populate_edge_attrs
     }
 };
 
-
-size_t cairo_draw(GraphInterface& gi,
-                  boost::any pos,
-                  boost::any vorder,
-                  boost::any eorder,
-                  bool nodesfirst,
-                  boost::python::dict ovattrs,
-                  boost::python::dict oeattrs,
-                  boost::python::dict ovdefaults,
-                  boost::python::dict oedefaults,
-                  double res,
-                  size_t offset,
-                  int64_t max_time,
-                  boost::python::object ocr)
+struct do_cairo_draw_edges
 {
-    attrs_t vattrs, eattrs, vdefaults, edefaults;
-
-    typedef graph_traits<GraphInterface::multigraph_t>::vertex_descriptor vertex_t;
-
-    populate_attrs<vertex_t, vertex_properties>(ovattrs, vattrs);
-    populate_defaults(ovdefaults, vdefaults);
-
-    run_action<graph_tool::detail::always_directed>()
-        (gi, std::bind(populate_edge_attrs(), std::placeholders::_1, oeattrs,
-                       std::ref(eattrs), oedefaults, std::ref(edefaults)))();
-
-    typedef boost::mpl::push_back<vertex_scalar_properties, no_order>::type
-        vorder_t;
-    typedef boost::mpl::push_back<edge_scalar_properties, no_order>::type
-        eorder_t;
-    if (vorder.empty())
-        vorder = no_order();
-    if (eorder.empty())
-        eorder = no_order();
-
-
-    size_t count = 0;
-    auto mtime = chrono::high_resolution_clock::now();
-    if (max_time < 0)
-        mtime = chrono::high_resolution_clock::time_point::max();
-    else
-        mtime += chrono::milliseconds(max_time);
-
-    Cairo::Context cr(PycairoContext_GET(ocr.ptr()));
-    if (nodesfirst)
-        run_action<graph_tool::detail::always_directed>()
-            (gi, std::bind(do_cairo_draw_vertices(), std::placeholders::_1,
-                           std::placeholders::_2, std::placeholders::_3,
-                           std::ref(vattrs), std::ref(eattrs), std::ref(vdefaults),
-                           std::ref(edefaults), offset, mtime, std::ref(count),
-                           std::ref(cr)),
-             vertex_scalar_vector_properties(),
-             vorder_t())(pos, vorder);
-
-    if (chrono::high_resolution_clock::now() > mtime)
-        return count;
-
-    run_action<graph_tool::detail::always_directed>()
-        (gi, std::bind(do_cairo_draw_edges(), std::placeholders::_1, std::placeholders::_2,
-                       std::placeholders::_3, std::ref(vattrs), std::ref(eattrs),
-                       std::ref(vdefaults), std::ref(edefaults), res, offset,
-                       mtime, std::ref(count), std::ref(cr)),
-         vertex_scalar_vector_properties(),
-         eorder_t())(pos, eorder);
-
-    if (!nodesfirst)
+    template <class Graph, class PosMap, class EdgeOrder, class Time,
+              class Yield>
+    void operator()(Graph& g, PosMap pos, EdgeOrder edge_order, attrs_t& vattrs,
+                    attrs_t& eattrs, attrs_t& vdefaults, attrs_t& edefaults,
+                    double res, Time max_time, int64_t dt, size_t& count,
+                    Cairo::Context& cr, Yield&& yield) const
     {
-        if (chrono::high_resolution_clock::now() > mtime)
-            return count;
-
-        run_action<graph_tool::detail::always_directed>()
-            (gi, std::bind(do_cairo_draw_vertices(), std::placeholders::_1,
-                           std::placeholders::_2, std::placeholders::_3,
-                           std::ref(vattrs), std::ref(eattrs), std::ref(vdefaults),
-                           std::ref(edefaults), offset, mtime, std::ref(count),
-                           std::ref(cr)),
-             vertex_scalar_vector_properties(),
-             vorder_t())(pos, vorder);
+        ordered_range<typename graph_traits<Graph>::edge_iterator>
+            edge_range(edges(g));
+        draw_edges(g, edge_range.get_range(edge_order), pos, eattrs,
+                   edefaults, vattrs, vdefaults, res, max_time, dt, count, cr,
+                   std::forward<Yield>(yield));
     }
+};
 
-    if (chrono::high_resolution_clock::now() < mtime)
-        count = 0;
+struct do_cairo_draw_vertices
+{
+    template <class Graph, class PosMap, class VertexOrder, class Time,
+              class Yield>
+    void operator()(Graph& g, PosMap pos, VertexOrder vertex_order,
+                    attrs_t& vattrs, attrs_t&, attrs_t& vdefaults, attrs_t&,
+                    Time max_time, int64_t dt, size_t& count,
+                    Cairo::Context& cr, Yield&& yield) const
+    {
+        ordered_range<typename graph_traits<Graph>::vertex_iterator>
+            vertex_range(vertices(g));
+        draw_vertices(g, vertex_range.get_range(vertex_order), pos, vattrs,
+                      vdefaults, max_time, dt, count, cr,
+                      std::forward<Yield>(yield));
+    }
+};
 
-    return count;
+boost::python::object cairo_draw(GraphInterface& gi,
+                                 boost::any pos,
+                                 boost::any vorder,
+                                 boost::any eorder,
+                                 bool nodesfirst,
+                                 boost::python::dict ovattrs,
+                                 boost::python::dict oeattrs,
+                                 boost::python::dict ovdefaults,
+                                 boost::python::dict oedefaults,
+                                 double res,
+                                 int64_t max_time,
+                                 boost::python::object ocr)
+{
+    auto dispatch = [=,&gi] (auto&& yield) mutable
+        {
+            attrs_t vattrs, eattrs, vdefaults, edefaults;
+
+            typedef graph_traits<GraphInterface::multigraph_t>::vertex_descriptor vertex_t;
+
+            populate_attrs<vertex_t, vertex_properties>(ovattrs, vattrs);
+            populate_defaults(ovdefaults, vdefaults);
+
+            run_action<>()
+                (gi, std::bind(populate_edge_attrs(), std::placeholders::_1,
+                               oeattrs, std::ref(eattrs), oedefaults,
+                               std::ref(edefaults)))();
+
+            typedef boost::mpl::push_back<vertex_scalar_properties, no_order>::type
+               vorder_t;
+            typedef boost::mpl::push_back<edge_scalar_properties, no_order>::type
+               eorder_t;
+            if (vorder.empty())
+                vorder = no_order();
+            if (eorder.empty())
+                eorder = no_order();
+
+            size_t count = 0;
+            auto mtime = std::chrono::high_resolution_clock::now();
+            if (max_time < 0)
+                mtime = std::chrono::high_resolution_clock::time_point::max();
+            else
+                mtime += std::chrono::milliseconds(max_time);
+            int64_t dt = max_time;
+
+            Cairo::Context cr(PycairoContext_GET(ocr.ptr()));
+
+            if (nodesfirst)
+                run_action<>()
+                    (gi, std::bind(do_cairo_draw_vertices(), std::placeholders::_1,
+                                   std::placeholders::_2, std::placeholders::_3,
+                                   std::ref(vattrs), std::ref(eattrs), std::ref(vdefaults),
+                                   std::ref(edefaults), mtime, dt, std::ref(count),
+                                   std::ref(cr), std::ref(yield)),
+                     vertex_scalar_vector_properties(),
+                     vorder_t())(pos, vorder);
+
+            run_action<>()
+                (gi, std::bind(do_cairo_draw_edges(), std::placeholders::_1,
+                               std::placeholders::_2, std::placeholders::_3,
+                               std::ref(vattrs), std::ref(eattrs),
+                               std::ref(vdefaults), std::ref(edefaults), res,
+                               mtime, dt, std::ref(count), std::ref(cr), std::ref(yield)),
+                 vertex_scalar_vector_properties(),
+                 eorder_t())(pos, eorder);
+
+            if (!nodesfirst)
+            {
+                run_action<>()
+                    (gi, std::bind(do_cairo_draw_vertices(), std::placeholders::_1,
+                                   std::placeholders::_2, std::placeholders::_3,
+                                   std::ref(vattrs), std::ref(eattrs), std::ref(vdefaults),
+                                   std::ref(edefaults), mtime, dt, std::ref(count),
+                                   std::ref(cr), std::ref(yield)),
+                     vertex_scalar_vector_properties(),
+                     vorder_t())(pos, vorder);
+            }
+        };
+
+    // dispatch([](auto){});
+    // auto fake_dispatch = [](auto&& yield){};
+    // return boost::python::object(CoroGenerator(fake_dispatch));
+    return boost::python::object(CoroGenerator(dispatch));
 }
 
 struct do_apply_transforms
@@ -1896,7 +1889,7 @@ void apply_transforms(GraphInterface& gi, boost::any pos, double xx, double yx,
                       double xy, double yy, double x0, double y0)
 {
     Cairo::Matrix m(xx, yx, xy, yy, x0, y0);
-    run_action<graph_tool::detail::always_directed>()
+    run_action<>()
         (gi, std::bind(do_apply_transforms(), std::placeholders::_1,
                        std::placeholders::_2, std::ref(m)),
          vertex_scalar_vector_properties())(pos);
@@ -2043,7 +2036,7 @@ void put_parallel_splines(GraphInterface& gi, boost::any opos,
     DynamicPropertyMapWrap<double, GraphInterface::vertex_t>
         angle(loop_angle, vertex_scalar_properties());
 
-    run_action<graph_tool::detail::always_directed>()
+    run_action<>()
         (gi, std::bind(do_put_parallel_splines(), std::placeholders::_1, pos, l,
                        std::placeholders::_2, angle, parallel_distance),
          edge_scalar_vector_properties())(splines);
@@ -2173,6 +2166,7 @@ void get_cts(GraphInterface& gi, GraphInterface& tgi, boost::any otpos,
 
 BOOST_PYTHON_MODULE(libgraph_tool_draw)
 {
+    docstring_options dopt(true, false);
     def("cairo_draw", &cairo_draw);
     def("put_parallel_splines", &put_parallel_splines);
     def("apply_transforms", &apply_transforms);
@@ -2239,7 +2233,8 @@ BOOST_PYTHON_MODULE(libgraph_tool_draw)
         .value("double_hexagon", SHAPE_DOUBLE_HEXAGON)
         .value("double_heptagon", SHAPE_DOUBLE_HEPTAGON)
         .value("double_octagon", SHAPE_DOUBLE_OCTAGON)
-        .value("pie", SHAPE_PIE);
+        .value("pie", SHAPE_PIE)
+        .value("none", SHAPE_NONE);
 
     enum_<edge_marker_t>("edge_marker")
         .value("none", MARKER_SHAPE_NONE)
