@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -174,7 +174,7 @@ void graph_path(Graph& g, size_t s, size_t t, vector<size_t>& path)
         cpred;
     auto pred = cpred.get_unchecked(num_vertices(g));
 
-    UndirectedAdaptor<Graph> ug(g);
+    undirected_adaptor<Graph> ug(g);
 
     boost::breadth_first_search(ug, s,
                                 boost::visitor(
