@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -50,16 +50,14 @@ struct get_weighted_edge_property_dispatch
 
 void sum_eprops(GraphInterface& gi, GraphInterface& cgi,
                 boost::any community_property,
-                boost::any condensed_community_property,
-                boost::any ceprop, boost::any eprop,
-                bool self_loops);
+                boost::any condensed_community_property, boost::any ceprop,
+                boost::any eprop, bool self_loops, bool parallel_edges);
 
 void community_network_eavg(GraphInterface& gi, GraphInterface& cgi,
                             boost::any community_property,
                             boost::any condensed_community_property,
-                            boost::any eweight,
-                            boost::python::list aeprops,
-                            bool self_loops)
+                            boost::any eweight, boost::python::list aeprops,
+                            bool self_loops, bool parallel_edges)
 {
     typedef boost::mpl::push_back<writable_edge_scalar_properties, no_eweight_map_t>::type
         eweight_properties;
@@ -87,7 +85,7 @@ void community_network_eavg(GraphInterface& gi, GraphInterface& cgi,
         if (!no_weight)
         {
             // compute weighted values to temp
-            run_action<graph_tool::detail::always_directed>()
+            run_action<>()
                 (gi, std::bind(get_weighted_edge_property_dispatch(),
                                std::placeholders::_1, std::placeholders::_2,
                                std::placeholders::_3, temp),
@@ -96,15 +94,15 @@ void community_network_eavg(GraphInterface& gi, GraphInterface& cgi,
 
             // sum weighted values
             sum_eprops(gi, cgi, community_property,
-                       condensed_community_property,
-                       ceprop, temp, self_loops);
+                       condensed_community_property, ceprop, temp,
+                       self_loops, parallel_edges);
         }
         else
         {
             // sum unweighted values
             sum_eprops(gi, cgi, community_property,
-                       condensed_community_property,
-                       ceprop, eprop, self_loops);
+                       condensed_community_property, ceprop, eprop,
+                       self_loops, parallel_edges);
         }
 
     }
