@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -96,7 +96,7 @@ void community_network_vavg(GraphInterface& gi, GraphInterface& cgi,
         if (!no_weight)
         {
             // compute weighted values to temp
-            run_action<graph_tool::detail::always_directed_never_reversed>()
+            run_action<>()
                 (gi, std::bind(get_weighted_vertex_property_dispatch(),
                                std::placeholders::_1, std::placeholders::_2,
                                std::placeholders::_3, temp),
@@ -104,7 +104,7 @@ void community_network_vavg(GraphInterface& gi, GraphInterface& cgi,
                 (vweight, vprop);
 
             // sum weighted values
-            run_action<graph_tool::detail::always_directed_never_reversed>()
+            run_action<>()
                 (gi, std::bind(get_vertex_sum_dispatch(),
                                std::placeholders::_1, std::ref(cgi.get_graph()),
                                std::placeholders::_2,
@@ -116,7 +116,7 @@ void community_network_vavg(GraphInterface& gi, GraphInterface& cgi,
         else
         {
             // sum unweighted values
-            run_action<graph_tool::detail::always_directed_never_reversed>()
+            run_action<>()
                 (gi, std::bind(get_vertex_sum_dispatch(),
                                std::placeholders::_1, std::ref(cgi.get_graph()),
                                std::placeholders::_2,
