@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -15,6 +15,8 @@
 // You should have received a copy of the GNU General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 
+#define BOOST_PYTHON_MAX_ARITY 40
+
 #include "graph.hh"
 #include "graph_util.hh"
 #include "graph_filtering.hh"
@@ -68,11 +70,17 @@ void generate_graph(GraphInterface& gi, size_t N,
                        std::ref(rng), verbose, verify))();
 }
 
+void generate_sbm(GraphInterface& gi, boost::any ab, boost::python::object ors,
+                  boost::python::object oss, boost::python::object oprobs,
+                  boost::any ain_deg, boost::any aout_deg, bool micro_ers,
+                  bool micro_degs, rng_t& rng);
+
 size_t random_rewire(GraphInterface& gi, string strat, size_t niter,
                      bool no_sweep, bool self_loops, bool parallel_edges,
-                     bool alias, bool traditional, bool persist,
-                     boost::python::object corr_prob, boost::any apin,
-                     boost::any block, bool cache, rng_t& rng, bool verbose);
+                     bool configuration, bool traditional, bool micro,
+                     bool persist, boost::python::object corr_prob,
+                     boost::any apin, boost::any block, bool cache, rng_t& rng,
+                     bool verbose);
 void predecessor_graph(GraphInterface& gi, GraphInterface& gpi,
                        boost::any pred_map);
 void line_graph(GraphInterface& gi, GraphInterface& lgi,
@@ -112,13 +120,15 @@ void community_network_eavg(GraphInterface& gi, GraphInterface& cgi,
                             boost::any community_property,
                             boost::any condensed_community_property,
                             boost::any eweight, boost::python::list aeprops,
-                            bool self_loops);
+                            bool self_loops, bool parallel_edges);
 
 using namespace boost::python;
 
 BOOST_PYTHON_MODULE(libgraph_tool_generation)
 {
+    docstring_options dopt(true, false);
     def("gen_graph", &generate_graph);
+    def("gen_sbm", &generate_sbm);
     def("random_rewire", &random_rewire);
     def("predecessor_graph", &predecessor_graph);
     def("line_graph", &line_graph);
@@ -147,6 +157,6 @@ BOOST_PYTHON_MODULE(libgraph_tool_generation)
              return_value_policy<copy_const_reference>())
         .def("insert", &DynamicSampler<int>::insert)
         .def("remove", &DynamicSampler<int>::remove)
-        .def("reset", &DynamicSampler<int>::reset)
+        .def("clear", &DynamicSampler<int>::clear)
         .def("rebuild", &DynamicSampler<int>::rebuild);
 }
