@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -47,7 +47,7 @@ struct get_line_graph
             vertex_map[v] = edge_index[e];
         }
 
-        if (boost::is_directed(g))
+        if (graph_tool::is_directed(g))
         {
             for (auto v : vertices_range(g))
             {
