@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -75,34 +75,38 @@ private:
 
 struct graph_rewire_block
 {
-    graph_rewire_block(bool alias, bool traditional) : alias(alias), traditional(traditional) {}
-    bool alias;
+    graph_rewire_block(bool traditional, bool micro) :
+        traditional(traditional), micro(micro) {}
     bool traditional;
+    bool micro;
 
     template <class Graph, class EdgeIndexMap, class CorrProb, class PinMap,
               class BlockProp>
     void operator()(Graph& g, EdgeIndexMap edge_index, CorrProb corr_prob,
-                    PinMap pin, pair<bool, bool> rest, BlockProp block_prop,
-                    pair<size_t, bool> iter_sweep,
-                    std::tuple<bool, bool, bool> cache_verbose, size_t& pcount, rng_t& rng)
-        const
+                    PinMap pin, pair<bool, bool> rest, bool configuration,
+                    BlockProp block_prop, pair<size_t, bool> iter_sweep,
+                    std::tuple<bool, bool, bool> cache_verbose, size_t& pcount,
+                    rng_t& rng) const
     {
         if (traditional)
         {
-            graph_rewire<TradBlockRewireStrategy>()
-                (g, edge_index, corr_prob, pin, rest.first, rest.second, iter_sweep,
-                 cache_verbose, pcount, rng, PropertyBlock<BlockProp>(block_prop));
+            if (micro)
+                graph_rewire<MicroTradBlockRewireStrategy>()
+                    (g, edge_index, corr_prob, pin, rest.first, rest.second,
+                     configuration, iter_sweep, cache_verbose, pcount, rng,
+                     PropertyBlock<BlockProp>(block_prop));
+            else
+                graph_rewire<CanTradBlockRewireStrategy>()
+                    (g, edge_index, corr_prob, pin, rest.first, rest.second,
+                     configuration, iter_sweep, cache_verbose, pcount, rng,
+                     PropertyBlock<BlockProp>(block_prop));
         }
         else
         {
-            if (alias)
-                graph_rewire<AliasProbabilisticRewireStrategy>()
-                    (g, edge_index, corr_prob, pin, rest.first, rest.second, iter_sweep,
-                     cache_verbose, pcount, rng, PropertyBlock<BlockProp>(block_prop));
-            else
-                graph_rewire<ProbabilisticRewireStrategy>()
-                    (g, edge_index, corr_prob, pin,  rest.first, rest.second, iter_sweep,
-                     cache_verbose, pcount, rng, PropertyBlock<BlockProp>(block_prop));
+            graph_rewire<ProbabilisticRewireStrategy>()
+                (g, edge_index, corr_prob, pin, rest.first, rest.second,
+                 configuration, iter_sweep, cache_verbose, pcount, rng,
+                 PropertyBlock<BlockProp>(block_prop));
         }
     }
 };
@@ -114,22 +118,24 @@ struct graph_rewire_correlated
               class BlockProp>
     void operator()(Graph& g, EdgeIndexMap edge_index, CorrProb corr_prob,
                     PinMap pin, bool self_loops, bool parallel_edges,
-                    pair<size_t, bool> iter_sweep,
-                    std::tuple<bool, bool, bool> cache_verbose,
-                    size_t& pcount, rng_t& rng, BlockProp block_prop) const
+                    bool configuration, pair<size_t, bool> iter_sweep,
+                    std::tuple<bool, bool, bool> cache_verbose, size_t& pcount,
+                    rng_t& rng, BlockProp block_prop) const
     {
         graph_rewire<CorrelatedRewireStrategy>()
-            (g, edge_index, corr_prob, pin, self_loops, parallel_edges, iter_sweep,
-             cache_verbose, pcount, rng, PropertyBlock<BlockProp>(block_prop));
+            (g, edge_index, corr_prob, pin, self_loops, parallel_edges,
+             configuration, iter_sweep, cache_verbose, pcount, rng,
+             PropertyBlock<BlockProp>(block_prop));
     }
 };
 
 
 size_t random_rewire(GraphInterface& gi, string strat, size_t niter,
                      bool no_sweep, bool self_loops, bool parallel_edges,
-                     bool alias, bool traditional, bool persist,
-                     boost::python::object corr_prob, boost::any apin,
-                     boost::any block, bool cache, rng_t& rng, bool verbose)
+                     bool configuration, bool traditional, bool micro,
+                     bool persist, boost::python::object corr_prob,
+                     boost::any apin, boost::any block, bool cache, rng_t& rng,
+                     bool verbose)
 {
     PythonFuncWrap corr(corr_prob);
     size_t pcount = 0;
@@ -139,36 +145,35 @@ size_t random_rewire(GraphInterface& gi, string strat, size_t niter,
     emap_t::unchecked_t pin =
         any_cast<emap_t>(apin).get_unchecked(gi.get_edge_index_range());
 
-
     if (strat == "erdos")
     {
         run_action<graph_tool::detail::never_reversed>()
             (gi, std::bind(graph_rewire<ErdosRewireStrategy>(),
                            std::placeholders::_1, gi.get_edge_index(),
                            std::ref(corr), pin,
-                           self_loops, parallel_edges,
+                           self_loops, parallel_edges, configuration,
                            make_pair(niter, no_sweep),
                            std::make_tuple(persist, cache, verbose),
                            std::ref(pcount), std::ref(rng)))();
     }
-    else if (strat == "uncorrelated")
+    else if (strat == "configuration")
     {
         run_action<graph_tool::detail::never_reversed>()
             (gi, std::bind(graph_rewire<RandomRewireStrategy>(),
                            std::placeholders::_1, gi.get_edge_index(), std::ref(corr),
-                           pin, self_loops, parallel_edges,
+                           pin, self_loops, parallel_edges, configuration,
                            make_pair(niter, no_sweep),
                            std::make_tuple(persist, cache, verbose),
                            std::ref(pcount), std::ref(rng)))();
     }
-    else if (strat == "correlated")
+    else if (strat == "constrained-configuration")
     {
         if (block.empty())
         {
             run_action<graph_tool::detail::never_reversed>()
                 (gi, std::bind(graph_rewire<CorrelatedRewireStrategy>(),
                                std::placeholders::_1, gi.get_edge_index(), std::ref(corr),
-                               pin, self_loops, parallel_edges,
+                               pin, self_loops, parallel_edges, configuration,
                                make_pair(niter, no_sweep),
                                std::make_tuple(persist, cache, verbose),
                                std::ref(pcount), std::ref(rng)))();
@@ -178,7 +183,7 @@ size_t random_rewire(GraphInterface& gi, string strat, size_t niter,
             run_action<graph_tool::detail::never_reversed>()
                 (gi, std::bind(graph_rewire_correlated(),
                                std::placeholders::_1, gi.get_edge_index(), std::ref(corr),
-                               pin, self_loops, parallel_edges,
+                               pin, self_loops, parallel_edges, configuration,
                                make_pair(niter, no_sweep),
                                std::make_tuple(persist, cache, verbose),
                                std::ref(pcount), std::ref(rng),
@@ -186,12 +191,12 @@ size_t random_rewire(GraphInterface& gi, string strat, size_t niter,
                  vertex_properties())(block);
         }
     }
-    else if (strat == "probabilistic")
+    else if (strat == "probabilistic-configuration")
     {
         run_action<>()
             (gi, std::bind(graph_rewire<ProbabilisticRewireStrategy>(),
                            std::placeholders::_1, gi.get_edge_index(), std::ref(corr),
-                           pin, self_loops, parallel_edges,
+                           pin, self_loops, parallel_edges, configuration,
                            make_pair(niter, no_sweep),
                            std::make_tuple(persist, cache, verbose),
                            std::ref(pcount), std::ref(rng)))();
@@ -199,10 +204,11 @@ size_t random_rewire(GraphInterface& gi, string strat, size_t niter,
     else if (strat == "blockmodel")
     {
         run_action<>()
-            (gi, std::bind(graph_rewire_block(alias, traditional),
+            (gi, std::bind(graph_rewire_block(traditional, micro),
                            std::placeholders::_1, gi.get_edge_index(),
                            std::ref(corr), pin,
                            make_pair(self_loops, parallel_edges),
+                           configuration,
                            std::placeholders::_2,
                            make_pair(niter, no_sweep),
                            std::make_tuple(persist, cache, verbose),
