@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -117,6 +117,21 @@ bool vector_nequal_compare(const vector<ValueType>& v1,
     return !vector_equal_compare(v1,v2);
 }
 
+template <class T>
+python::object get_vector_state(std::vector<T>& v)
+{
+    return wrap_vector_owned(v);
+}
+
+template <class T>
+void set_vector_state(std::vector<T>& v, python::object state)
+{
+    auto a = get_array<T,1>(state);
+    v.clear();
+    v.reserve(a.size());
+    v.insert(v.end(), a.begin(), a.end());
+}
+
 struct export_vector_types
 {
     template <class ValueType>
@@ -138,12 +153,15 @@ struct export_vector_types
             [] (vector<ValueType>& v) { v.shrink_to_fit(); };
         std::function<bool(vector<ValueType>&)> empty =
             [] (vector<ValueType>& v) -> bool { return v.empty(); };
+        std::function<void(vector<ValueType>&)> clear =
+            [] (vector<ValueType>& v) { v.clear(); };
         vc.def(vector_indexing_suite<vector<ValueType> >())
             .def("__eq__", &vector_equal_compare<ValueType>)
             .def("__ne__", &vector_nequal_compare<ValueType>)
             .def("__hash__", hasher)
             .def("resize", resize)
             .def("shrink_to_fit", shrink_to_fit)
+            .def("clear", clear)
             .def("empty", empty);
         wrap_array(vc, typename boost::mpl::has_key<numpy_types,ValueType>::type());
         vector_from_list<ValueType>();
@@ -152,7 +170,10 @@ struct export_vector_types
     template <class ValueType>
     void wrap_array(class_<vector<ValueType> >& vc, boost::mpl::true_) const
     {
-        vc.def("get_array", &wrap_vector_not_owned<ValueType>);
+        vc.def("get_array", &wrap_vector_not_owned<ValueType>)
+            .def("__getstate__", &get_vector_state<ValueType>)
+            .def("__setstate__", &set_vector_state<ValueType>)
+            .enable_pickling();
     }
 
     template <class ValueType>
@@ -365,8 +386,7 @@ struct graph_type_name
     template <class Graph>
     void operator()(const Graph&, string& name) const
     {
-        using boost::python::detail::gcc_demangle;
-        name = string(gcc_demangle(typeid(Graph).name()));
+        name = name_demangle(typeid(Graph).name());
     }
 };
 
@@ -422,7 +442,7 @@ void export_openmp();
 
 BOOST_PYTHON_MODULE(libgraph_tool_core)
 {
-    using namespace boost::python;
+    python::docstring_options dopt(true, false);
 
     // numpy
     do_import_array();
@@ -448,13 +468,14 @@ BOOST_PYTHON_MODULE(libgraph_tool_core)
     class_<std::type_info, boost::noncopyable>("type_info", no_init)
         .def("name", &std::type_info::name)
         .def("hash_code", &std::type_info::hash_code);
-    def("gcc_demangle", &python::detail::gcc_demangle);
+    def("name_demangle", &name_demangle);
 
     def("graph_filtering_enabled", &graph_filtering_enabled);
     export_openmp();
 
     boost::mpl::for_each<boost::mpl::push_back<scalar_types,string>::type>(export_vector_types());
     export_vector_types()(size_t(), "size_t");
+    export_vector_types()(std::vector<double>(), "Vector_double");
 
     class_<GraphInterface>("GraphInterface", init<>())
         .def(init<GraphInterface,bool,boost::python::object,
@@ -525,14 +546,6 @@ BOOST_PYTHON_MODULE(libgraph_tool_core)
         ("shared_ptr<multigraph_t>", no_init)
         .def("get", &get_ptr);
 
-#ifdef HAVE_SCIPY
-    to_python_converter<py::object, scxx_to_python<py::object> >();
-    to_python_converter<py::tuple, scxx_to_python<py::tuple> >();
-    to_python_converter<py::list, scxx_to_python<py::list> >();
-    to_python_converter<py::dict, scxx_to_python<py::dict> >();
-    to_python_converter<py::str, scxx_to_python<py::str> >();
-#endif
-
     class_<IStream>("IStream", no_init).def("read", &IStream::read);
     class_<OStream>("OStream", no_init).def("write", &OStream::write).
         def("flush", &OStream::flush);
@@ -563,4 +576,7 @@ BOOST_PYTHON_MODULE(libgraph_tool_core)
         .add_property("gcc_version", &LibInfo::GetGCCVersion);
 
     def("get_graph_type", &get_graph_type);
+
+    def("get_null_vertex",
+        +[](){ return graph_traits<GraphInterface::multigraph_t>::null_vertex();});
 }
