@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -69,17 +69,13 @@ struct copy_vertex_property_dispatch
         auto p_src = psrc->get_unchecked(num_vertices(src));
         auto p_tgt = ptgt->get_unchecked(num_vertices(tgt));
 
-        int i, N = num_vertices(src);
-        #pragma omp parallel for default(shared) private(i) \
-            schedule(runtime) if (N > 100)
-        for (i = 0; i < N; ++i)
-        {
-            auto v = vertex(i, src);
-            if (!is_valid_vertex(v, src))
-                continue;
-            auto new_v = vertex(index_map[i], tgt);
-            p_tgt[new_v] = p_src[v];
-        }
+        parallel_vertex_loop
+            (src,
+             [&](auto v)
+             {
+                 auto new_v = vertex(index_map[v], tgt);
+                 p_tgt[new_v] = p_src[v];
+             });
     }
 };
 
@@ -149,7 +145,7 @@ struct copy_edge_property_dispatch
                  {
                      auto s = source(e, src);
                      auto t = target(e, src);
-                     if (!is_directed::apply<GraphSrc>::type::value && s > t)
+                     if (!is_directed_::apply<GraphSrc>::type::value && s > t)
                          continue;
                      size_t ei = src_edge_index[e];
                      const auto& new_e = index_map[ei];
