@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -16,33 +16,16 @@
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 #include "graph_filtering.hh"
-#include <cxxabi.h>
+#include "demangle.hh"
 
 using namespace graph_tool;
 using namespace graph_tool::detail;
 using namespace boost;
+using namespace std;
 
 bool graph_tool::graph_filtering_enabled()
 {
-#ifndef NO_GRAPH_FILTERING
     return true;
-#else
-    return false;
-#endif
-}
-
-namespace graph_tool
-{
-string name_demangle(string name)
-{
-    int status = 0;
-    char *realname = abi::__cxa_demangle(name.c_str(), 0, 0, &status);
-    if (realname == nullptr)
-        return name + " (cannot demangle symbol)";
-    string ret(realname);
-    free(realname);
-    return ret;
-}
 }
 
 // Whenever no implementation is called, the following exception is thrown
@@ -50,8 +33,6 @@ graph_tool::ActionNotFound::ActionNotFound(const type_info& action,
                                            const vector<const type_info*>& args)
     : GraphException(""), _action(action), _args(args)
 {
-    using python::detail::gcc_demangle;
-
     _error =
         "No static implementation was found for the desired routine. "
         "This is a graph_tool bug. :-( Please submit a bug report at "
@@ -65,40 +46,6 @@ graph_tool::ActionNotFound::ActionNotFound(const type_info& action,
     }
 }
 
-// this will check whether a graph is reversed and return the proper view
-// encapsulated
-template <class Graph>
-boost::any check_reverse(const Graph& g, bool reverse, GraphInterface& gi)
-{
-    if (reverse)
-    {
-        typedef typename boost::mpl::if_<std::is_const<Graph>,
-                                         const reverse_graph<typename std::remove_const<Graph>::type>,
-                                         reverse_graph<Graph> >::type
-            reverse_graph_t;
-
-        reverse_graph_t rg(g);
-        return std::ref(*retrieve_graph_view(gi, rg));
-    }
-
-    return boost::any(std::ref(const_cast<Graph&>(g)));
-};
-
-// this will check whether a graph is directed and return the proper view
-// encapsulated
-template <class Graph>
-boost::any check_directed(const Graph &g, bool reverse, bool directed,
-                          GraphInterface& gi)
-{
-    if (directed)
-    {
-        return check_reverse(g, reverse, gi);
-    }
-
-    typedef UndirectedAdaptor<Graph> ug_t;
-    ug_t ug(g);
-    return std::ref(*retrieve_graph_view(gi, ug));
-};
 
 // this will check whether a graph is filtered and return the proper view
 // encapsulated
@@ -109,37 +56,67 @@ check_filtered(const Graph& g, const EdgeFilter& edge_filter,
                const VertexFilter& vertex_filter, const bool& v_invert,
                bool v_active, GraphInterface& gi, bool reverse, bool directed)
 {
-#ifndef NO_GRAPH_FILTERING
-    if (e_active || v_active)
-    {
-        MaskFilter<EdgeFilter>
-            e_filter(const_cast<EdgeFilter&>(edge_filter),
-                     const_cast<bool&>(e_invert));
-        MaskFilter<VertexFilter>
-            v_filter(const_cast<VertexFilter&>(vertex_filter),
-                     const_cast<bool&>(v_invert));
 
-        if (max_eindex > 0)
-            edge_filter.reserve(max_eindex);
-        if (num_vertices(g) > 0)
-            vertex_filter.reserve(num_vertices(g));
+    auto check_filt = [&](auto&& u) -> boost::any
+        {
+            typedef typename std::remove_const<
+                typename std::remove_reference<decltype(u)>::type>::type g_t;
+            if (e_active || v_active)
+            {
+                MaskFilter<EdgeFilter>
+                    e_filter(const_cast<EdgeFilter&>(edge_filter),
+                             const_cast<bool&>(e_invert));
+                MaskFilter<VertexFilter>
+                    v_filter(const_cast<VertexFilter&>(vertex_filter),
+                             const_cast<bool&>(v_invert));
+                if (max_eindex > 0)
+                    edge_filter.reserve(max_eindex);
+                if (num_vertices(g) > 0)
+                    vertex_filter.reserve(num_vertices(g));
 
-        typedef filtered_graph<Graph, MaskFilter<EdgeFilter>,
-                               MaskFilter<VertexFilter> > fg_t;
+                typedef filt_graph<g_t,
+                                   MaskFilter<EdgeFilter>,
+                                   MaskFilter<VertexFilter>> fg_t;
 
-        fg_t init(g, e_filter, v_filter);
+                fg_t init(u, e_filter, v_filter);
+                fg_t& fg = *retrieve_graph_view(gi, init);
+                return std::ref(fg);
+            }
+            else
+            {
+                return std::ref(const_cast<g_t&>(u));
+            }
+        };
 
-        fg_t& fg = *retrieve_graph_view(gi, init);
+    auto check_reverse = [&](auto&& u) -> boost::any
+        {
+            typedef typename std::remove_const<
+                typename std::remove_reference<decltype(u)>::type>::type g_t;
+            if (reverse)
+            {
+                typedef typename
+                    std::conditional<std::is_const<g_t>::value,
+                                     const reversed_graph<typename std::remove_const<g_t>::type>,
+                                     reversed_graph<g_t> >::type
+                    reversed_graph_t;
+                reversed_graph_t rg(u);
+                return check_filt(*retrieve_graph_view(gi, rg));
+            }
+            return check_filt(u);
+        };
 
-        return check_directed(fg, reverse, directed, gi);
-    }
-    else
-    {
-        return check_directed(g, reverse, directed, gi);
-    }
-#else
-    return check_directed(g, reverse, directed, gi);
-#endif
+    auto check_directed = [&](auto&& u) -> boost::any
+        {
+            typedef typename std::remove_const<
+                typename std::remove_reference<decltype(u)>::type>::type g_t;
+            if (directed)
+                return check_reverse(u);
+            typedef undirected_adaptor<g_t> ug_t;
+            ug_t ug(u);
+            return check_filt(*retrieve_graph_view(gi, ug));
+        };
+
+    return check_directed(g);
 }
 
 // gets the correct graph view at run time
@@ -198,7 +175,7 @@ void GraphInterface::purge_vertices(boost::any aold_index)
     if (!is_vertex_filter_active())
         return;
 
-    typedef vprop_map_t<int32_t>::type index_prop_t;
+    typedef vprop_map_t<int64_t>::type index_prop_t;
     index_prop_t old_index = any_cast<index_prop_t>(aold_index);
 
     MaskFilter<vertex_filter_t> filter(_vertex_filter_map,
@@ -209,36 +186,22 @@ void GraphInterface::purge_vertices(boost::any aold_index)
         deleted[i] = !filter(vertex(i, *_mg));
     vector<int> old_indexes;
 
-    vector<graph_traits<multigraph_t>::edge_descriptor> edges;
-
     //remove vertices
-    for (int i = N-1; i >= 0; --i)
+    for (int64_t i = N-1; i >= 0; --i)
     {
         if (deleted[i])
-        {
-            graph_traits<multigraph_t>::vertex_descriptor v =
-                vertex(i, *_mg);
-            remove_vertex(v, *_mg);
-        }
+            remove_vertex(vertex(i, *_mg), *_mg);
         else
-        {
             old_indexes.push_back(i);
-        }
     }
 
     N = old_indexes.size();
-    for (int i = N-1; i >= 0; --i)
-    {
+    for (int64_t i = N-1; i >= 0; --i)
         old_index[vertex((N - 1) - i, *_mg)] = old_indexes[i];
-    }
 }
 
 void GraphInterface::set_vertex_filter_property(boost::any property, bool invert)
 {
-#ifdef NO_GRAPH_FILTERING
-    throw GraphException("graph filtering was not enabled at compile time");
-#endif
-
     try
     {
         _vertex_filter_map =
@@ -256,10 +219,6 @@ void GraphInterface::set_vertex_filter_property(boost::any property, bool invert
 
 void GraphInterface::set_edge_filter_property(boost::any property, bool invert)
 {
-#ifdef NO_GRAPH_FILTERING
-    throw GraphException("graph filtering was not enabled at compile time");
-#endif
-
     try
     {
         _edge_filter_map =
