@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -218,29 +218,12 @@ python::object add_edge(GraphInterface& gi, size_t s, size_t t)
     return new_e;
 }
 
-struct get_edge_descriptor
+void remove_edge(GraphInterface& gi, EdgeBase& e)
 {
-    template <class Graph>
-    void operator()(const Graph&, const python::object& e,
-                    typename GraphInterface::edge_t& edge,
-                    bool& found)  const
-    {
-        PythonEdge<Graph>& pe = python::extract<PythonEdge<Graph>&>(e);
-        pe.check_valid();
-        edge = pe.get_descriptor();
-        found = true;
-    }
-};
-
-void remove_edge(GraphInterface& gi, const python::object& e)
-{
-    GraphInterface::edge_t de;
-    bool found = false;
-    run_action<>()(gi, std::bind(get_edge_descriptor(), std::placeholders::_1,
-                                 std::ref(e), std::ref(de), std::ref(found)))();
-    remove_edge(de, gi.get_graph());
-    if (!found)
-        throw ValueException("invalid edge descriptor");
+    e.check_valid();
+    auto edge = e.get_descriptor();
+    run_action<>()(gi, [&](auto& g) { remove_edge(edge, g); })();
+    e.invalidate();
 }
 
 struct get_edge_dispatch
@@ -250,7 +233,7 @@ struct get_edge_dispatch
                     bool all_edges, boost::python::list& es) const
     {
         auto gp = retrieve_graph_view<Graph>(gi, g);
-        size_t k_t = is_directed::apply<Graph>::type::value ?
+        size_t k_t = graph_tool::is_directed(g) ?
             in_degreeS()(t, g) : out_degree(t, g);
         if (out_degree(s, g) <= k_t)
         {
@@ -268,12 +251,11 @@ struct get_edge_dispatch
         {
             for (auto e : in_or_out_edges_range(vertex(t, g), g))
             {
-                auto w = is_directed::apply<Graph>::type::value ?
-                    source(e, g) : target(e, g);
+                auto w = source(e, g);
                 if (w == vertex(s, g))
                 {
-                    if (!is_directed::apply<Graph>::type::value)
-                        e.inv ^= true;
+                    if (!graph_tool::is_directed(g) && e.s != s)
+                        std::swap(e.s, e.t);
                     es.append(PythonEdge<Graph>(gp, e));
                     if (!all_edges)
                         break;
@@ -306,15 +288,12 @@ struct get_degree_map
         map_t cdeg_map(get(vertex_index, g));
         typename map_t::unchecked_t deg_map = cdeg_map.get_unchecked(num_vertices(g));
 
-        int i, N = num_vertices(g);
-        #pragma omp parallel for default(shared) private(i) schedule(runtime) if (N > 100)
-        for (i = 0; i < N; ++i)
-        {
-            auto v = vertex(i, g);
-            if (!is_valid_vertex(v, g))
-                continue;
-            deg_map[v] = deg(v, g, weight);
-        }
+        parallel_vertex_loop
+            (g,
+             [&](auto v)
+             {
+                 deg_map[v] = deg(v, g, weight);
+             });
 
         odeg_map = python::object(PythonPropertyMap<map_t>(cdeg_map));
     }
@@ -348,6 +327,160 @@ python::object GraphInterface::degree_map(string deg, boost::any weight) const
     return deg_map;
 }
 
+python::object get_vertex_list(GraphInterface& gi)
+{
+    std::vector<size_t> vlist;
+    vlist.reserve(gi.get_num_vertices());
+    run_action<>()(gi,
+                   [&](auto& g)
+                   {
+                       for (auto v: vertices_range(g))
+                           vlist.push_back(v);
+                   })();
+    return wrap_vector_owned(vlist);
+}
+
+python::object get_edge_list(GraphInterface& gi)
+{
+    std::vector<size_t> elist;
+    run_action<>()(gi,
+                   [&](auto& g)
+                   {
+                       auto edge_index = get(edge_index_t(), g);
+                       for (auto e: edges_range(g))
+                       {
+                           elist.push_back(source(e, g));
+                           elist.push_back(target(e, g));
+                           elist.push_back(edge_index[e]);
+                       }
+                   })();
+    return wrap_vector_owned(elist);
+}
+
+python::object get_out_edge_list(GraphInterface& gi, size_t v)
+{
+    std::vector<size_t> elist;
+    run_action<>()(gi,
+                   [&](auto& g)
+                   {
+                       if (!is_valid_vertex(v, g))
+                           throw ValueException("invalid vertex: " +
+                                                lexical_cast<string>(v));
+                       auto edge_index = get(edge_index_t(), g);
+                       elist.reserve(3 * out_degree(v, g));
+                       for (auto e: out_edges_range(v, g))
+                       {
+                           elist.push_back(source(e, g));
+                           elist.push_back(target(e, g));
+                           elist.push_back(edge_index[e]);
+                       }
+                   })();
+    return wrap_vector_owned(elist);
+}
+
+python::object get_in_edge_list(GraphInterface& gi, size_t v)
+{
+    std::vector<size_t> elist;
+    run_action<>()(gi,
+                   [&](auto& g)
+                   {
+                       if (!is_valid_vertex(v, g))
+                           throw ValueException("invalid vertex: " +
+                                                lexical_cast<string>(v));
+                       auto edge_index = get(edge_index_t(), g);
+                       elist.reserve(3 * in_degree(v, g));
+                       for (auto e: in_edges_range(v, g))
+                       {
+                           elist.push_back(source(e, g));
+                           elist.push_back(target(e, g));
+                           elist.push_back(edge_index[e]);
+                       }
+                   })();
+    return wrap_vector_owned(elist);
+}
+
+python::object get_out_neighbors_list(GraphInterface& gi, size_t v)
+{
+    std::vector<size_t> vlist;
+    run_action<>()(gi,
+                   [&](auto& g)
+                   {
+                       if (!is_valid_vertex(v, g))
+                           throw ValueException("invalid vertex: " +
+                                                lexical_cast<string>(v));
+                       vlist.reserve(out_degree(v, g));
+                       for (auto u: out_neighbors_range(v, g))
+                           vlist.push_back(u);
+                   })();
+    return wrap_vector_owned(vlist);
+}
+
+python::object get_in_neighbors_list(GraphInterface& gi, size_t v)
+{
+    std::vector<size_t> vlist;
+    run_action<>()(gi,
+                   [&](auto& g)
+                   {
+                       if (!is_valid_vertex(v, g))
+                           throw ValueException("invalid vertex: " +
+                                                lexical_cast<string>(v));
+                       vlist.reserve(in_degree(v, g));
+                       for (auto u: in_neighbors_range(v, g))
+                           vlist.push_back(u);
+                   })();
+    return wrap_vector_owned(vlist);
+}
+
+python::object get_degree_list(GraphInterface& gi, python::object ovlist,
+                               boost::any eprop, bool out)
+{
+    python::object ret;
+    auto vlist = get_array<uint64_t,1>(ovlist);
+
+    typedef UnityPropertyMap<size_t,
+                             graph_traits<GraphInterface::multigraph_t>::edge_descriptor>
+        empty_t;
+    if (eprop.empty())
+    {
+        eprop = empty_t();
+    }
+    else
+    {
+        if (!belongs<edge_scalar_properties>()(eprop))
+            throw ValueException("edge weight property map must be of scalar type");
+    }
+
+    typedef mpl::push_back<edge_scalar_properties,
+                           empty_t>::type eprops_t;
+
+    auto get_degs = [&](auto deg)
+        {
+            run_action<>()(gi,
+                           [&](auto& g, auto& ew)
+                           {
+                               typedef typename std::remove_reference
+                                   <decltype(ew)>::type::value_type val_t;
+                               std::vector<val_t> dlist;
+                               dlist.reserve(vlist.size());
+                               for (auto v : vlist)
+                               {
+                                   if (!is_valid_vertex(v, g))
+                                       throw ValueException("invalid vertex: " +
+                                                            lexical_cast<string>(v));
+                                   dlist.push_back(val_t(deg(v, g, ew)));
+                               }
+                               ret = wrap_vector_owned(dlist);
+                           }, eprops_t())(eprop);
+        };
+
+    if (out)
+        get_degs(out_degreeS());
+    else
+        get_degs(in_degreeS());
+
+    return ret;
+}
+
 //
 // Below are the functions with will properly register all the types to python,
 // for every filter, type, etc.
@@ -540,7 +673,7 @@ void export_python_interface()
     using namespace boost::python;
 
     class_<VertexBase>("VertexBase", no_init);
-    class_<EdgeBase>("EdgeBase", no_init);
+    class_<EdgeBase, boost::noncopyable>("EdgeBase", no_init);
 
     typedef boost::mpl::transform<graph_tool::all_graph_views,
                                   boost::mpl::quote1<std::add_const> >::type const_graph_views;
@@ -573,6 +706,14 @@ void export_python_interface()
     def("add_edge_list_iter", graph_tool::do_add_edge_list_iter);
     def("get_edge", get_edge);
 
+    def("get_vertex_list", get_vertex_list);
+    def("get_edge_list", get_edge_list);
+    def("get_out_edge_list", get_out_edge_list);
+    def("get_in_edge_list", get_in_edge_list);
+    def("get_out_neighbors_list", get_out_neighbors_list);
+    def("get_in_neighbors_list", get_in_neighbors_list);
+    def("get_degree_list", get_degree_list);
+
     def("get_vertex_index", get_vertex_index);
     def("get_edge_index", do_get_edge_index);
 
