@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
 #include "graph_filtering.hh"
 #include "graph.hh"
 #include "graph_python_interface.hh"
+#include "demangle.hh"
 
 #include <boost/python.hpp>
 #include <boost/lambda/bind.hpp>
@@ -34,8 +35,6 @@ struct export_vertex_property_map
     template <class PropertyMap>
     void operator()(PropertyMap) const
     {
-        using boost::python::detail::gcc_demangle;
-
         typedef PythonPropertyMap<PropertyMap> pmap_t;
 
         string type_name;
@@ -43,7 +42,7 @@ struct export_vertex_property_map
                                                    typename pmap_t::value_type>::type,
                          typename boost::mpl::end<value_types>::type>::value)
             type_name =
-                gcc_demangle(typeid(typename pmap_t::value_type).name());
+                name_demangle(typeid(typename pmap_t::value_type).name());
         else
             type_name =
                 type_names[boost::mpl::find<value_types,typename pmap_t::value_type>
@@ -66,7 +65,8 @@ struct export_vertex_property_map
             .def("is_writable", &pmap_t::is_writable)
             .def("reserve", &pmap_t::reserve)
             .def("resize", &pmap_t::resize)
-            .def("shrink_to_fit", &pmap_t::shrink_to_fit);
+            .def("shrink_to_fit", &pmap_t::shrink_to_fit)
+            .def("data_ptr", &pmap_t::data_ptr);
 
         typedef boost::mpl::transform<graph_tool::all_graph_views,
                                       boost::mpl::quote1<std::add_const> >::type const_graph_views;
@@ -118,8 +118,6 @@ struct export_edge_property_map
     template <class PropertyMap>
     void operator()(PropertyMap) const
     {
-        using boost::python::detail::gcc_demangle;
-
         typedef PythonPropertyMap<PropertyMap> pmap_t;
 
         string type_name;
@@ -127,7 +125,7 @@ struct export_edge_property_map
                                                    typename pmap_t::value_type>::type,
                          typename boost::mpl::end<value_types>::type>::value)
             type_name =
-                gcc_demangle(typeid(typename pmap_t::value_type).name());
+                name_demangle(typeid(typename pmap_t::value_type).name());
         else
             type_name =
                 type_names[boost::mpl::find<value_types,typename pmap_t::value_type>
@@ -150,7 +148,8 @@ struct export_edge_property_map
             .def("is_writable", &pmap_t::is_writable)
             .def("reserve", &pmap_t::reserve)
             .def("resize", &pmap_t::resize)
-            .def("shrink_to_fit", &pmap_t::shrink_to_fit);
+            .def("shrink_to_fit", &pmap_t::shrink_to_fit)
+            .def("data_ptr", &pmap_t::data_ptr);
 
 
         typedef boost::mpl::transform<graph_tool::all_graph_views,
