@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -22,11 +22,14 @@ void export_arf();
 void export_fruchterman_reingold();
 void export_sfdp();
 void export_radial();
+void export_planar();
 
 BOOST_PYTHON_MODULE(libgraph_tool_layout)
 {
+    docstring_options dopt(true, false);
     export_arf();
     export_fruchterman_reingold();
     export_sfdp();
     export_radial();
+    export_planar();
 }
