@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -25,9 +25,8 @@
 #include "graph_selectors.hh"
 #include "graph_util.hh"
 
-#ifdef HAVE_BOOST_COROUTINE
-#include <boost/coroutine/all.hpp>
-#endif // HAVE_BOOST_COROUTINE
+#include "coroutine.hh"
+#include "graph_python_interface.hh"
 
 using namespace std;
 using namespace boost;
@@ -107,20 +106,32 @@ private:
     boost::python::object _vis;
 };
 
-struct do_bfs
+template <class Graph, class Visitor>
+void do_bfs(Graph& g, size_t s, Visitor&& vis)
 {
-    template <class Graph, class Visitor>
-    void operator()(Graph& g, size_t s, Visitor vis) const
-    {
-        breadth_first_search(g, vertex(s, g), visitor(vis));
-    }
-};
+    typename vprop_map_t<default_color_type>::type
+        color(get(vertex_index_t(), g));
 
-void bfs_search(GraphInterface& g, size_t s, python::object vis)
+    auto v = vertex(s, g);
+    if (v == graph_traits<Graph>::null_vertex())
+    {
+        for (auto u : vertices_range(g))
+        {
+            if (color[u] == color_traits<default_color_type>::black())
+                continue;
+            breadth_first_visit(g, u, visitor(vis).color_map(color));
+        }
+    }
+    else
+    {
+        breadth_first_visit(g, v, visitor(vis).color_map(color));
+    }
+}
+
+void bfs_search(GraphInterface& gi, size_t s, python::object vis)
 {
     run_action<graph_tool::all_graph_views,mpl::true_>()
-        (g, std::bind(do_bfs(), std::placeholders::_1, s,
-                      BFSVisitorWrapper(g, vis)))();
+        (gi, [&](auto &g){ do_bfs(g, s, BFSVisitorWrapper(gi, vis)); })();
 }
 
 #ifdef HAVE_BOOST_COROUTINE
@@ -153,7 +164,7 @@ boost::python::object bfs_search_generator(GraphInterface& g, size_t s)
         {
             BFSGeneratorVisitor vis(g, yield);
             run_action<graph_tool::all_graph_views,mpl::true_>()
-                (g, std::bind(do_bfs(), std::placeholders::_1, s, vis))();
+                (g, [&](auto &g){ do_bfs(g, s, vis); })();
         };
     return boost::python::object(CoroGenerator(dispatch));
 #else
@@ -161,9 +172,36 @@ boost::python::object bfs_search_generator(GraphInterface& g, size_t s)
 #endif
 }
 
+class BFSArrayVisitor : public bfs_visitor<>
+{
+public:
+    BFSArrayVisitor(std::vector<std::array<size_t, 2>>& edges)
+        : _edges(edges) {}
+
+    template <class Edge, class Graph>
+    void tree_edge(const Edge& e, Graph& g)
+    {
+        _edges.push_back({{source(e, g), target(e,g)}});
+    }
+
+private:
+    std::vector<std::array<size_t, 2>>& _edges;
+};
+
+boost::python::object bfs_search_array(GraphInterface& g, size_t s)
+{
+    std::vector<std::array<size_t, 2>> edges;
+    BFSArrayVisitor vis(edges);
+    run_action<graph_tool::all_graph_views,mpl::true_>()
+        (g, [&](auto &g){ do_bfs(g, s, vis); })();
+    return wrap_vector_owned<size_t,2>(edges);
+}
+
+
 void export_bfs()
 {
     using namespace boost::python;
     def("bfs_search", &bfs_search);
     def("bfs_search_generator", &bfs_search_generator);
+    def("bfs_search_array", &bfs_search_array);
 }
