@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -25,9 +25,8 @@
 #include "graph_selectors.hh"
 #include "graph_util.hh"
 
-#ifdef HAVE_BOOST_COROUTINE
-#include <boost/coroutine/all.hpp>
-#endif // HAVE_BOOST_COROUTINE
+#include "coroutine.hh"
+#include "graph_python_interface.hh"
 
 using namespace std;
 using namespace boost;
@@ -141,11 +140,33 @@ struct do_djk_search
         typedef typename graph_traits<Graph>::edge_descriptor edge_t;
         DynamicPropertyMapWrap<dtype_t, edge_t> weight(aweight,
                                                        edge_properties());
-        dijkstra_shortest_paths_no_color_map
-            (g, vertex(s, g), visitor(vis).weight_map(weight).
-             predecessor_map(pred_map).
-             distance_map(dist).distance_compare(cmp).
-             distance_combine(cmb).distance_inf(i).distance_zero(z));
+
+        if (vertex(s, g) == graph_traits<Graph>::null_vertex())
+        {
+            for (auto u : vertices_range(g))
+            {
+                vis.initialize_vertex(u, g);
+                put(dist, u, i);
+                put(pred_map, u, u);
+            }
+            for (auto u : vertices_range(g))
+            {
+                if (dist[u] != i)
+                    continue;
+                dist[u] = z;
+                dijkstra_shortest_paths_no_color_map_no_init
+                    (g, u, pred_map, dist, weight, get(vertex_index_t(), g),
+                     cmp, cmb, i, z, vis);
+            }
+        }
+        else
+        {
+            dijkstra_shortest_paths_no_color_map
+                (g, vertex(s, g), visitor(vis).weight_map(weight).
+                 predecessor_map(pred_map).
+                 distance_map(dist).distance_compare(cmp).
+                 distance_combine(cmb).distance_inf(i).distance_zero(z));
+        }
     }
 };
 
@@ -159,9 +180,31 @@ struct do_djk_search_fast
         typedef typename property_traits<DistanceMap>::value_type dtype_t;
         dtype_t z = python::extract<dtype_t>(range.first);
         dtype_t i = python::extract<dtype_t>(range.second);
-        dijkstra_shortest_paths_no_color_map
-            (g, vertex(s, g), visitor(vis).weight_map(weight).
-             distance_map(dist).distance_inf(i).distance_zero(z));
+
+        if (vertex(s, g) == graph_traits<Graph>::null_vertex())
+        {
+            for (auto u : vertices_range(g))
+            {
+                vis.initialize_vertex(u, g);
+                put(dist, u, i);
+            }
+            for (auto u : vertices_range(g))
+            {
+                if (dist[u] != i)
+                    continue;
+                dist[u] = z;
+                dijkstra_shortest_paths_no_color_map_no_init
+                    (g, u, dummy_property_map(), dist, weight,
+                     get(vertex_index_t(), g), std::less<dtype_t>(),
+                     boost::closed_plus<dtype_t>(), i, z, vis);
+            }
+        }
+        else
+        {
+            dijkstra_shortest_paths_no_color_map
+                (g, vertex(s, g), visitor(vis).weight_map(weight).
+                 distance_map(dist).distance_inf(i).distance_zero(z));
+        }
     }
 };
 
@@ -254,10 +297,67 @@ boost::python::object dijkstra_search_generator_fast(GraphInterface& g,
 #endif
 }
 
+class DJKArrayVisitor: public dijkstra_visitor<>
+{
+public:
+    DJKArrayVisitor(std::vector<std::array<size_t, 2>>& edges)
+        : _edges(edges) {}
+
+    template <class Edge, class Graph>
+    void edge_relaxed(const Edge& e, Graph& g)
+    {
+        _edges.push_back({{source(e, g), target(e,g)}});
+    }
+
+private:
+    std::vector<std::array<size_t, 2>>& _edges;
+};
+
+
+boost::python::object dijkstra_search_array(GraphInterface& g,
+                                            size_t source,
+                                            boost::any dist_map,
+                                            boost::any weight,
+                                            python::object cmp,
+                                            python::object cmb,
+                                            python::object zero,
+                                            python::object inf)
+{
+    std::vector<std::array<size_t, 2>> edges;
+    DJKArrayVisitor vis(edges);
+    run_action<graph_tool::all_graph_views, mpl::true_>()
+        (g, std::bind(do_djk_search(), std::placeholders::_1, source,
+                      std::placeholders::_2, dummy_property_map(), weight,
+                      vis, DJKCmp(cmp), DJKCmb(cmb),
+                      make_pair(zero, inf)),
+         writable_vertex_properties())(dist_map);
+    return wrap_vector_owned<size_t,2>(edges);
+}
+
+boost::python::object dijkstra_search_array_fast(GraphInterface& g,
+                                                 size_t source,
+                                                 boost::any dist_map,
+                                                 boost::any weight,
+                                                 python::object zero,
+                                                 python::object inf)
+{
+    std::vector<std::array<size_t, 2>> edges;
+    DJKArrayVisitor vis(edges);
+    run_action<graph_tool::all_graph_views, mpl::true_>()
+        (g, std::bind(do_djk_search_fast(), std::placeholders::_1, source,
+                      std::placeholders::_2, std::placeholders::_3,
+                      vis, make_pair(zero, inf)),
+         writable_vertex_scalar_properties(),
+         edge_scalar_properties())(dist_map, weight);
+    return wrap_vector_owned<size_t,2>(edges);
+}
+
 void export_dijkstra()
 {
     using namespace boost::python;
     def("dijkstra_search", &dijkstra_search);
     def("dijkstra_generator", &dijkstra_search_generator);
     def("dijkstra_generator_fast", &dijkstra_search_generator_fast);
+    def("dijkstra_array", &dijkstra_search_array);
+    def("dijkstra_array_fast", &dijkstra_search_array_fast);
 }
