@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -29,35 +29,51 @@ struct get_bipartite
 {
     template <class Graph, class VertexIndex, class PartMap>
     void operator()(Graph& g, VertexIndex vertex_index, PartMap part_map,
-                    bool& is_bip) const
+                    bool& is_bip, bool find_cycle, vector<size_t>& odd_cycle)
+        const
     {
         unchecked_vector_property_map<default_color_type, VertexIndex>
             part(vertex_index, num_vertices(g));
 
-        is_bip = is_bipartite(g, vertex_index, part);
-
-        int i, N = num_vertices(g);
-        #pragma omp parallel for default(shared) private(i) schedule(runtime) if (N > 100)
-        for (i = 0; i < N; ++i)
+        if (!find_cycle)
         {
-            auto v = vertex(i, g);
-            if (!is_valid_vertex(v, g))
-                continue;
-            part_map[v] = (part[v] == color_traits<default_color_type>::white());
+            is_bip = is_bipartite(g, vertex_index, part);
         }
+        else
+        {
+            find_odd_cycle(g, vertex_index, part, std::back_inserter(odd_cycle));
+            is_bip = odd_cycle.empty();
+        }
+
+        parallel_vertex_loop
+            (g,
+             [&](auto v)
+             {
+                 part_map[v] = (part[v] == color_traits<default_color_type>::white());
+             });
     }
 
     template <class Graph, class VertexIndex>
-    void operator()(Graph& g, VertexIndex vertex_index,
-                    dummy_property_map, bool& is_bip) const
+    void operator()(Graph& g, VertexIndex vertex_index, dummy_property_map,
+                    bool& is_bip, bool find_cycle, vector<size_t>& odd_cycle) const
     {
-        is_bip = is_bipartite(g, vertex_index);
+        if (!find_cycle)
+        {
+            is_bip = is_bipartite(g, vertex_index);
+        }
+        else
+        {
+            find_odd_cycle(g, vertex_index, std::back_inserter(odd_cycle));
+            is_bip = odd_cycle.empty();
+        }
     }
 };
 
-bool is_bipartite(GraphInterface& gi, boost::any part_map)
+bool is_bipartite(GraphInterface& gi, boost::any part_map, bool find_cycle,
+                  boost::python::list cycle)
 {
     bool is_bip;
+    vector<size_t> vcycle;
 
     if (part_map.empty())
         part_map = dummy_property_map();
@@ -66,8 +82,13 @@ bool is_bipartite(GraphInterface& gi, boost::any part_map)
                            dummy_property_map>::type vertex_map_types;
 
     run_action<graph_tool::detail::never_directed>()
-        (gi, std::bind(get_bipartite(), std::placeholders::_1, gi.get_vertex_index(),
-                       std::placeholders::_2, std::ref(is_bip)),
+        (gi, std::bind(get_bipartite(), std::placeholders::_1,
+                       gi.get_vertex_index(), std::placeholders::_2,
+                       std::ref(is_bip), find_cycle, std::ref(vcycle)),
          vertex_map_types())(part_map);
+
+    for (auto v : vcycle)
+        cycle.append(v);
+
     return is_bip;
 }
