@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -22,6 +22,8 @@
 #include "graph_python_interface.hh"
 #include "numpy_bind.hh"
 #include "hash_map_wrap.hh"
+#include "coroutine.hh"
+#include "graph_python_interface.hh"
 
 #include <boost/graph/breadth_first_search.hpp>
 #include <boost/graph/dijkstra_shortest_paths_no_color_map.hpp>
@@ -29,9 +31,9 @@
 #include <boost/python/stl_iterator.hpp>
 #include <boost/python.hpp>
 
-#ifdef HAVE_BOOST_COROUTINE
-#include <boost/coroutine/all.hpp>
-#endif // HAVE_BOOST_COROUTINE
+#if (BOOST_VERSION >= 106000)
+# include <boost/math/special_functions/relative_difference.hpp>
+#endif
 
 using namespace std;
 using namespace boost;
@@ -45,20 +47,18 @@ class bfs_max_visitor:
 {
 public:
     bfs_max_visitor(DistMap dist_map, PredMap pred, size_t max_dist,
-                    size_t source, size_t target)
+                    size_t source, size_t target, std::vector<size_t>& reached)
         : _dist_map(dist_map), _pred(pred), _max_dist(max_dist),
-          _source(source), _target(target), _dist(0) {}
+          _source(source), _target(target), _dist(0), _reached(reached) {}
 
-    template <class Graph>
-    void initialize_vertex(typename graph_traits<Graph>::vertex_descriptor v,
-                           Graph&)
+    ~bfs_max_visitor()
     {
         typedef typename property_traits<DistMap>::value_type dist_t;
-        dist_t inf = std::is_floating_point<dist_t>::value ?
+        constexpr dist_t inf = std::is_floating_point<dist_t>::value ?
             numeric_limits<dist_t>::infinity() :
             numeric_limits<dist_t>::max();
-        _dist_map[v] = (v == _source) ? 0 : inf;
-        _pred[v] = v;
+        for (auto v : _unreached)
+            _dist_map[v] = inf;
     }
 
     template <class Graph>
@@ -72,8 +72,7 @@ public:
     void examine_vertex(typename graph_traits<Graph>::vertex_descriptor v,
                         Graph&)
     {
-        typedef typename property_traits<DistMap>::value_type val_t;
-        if ( _dist_map[v] > val_t(_max_dist))
+        if ( _dist_map[v] > _max_dist)
             throw stop_search();
     }
 
@@ -81,9 +80,13 @@ public:
     void discover_vertex(typename graph_traits<Graph>::vertex_descriptor v,
                          Graph&)
     {
-        if (size_t(_pred[v]) == v)
+        auto p = _pred[v];
+        if (size_t(p) == v)
             return;
-        _dist_map[v] = _dist_map[_pred[v]] + 1;
+        _dist_map[v] = _dist_map[p] + 1;
+        _reached.push_back(v);
+        if (_dist_map[v] > _max_dist)
+            _unreached.push_back(v);
         if (v == _target)
             throw stop_search();
     }
@@ -91,10 +94,12 @@ public:
 private:
     DistMap _dist_map;
     PredMap _pred;
-    size_t _max_dist;
+    typename property_traits<DistMap>::value_type _max_dist;
     size_t _source;
     size_t _target;
     size_t _dist;
+    std::vector<size_t> _unreached;
+    std::vector<size_t>& _reached;
 };
 
 template <class DistMap, class PredMap>
@@ -104,20 +109,19 @@ class bfs_max_multiple_targets_visitor:
 public:
     bfs_max_multiple_targets_visitor(DistMap dist_map, PredMap pred,
                                      size_t max_dist, size_t source,
-                                     gt_hash_set<std::size_t> target)
+                                     gt_hash_set<std::size_t> target,
+                                     std::vector<size_t>& reached)
         : _dist_map(dist_map), _pred(pred), _max_dist(max_dist),
-          _source(source), _target(target), _dist(0) {}
+          _source(source), _target(target), _dist(0), _reached(reached) {}
 
-    template <class Graph>
-    void initialize_vertex(typename graph_traits<Graph>::vertex_descriptor v,
-                           Graph&)
+    ~bfs_max_multiple_targets_visitor()
     {
         typedef typename property_traits<DistMap>::value_type dist_t;
-        dist_t inf = std::is_floating_point<dist_t>::value ?
+        constexpr dist_t inf = std::is_floating_point<dist_t>::value ?
             numeric_limits<dist_t>::infinity() :
             numeric_limits<dist_t>::max();
-        _dist_map[v] = (v == _source) ? 0 : inf;
-        _pred[v] = v;
+        for (auto v : _unreached)
+            _dist_map[v] = inf;
     }
 
     template <class Graph>
@@ -131,8 +135,7 @@ public:
     void examine_vertex(typename graph_traits<Graph>::vertex_descriptor v,
                         Graph&)
     {
-        typedef typename property_traits<DistMap>::value_type val_t;
-        if ( _dist_map[v] > val_t(_max_dist))
+        if ( _dist_map[v] > _max_dist)
             throw stop_search();
     }
 
@@ -140,9 +143,13 @@ public:
     void discover_vertex(typename graph_traits<Graph>::vertex_descriptor v,
                          Graph&)
     {
-        if (size_t(_pred[v]) == v)
+        auto p = _pred[v];
+        if (size_t(p) == v)
             return;
-        _dist_map[v] = _dist_map[_pred[v]] + 1;
+        _dist_map[v] = _dist_map[p] + 1;
+
+        if (_dist_map[v] > _max_dist)
+            _unreached.push_back(v);
 
         auto iter = _target.find(v);
         if (iter != _target.end())
@@ -156,10 +163,12 @@ public:
 private:
     DistMap _dist_map;
     PredMap _pred;
-    size_t _max_dist;
+    typename property_traits<DistMap>::value_type _max_dist;
     size_t _source;
     gt_hash_set<std::size_t> _target;
     size_t _dist;
+    std::vector<size_t> _unreached;
+    std::vector<size_t>& _reached;
 };
 
 
@@ -170,9 +179,16 @@ class djk_max_visitor:
 public:
     djk_max_visitor(DistMap dist_map,
                     typename property_traits<DistMap>::value_type max_dist,
-                    size_t target)
-        : _dist_map(dist_map), _max_dist(max_dist), _target(target) {}
+                    typename property_traits<DistMap>::value_type inf,
+                    size_t target, std::vector<size_t>& reached)
+        : _dist_map(dist_map), _max_dist(max_dist), _inf(inf),
+          _target(target), _reached(reached) {}
 
+    ~djk_max_visitor()
+    {
+        for (auto v : _unreached)
+            _dist_map[v] = _inf;
+    }
 
     template <class Graph>
     void examine_vertex(typename graph_traits<Graph>::vertex_descriptor u,
@@ -185,11 +201,22 @@ public:
             throw stop_search();
     }
 
+    template <class Graph>
+    void discover_vertex(typename graph_traits<Graph>::vertex_descriptor u,
+                         Graph&)
+    {
+        if (_dist_map[u] > _max_dist)
+            _unreached.push_back(u);
+        _reached.push_back(u);
+    }
 
 private:
     DistMap _dist_map;
     typename property_traits<DistMap>::value_type _max_dist;
+    typename property_traits<DistMap>::value_type _inf;
     size_t _target;
+    std::vector<size_t> _unreached;
+    std::vector<size_t>& _reached;
 };
 
 
@@ -200,9 +227,17 @@ class djk_max_multiple_targets_visitor:
 public:
     djk_max_multiple_targets_visitor(DistMap dist_map,
                                      typename property_traits<DistMap>::value_type max_dist, 
-                                     gt_hash_set<std::size_t> target)
-        : _dist_map(dist_map), _max_dist(max_dist), _target(target) {}
+                                     typename property_traits<DistMap>::value_type inf, 
+                                     gt_hash_set<std::size_t> target,
+                                     std::vector<size_t>& reached)
+        : _dist_map(dist_map), _max_dist(max_dist), _inf(inf),
+          _target(target), _reached(reached) {}
 
+    ~djk_max_multiple_targets_visitor()
+    {
+        for (auto v : _unreached)
+            _dist_map[v] = _inf;
+    }
 
     template <class Graph>
     void examine_vertex(typename graph_traits<Graph>::vertex_descriptor u,
@@ -220,11 +255,22 @@ public:
         };
     }
 
+    template <class Graph>
+    void discover_vertex(typename graph_traits<Graph>::vertex_descriptor u,
+                         Graph&)
+    {
+        if (_dist_map[u] > _max_dist)
+            _unreached.push_back(u);
+        _reached.push_back(u);
+    }
 
 private:
     DistMap _dist_map;
     typename property_traits<DistMap>::value_type _max_dist;
+    typename property_traits<DistMap>::value_type _inf;
     gt_hash_set<std::size_t> _target;
+    std::vector<size_t> _unreached;
+    std::vector<size_t>& _reached;
 };
 
 struct do_bfs_search
@@ -233,7 +279,8 @@ struct do_bfs_search
     void operator()(const Graph& g, size_t source,
                     boost::python::object otarget_list,
                     VertexIndexMap vertex_index, DistMap dist_map,
-                    PredMap pred_map, long double max_dist) const
+                    PredMap pred_map, long double max_dist,
+                    std::vector<size_t>& reached) const
     {
         typedef typename property_traits<DistMap>::value_type dist_t;
 
@@ -247,6 +294,8 @@ struct do_bfs_search
 
         dist_t max_d = (max_dist > 0) ? max_dist : inf;
 
+        dist_map[source] = 0;
+
         unchecked_vector_property_map<boost::default_color_type, VertexIndexMap>
         color_map(vertex_index, num_vertices(g));
         try
@@ -256,21 +305,21 @@ struct do_bfs_search
                 size_t target = tgt.empty() ?
                     graph_traits<GraphInterface::multigraph_t>::null_vertex() :
                     *tgt.begin();
-                breadth_first_search(g, vertex(source, g),
-                                     visitor(bfs_max_visitor<DistMap, PredMap>
-                                             (dist_map, pred_map, max_d,
-                                              source, target)).
-                                     vertex_index_map(vertex_index).
-                                     color_map(color_map));
+                breadth_first_visit(g, vertex(source, g),
+                                    visitor(bfs_max_visitor<DistMap, PredMap>
+                                            (dist_map, pred_map, max_d,
+                                             source, target, reached)).
+                                    vertex_index_map(vertex_index).
+                                    color_map(color_map));
             }
             else
             {
-                breadth_first_search(g, vertex(source, g),
-                                     visitor(bfs_max_multiple_targets_visitor<DistMap, PredMap>
-                                             (dist_map, pred_map, max_d,
-                                              source, tgt)).
-                                     vertex_index_map(vertex_index).
-                                     color_map(color_map));
+                breadth_first_visit(g, vertex(source, g),
+                                    visitor(bfs_max_multiple_targets_visitor<DistMap, PredMap>
+                                            (dist_map, pred_map, max_d,
+                                             source, tgt, reached)).
+                                    vertex_index_map(vertex_index).
+                                    color_map(color_map));
             }
 
         }
@@ -285,27 +334,22 @@ struct do_djk_search
     void operator()(const Graph& g, size_t source,
                     boost::python::object otarget_list,
                     VertexIndexMap vertex_index, DistMap dist_map,
-                    PredMap pred_map, WeightMap weight, long double max_dist) const
+                    PredMap pred_map, WeightMap weight, long double max_dist,
+                    std::vector<size_t>& reached) const
     {
         auto target_list = get_array<int64_t, 1>(otarget_list);
+
         typedef typename property_traits<DistMap>::value_type dist_t;
-        dist_t max_d = (max_dist > 0) ?
-            max_dist : (std::is_floating_point<dist_t>::value ?
-                        numeric_limits<dist_t>::infinity() :
-                        numeric_limits<dist_t>::max());
+
+        constexpr dist_t inf = (std::is_floating_point<dist_t>::value) ?
+            numeric_limits<dist_t>::infinity() :
+            numeric_limits<dist_t>::max();
+
+        dist_t max_d = (max_dist > 0) ? max_dist : inf;
 
         gt_hash_set<std::size_t> tgt(target_list.begin(),
                                      target_list.end());
 
-        dist_t inf = (std::is_floating_point<dist_t>::value) ?
-            numeric_limits<dist_t>::infinity() :
-            numeric_limits<dist_t>::max();
-
-        int i, N = num_vertices(g);
-        #pragma omp parallel for default(shared) private(i) \
-            schedule(runtime) if (N > 100)
-        for (i = 0; i < N; ++i)
-            dist_map[i] = inf;
         dist_map[source] = 0;
 
         try
@@ -315,31 +359,27 @@ struct do_djk_search
                 size_t target = tgt.empty() ?
                     graph_traits<GraphInterface::multigraph_t>::null_vertex() :
                     *tgt.begin();
-                dijkstra_shortest_paths_no_color_map
-                    (g, vertex(source, g),
-                     weight_map(weight).
-                     distance_map(dist_map).
-                     vertex_index_map(vertex_index).
-                     predecessor_map(pred_map).
-                     distance_inf(inf).
-                     visitor(djk_max_visitor<DistMap>
-                             (dist_map, max_d, target)));
+                dijkstra_shortest_paths_no_color_map_no_init
+                    (g, vertex(source, g), pred_map, dist_map, weight,
+                     vertex_index, std::less<dist_t>(),
+                     boost::closed_plus<dist_t>(), inf, dist_t(),
+                     djk_max_visitor<DistMap>(dist_map, max_d, inf, target,
+                                              reached));
             }
             else
             {
-                dijkstra_shortest_paths_no_color_map
-                    (g, vertex(source, g),
-                     weight_map(weight).
-                     distance_map(dist_map).
-                     vertex_index_map(vertex_index).
-                     predecessor_map(pred_map).
-                     distance_inf(inf).
-                     visitor(djk_max_multiple_targets_visitor<DistMap>
-                             (dist_map, max_d, tgt)));
+                dijkstra_shortest_paths_no_color_map_no_init
+                    (g, vertex(source, g), pred_map, dist_map, weight,
+                     vertex_index, std::less<dist_t>(),
+                     boost::closed_plus<dist_t>(), inf, dist_t(),
+                     djk_max_multiple_targets_visitor<DistMap>(dist_map, max_d,
+                                                               inf, tgt,
+                                                               reached));
             }
 
         }
         catch (stop_search&) {}
+
     }
 };
 
@@ -371,7 +411,7 @@ struct do_bf_search
 
 void get_dists(GraphInterface& gi, size_t source, boost::python::object tgt,
                boost::any dist_map, boost::any weight, boost::any pred_map,
-               long double max_dist, bool bf)
+               long double max_dist, bool bf, std::vector<size_t>& reached)
 {
     typedef property_map_type
         ::apply<int64_t, GraphInterface::vertex_index_map_t>::type pred_map_t;
@@ -383,7 +423,7 @@ void get_dists(GraphInterface& gi, size_t source, boost::python::object tgt,
         run_action<>()
             (gi, std::bind(do_bfs_search(), std::placeholders::_1, source, tgt, gi.get_vertex_index(),
                            std::placeholders::_2, pmap.get_unchecked(num_vertices(gi.get_graph())),
-                           max_dist),
+                           max_dist, std::ref(reached)),
              writable_vertex_scalar_properties())
             (dist_map);
     }
@@ -404,7 +444,7 @@ void get_dists(GraphInterface& gi, size_t source, boost::python::object tgt,
             run_action<>()
                 (gi, std::bind(do_djk_search(), std::placeholders::_1, source, tgt, gi.get_vertex_index(),
                                std::placeholders::_2, pmap.get_unchecked(num_vertices(gi.get_graph())),
-                               std::placeholders::_3, max_dist),
+                               std::placeholders::_3, max_dist, std::ref(reached)),
                  writable_vertex_scalar_properties(),
                  edge_scalar_properties())
                 (dist_map, weight);
@@ -412,31 +452,43 @@ void get_dists(GraphInterface& gi, size_t source, boost::python::object tgt,
     }
 }
 
-template <class Graph, class Dist, class Pred, class Preds>
-void get_all_preds(Graph g, Dist dist, Pred pred, Preds preds)
+template <class Graph, class Dist, class Pred, class Weight, class Preds>
+void get_all_preds(Graph g, Dist dist, Pred pred, Weight weight, Preds preds,
+                   long double epsilon)
 {
-    int i, N = num_vertices(g);
-    #pragma omp parallel for default(shared) private(i)     \
-        schedule(runtime) if (N > 100)
-    for (i = 0; i < N; ++i)
-    {
-        size_t v = vertex(i, g);
-        if (!is_valid_vertex(v, g))
-            continue;
-        if (size_t(pred[v]) == v)
-            continue;
-        auto d = dist[pred[v]];
-        for (auto e : in_or_out_edges_range(v, g))
-        {
-            auto u = boost::is_directed(g) ? source(e, g) : target(e, g);
-            if (dist[u] == d)
-                preds[v].push_back(u);
-        }
-    }
+    parallel_vertex_loop
+        (g,
+         [&](auto v)
+         {
+            if (size_t(pred[v]) == v)
+                return;
+            auto d = dist[v];
+            typedef decltype(d) dist_t;
+            for (auto e : in_or_out_edges_range(v, g))
+            {
+                auto u = source(e, g);
+                if (!std::is_floating_point<dist_t>::value)
+                {
+                    if (dist_t(dist[u] + get(weight, e)) == d)
+                        preds[v].push_back(u);
+                }
+                else
+                {
+#if (BOOST_VERSION >= 106000)
+                    if (boost::math::relative_difference(dist_t(dist[u] + get(weight, e)), d) < epsilon)
+                        preds[v].push_back(u);
+#else
+                    if (abs(dist_t(dist[u] + get(weight, e)) - d) < epsilon)
+                        preds[v].push_back(u);
+#endif
+                }
+            }
+         });
 };
 
-void do_get_all_preds(GraphInterface& gi, boost::any adist,
-                      boost::any apred, boost::any apreds)
+void do_get_all_preds(GraphInterface& gi, boost::any adist, boost::any apred,
+                      boost::any aweight, boost::any apreds,
+                      long double epsilon)
 {
     typedef property_map_type
         ::apply<int64_t, GraphInterface::vertex_index_map_t>::type pred_map_t;
@@ -446,11 +498,19 @@ void do_get_all_preds(GraphInterface& gi, boost::any adist,
     pred_map_t pred = any_cast<pred_map_t>(apred);
     preds_map_t preds = any_cast<preds_map_t>(apreds);
 
+    typedef UnityPropertyMap<int,GraphInterface::edge_t> weight_map_t;
+    typedef boost::mpl::push_back<edge_scalar_properties, weight_map_t>::type
+        weight_props_t;
+
+    if (aweight.empty())
+        aweight = weight_map_t();
+
     run_action<>()
-        (gi, [&](auto& g, auto dist)
+        (gi, [&](auto& g, auto dist, auto weight)
              {get_all_preds(g, dist, pred.get_unchecked(num_vertices(g)),
-                            preds.get_unchecked(num_vertices(g)));},
-         vertex_scalar_properties())(adist);
+                            weight, preds.get_unchecked(num_vertices(g)),
+                            epsilon);},
+         vertex_scalar_properties(), weight_props_t())(adist, aweight);
 }
 
 
