@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -17,18 +17,15 @@
 
 #include "graph.hh"
 #include "graph_filtering.hh"
-
 #include "random.hh"
+#include "coroutine.hh"
 
 #include <boost/graph/vf2_sub_graph_iso.hpp>
 #include <graph_python_interface.hh>
 
-#ifdef HAVE_BOOST_COROUTINE
-#include <boost/coroutine/all.hpp>
-#endif // HAVE_BOOST_COROUTINE
-
 using namespace graph_tool;
 using namespace boost;
+using namespace std;
 
 struct ListMatch
 {
@@ -78,7 +75,7 @@ struct ListMatch
 
 #ifdef HAVE_BOOST_COROUTINE
 
-typedef boost::coroutines::asymmetric_coroutine<boost::python::object> coro_t;
+typedef graph_tool::coroutines::asymmetric_coroutine<boost::python::object> coro_t;
 
 struct GenMatch
 {
