@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -18,10 +18,12 @@
 #include <boost/python.hpp>
 #include "graph.hh"
 #include "random.hh"
+#include "numpy_bind.hh"
 
 using namespace boost;
 using namespace boost::python;
 using namespace graph_tool;
+using namespace std;
 
 bool check_isomorphism(GraphInterface& gi1, GraphInterface& gi2,
                        boost::any ainv_map1, boost::any ainv_map2,
@@ -44,7 +46,8 @@ python::object subgraph_isomorphism(GraphInterface& gi1, GraphInterface& gi2,
 double reciprocity(GraphInterface& gi);
 size_t sequential_coloring(GraphInterface& gi, boost::any order,
                            boost::any color);
-bool is_bipartite(GraphInterface& gi, boost::any part_map);
+bool is_bipartite(GraphInterface& gi, boost::any part_map, bool find_cycle,
+                  boost::python::list cycle);
 void get_random_spanning_tree(GraphInterface& gi, size_t root,
                               boost::any weight_map, boost::any tree_map,
                               rng_t& rng);
@@ -52,9 +55,11 @@ vector<int32_t> get_tsp(GraphInterface& gi, size_t src, boost::any weight_map);
 
 void export_components();
 void export_kcore();
+void export_percolation();
 void export_similarity();
 void export_dists();
 void export_all_dists();
+void export_all_circuits();
 void export_diam();
 void export_random_matching();
 void export_maximal_vertex_set();
@@ -63,6 +68,7 @@ void export_vertex_similarity();
 
 BOOST_PYTHON_MODULE(libgraph_tool_topology)
 {
+    docstring_options dopt(true, false);
     def("check_isomorphism", &check_isomorphism);
     def("subgraph_isomorphism", &subgraph_isomorphism);
     def("get_kruskal_spanning_tree", &get_kruskal_spanning_tree);
@@ -79,9 +85,11 @@ BOOST_PYTHON_MODULE(libgraph_tool_topology)
     def("get_tsp", &get_tsp);
     export_components();
     export_kcore();
+    export_percolation();
     export_similarity();
     export_dists();
     export_all_dists();
+    export_all_circuits();
     export_diam();
     export_random_matching();
     export_maximal_vertex_set();
