@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@ from __future__ import division, absolute_import, print_function
 from .. dl_import import dl_import
 dl_import("from . import libgraph_tool_centrality")
 
-from .. import _prop, ungroup_vector_property
+from .. import _prop, ungroup_vector_property, Vector_size_t
 from .. topology import shortest_distance
 import sys
 import numpy
@@ -109,7 +109,7 @@ def pagerank(g, damping=0.85, pers=None, weight=None, prop=None, epsilon=1e-6,
         PR(v) = \frac{1-d}{N} + d \sum_{u \in \Gamma^{-}(v)}
                 \frac{PR (u)}{d^{+}(u)}
 
-    where :math:`\Gamma^{-}(v)` are the in-neighbours of v, :math:`d^{+}(w)` is
+    where :math:`\Gamma^{-}(v)` are the in-neighbors of v, :math:`d^{+}(w)` is
     the out-degree of w, and d is a damping factor.
 
     If a personalization property :math:`p(v)` is given, the definition becomes:
@@ -212,9 +212,9 @@ def pagerank(g, damping=0.85, pers=None, weight=None, prop=None, epsilon=1e-6,
        Weblogging Ecosystem (2005). :DOI:`10.1145/1134271.1134277`
     """
 
-    if max_iter == None:
+    if max_iter is None:
         max_iter = 0
-    if prop == None:
+    if prop is None:
         prop = g.new_vertex_property("double")
         N = len(prop.fa)
         prop.fa = pers.fa[:N] if pers is not None else 1. / g.num_vertices()
@@ -228,14 +228,18 @@ def pagerank(g, damping=0.85, pers=None, weight=None, prop=None, epsilon=1e-6,
         return prop
 
 
-def betweenness(g, vprop=None, eprop=None, weight=None, norm=True):
-    r"""
-    Calculate the betweenness centrality for each vertex and edge.
+def betweenness(g, pivots=None, vprop=None, eprop=None, weight=None, norm=True):
+    r"""Calculate the betweenness centrality for each vertex and edge.
 
     Parameters
     ----------
     g : :class:`~graph_tool.Graph`
         Graph to be used.
+    pivots : list or :class:`~numpy.ndarray`, optional (default: None)
+        If provided, the betweenness will be estimated using the vertices in
+        this list as pivots. If the list contains all nodes (the default) the
+        algorithm will be exact, and if the vertices are randomly chosen the
+        result will be an unbiased estimator.
     vprop : :class:`~graph_tool.PropertyMap`, optional (default: None)
         Vertex property map to store the vertex betweenness values.
     eprop : :class:`~graph_tool.PropertyMap`, optional (default: None)
@@ -268,15 +272,19 @@ def betweenness(g, vprop=None, eprop=None, weight=None, norm=True):
         C_B(v)= \sum_{s \neq v \neq t \in V \atop s \neq t}
                 \frac{\sigma_{st}(v)}{\sigma_{st}}
 
-    where :math:`\sigma_{st}` is the number of shortest geodesic paths from s to
-    t, and :math:`\sigma_{st}(v)` is the number of shortest geodesic paths from
-    s to t that pass through a vertex v.  This may be normalised by dividing
-    through the number of pairs of vertices not including v, which is
-    :math:`(n-1)(n-2)/2`.
+    where :math:`\sigma_{st}` is the number of shortest paths from s to t, and
+    :math:`\sigma_{st}(v)` is the number of shortest paths from s to t that pass
+    through a vertex :math:`v`. This may be normalised by dividing through the
+    number of pairs of vertices not including v, which is :math:`(n-1)(n-2)/2`,
+    for undirected graphs, or :math:`(n-1)(n-2)` for directed ones.
 
     The algorithm used here is defined in [brandes-faster-2001]_, and has a
-    complexity of :math:`O(VE)` for unweighted graphs and :math:`O(VE + V(V+E)
-    \log V)` for weighted graphs. The space complexity is :math:`O(VE)`.
+    complexity of :math:`O(VE)` for unweighted graphs and :math:`O(VE +
+    V(V+E)\log V)` for weighted graphs. The space complexity is :math:`O(VE)`.
+
+    If the ``pivots`` parameter is given, the complexity will be instead
+    :math:`O(PE)` for unweighted graphs and :math:`O(PE + P(V+E)\log V)` for
+    weighted graphs, where :math:`P` is the number of pivot vertices.
 
     If enabled during compilation, this algorithm runs in parallel.
 
@@ -319,20 +327,30 @@ def betweenness(g, vprop=None, eprop=None, weight=None, norm=True):
     .. [betweenness-wikipedia] http://en.wikipedia.org/wiki/Centrality#Betweenness_centrality
     .. [brandes-faster-2001] U. Brandes, "A faster algorithm for betweenness
        centrality", Journal of Mathematical Sociology, 2001, :doi:`10.1080/0022250X.2001.9990249`
+    .. [brandes-centrality-2007] U. Brandes, C. Pich, "Centrality estimation in
+       large networks", Int. J. Bifurcation Chaos 17, 2303 (2007).
+       :DOI:`10.1142/S0218127407018403`
     .. [adamic-polblogs] L. A. Adamic and N. Glance, "The political blogosphere
        and the 2004 US Election", in Proceedings of the WWW-2005 Workshop on the
        Weblogging Ecosystem (2005). :DOI:`10.1145/1134271.1134277`
+
     """
-    if vprop == None:
+    if vprop is None:
         vprop = g.new_vertex_property("double")
-    if eprop == None:
+    if eprop is None:
         eprop = g.new_edge_property("double")
-    if weight != None and weight.value_type() != eprop.value_type():
+    if weight is not None and weight.value_type() != eprop.value_type():
         nw = g.new_edge_property(eprop.value_type())
         g.copy_property(weight, nw)
         weight = nw
+    if pivots is not None:
+        pivots = numpy.asarray(pivots, dtype="uint64")
+    else:
+        pivots = g.get_vertices()
+    vpivots = Vector_size_t(len(pivots))
+    vpivots.a = pivots
     libgraph_tool_centrality.\
-            get_betweenness(g._Graph__graph, _prop("e", g, weight),
+            get_betweenness(g._Graph__graph, vpivots, _prop("e", g, weight),
                             _prop("e", g, eprop), _prop("v", g, vprop), norm)
     return vprop, eprop
 
@@ -397,7 +415,7 @@ def closeness(g, weight=None, source=None, vprop=None, norm=True, harmonic=False
     :math:`V-1`.
 
     The algorithm complexity of :math:`O(V(V + E))` for unweighted graphs and
-    :math:`O(V(v+E) \log V)` for weighted graphs. If the option ``source`` is
+    :math:`O(V(V+E) \log V)` for weighted graphs. If the option ``source`` is
     specified, this drops to :math:`O(V + E)` and :math:`O((V+E)\log V)`
     respectively.
 
@@ -447,7 +465,7 @@ def closeness(g, weight=None, source=None, vprop=None, norm=True, harmonic=False
 
     """
     if source is None:
-        if vprop == None:
+        if vprop is None:
             vprop = g.new_vertex_property("double")
         libgraph_tool_centrality.\
             closeness(g._Graph__graph, _prop("e", g, weight),
@@ -1005,7 +1023,7 @@ def eigentrust(g, trust_map, vprop=None, norm=False, epsilon=1e-6, max_iter=0,
        Weblogging Ecosystem (2005). :DOI:`10.1145/1134271.1134277`
     """
 
-    if vprop == None:
+    if vprop is None:
         vprop = g.new_vertex_property("double")
     i = libgraph_tool_centrality.\
            get_eigentrust(g._Graph__graph, _prop("e", g, trust_map),
@@ -1076,7 +1094,7 @@ def trust_transitivity(g, trust_map, source=None, target=None, vprop=None):
        w_G(i\to j) = \prod_{e\in i\to j} c_e.
 
     The algorithm measures the transitive trust by finding the paths with
-    maximum weight, using Dijkstra's algorithm, to all in-neighbours of a given
+    maximum weight, using Dijkstra's algorithm, to all in-neighbors of a given
     target. This search needs to be performed repeatedly for every target, since
     it needs to be removed from the graph first. For each given source, the
     resulting complexity is therefore :math:`O(V^2\log V)` for all targets, and
@@ -1138,15 +1156,15 @@ def trust_transitivity(g, trust_map, source=None, target=None, vprop=None):
 
     """
 
-    if vprop == None:
+    if vprop is None:
         vprop = g.new_vertex_property("vector<double>")
 
-    if target == None:
+    if target is None:
         target = -1
     else:
         target = g.vertex_index[target]
 
-    if source == None:
+    if source is None:
         source = -1
     else:
         source = g.vertex_index[source]
