@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -99,7 +99,7 @@ def local_clustering(g, prop=None, undirected=True):
     .. math::
        N_i = \{v_j : e_{ij} \in E\}
 
-    is the set of out-neighbours of vertex :math:`i`. For undirected graphs the
+    is the set of out-neighbors of vertex :math:`i`. For undirected graphs the
     value of :math:`c_i` is normalized as
 
     .. math::
@@ -121,7 +121,7 @@ def local_clustering(g, prop=None, undirected=True):
     >>> g = gt.random_graph(1000, lambda: (5,5))
     >>> clust = gt.local_clustering(g)
     >>> print(gt.vertex_average(g, clust))
-    (0.006177777777777778, 0.00036966184079941211)
+    (0.008177777777777779, 0.00042080229075093...)
 
     References
     ----------
@@ -130,10 +130,10 @@ def local_clustering(g, prop=None, undirected=True):
        :doi:`10.1038/30918`
     """
 
-    if prop == None:
+    if prop is None:
         prop = g.new_vertex_property("double")
     if g.is_directed() and undirected:
-        g = GraphView(g, directed=False)
+        g = GraphView(g, directed=False, skip_properties=True)
     _gt.local_clustering(g._Graph__graph, _prop("v", g, prop))
     return prop
 
@@ -182,7 +182,7 @@ def global_clustering(g):
 
     >>> g = gt.random_graph(1000, lambda: (5,5))
     >>> print(gt.global_clustering(g))
-    (0.006177777777777778, 0.0003700318726720...)
+    (0.008177777777777779, 0.0004212235142651...)
 
     References
     ----------
@@ -191,6 +191,8 @@ def global_clustering(g):
        :doi:`10.1137/S003614450342480`
     """
 
+    if g.is_directed():
+        g = GraphView(g, directed=False, skip_properties=True)
     c = _gt.global_clustering(g._Graph__graph)
     return c
 
@@ -238,7 +240,7 @@ def extended_clustering(g, props=None, max_depth=3, undirected=False):
     .. math::
        N_i = \{v_j : e_{ij} \in E\}
 
-    is the set of out-neighbours of :math:`i`. According to the above
+    is the set of out-neighbors of :math:`i`. According to the above
     definition, we have that the traditional local clustering coefficient is
     recovered for :math:`d=1`, i.e., :math:`c^1_i = c_i`.
 
@@ -261,11 +263,11 @@ def extended_clustering(g, props=None, max_depth=3, undirected=False):
     >>> for i in range(0, 5):
     ...    print(gt.vertex_average(g, clusts[i]))
     ...
-    (0.00421, 0.00041685103920811...)
-    (0.027226666666666666, 0.00100735228307788...)
-    (0.11549166666666667, 0.00208139908541773...)
-    (0.4128066666666666, 0.00294792212319872...)
-    (0.4205716666666667, 0.00315646574114149...)
+    (0.0050483333333333335, 0.0004393940240073...)
+    (0.024593787878787878, 0.0009963004021144...)
+    (0.11238924242424242, 0.001909615401971...)
+    (0.40252272727272725, 0.003113987400030...)
+    (0.43629378787878786, 0.003144159256565...)
 
     References
     ----------
@@ -275,7 +277,7 @@ def extended_clustering(g, props=None, max_depth=3, undirected=False):
 
     if g.is_directed() and undirected:
         g = GraphView(g, directed=False)
-    if props == None:
+    if props is None:
         props = []
         for i in range(0, max_depth):
             props.append(g.new_vertex_property("double"))
@@ -347,10 +349,9 @@ def motifs(g, k, p=1.0, motif_list=None, return_maps=False):
     >>> g = gt.random_graph(1000, lambda: (5,5))
     >>> motifs, counts = gt.motifs(gt.GraphView(g, directed=False), 4)
     >>> print(len(motifs))
-    13
+    18
     >>> print(counts)
-    [115408, 388542, 1031, 1182, 2662, 2138, 833, 28, 16, 5, 5, 3, 4]
-
+    [115557, 390005, 627, 700, 1681, 2815, 820, 12, 27, 44, 15, 7, 12, 4, 6, 1, 2, 1]
 
     References
     ----------
@@ -370,7 +371,7 @@ def motifs(g, k, p=1.0, motif_list=None, return_maps=False):
             if m.is_directed() != directed_motifs:
                 raise ValueError("all motif graphs must be either directed or undirected!")
             if m.num_vertices() != k:
-                raise ValueError("all motifs must have the same number of vertices: " + k)
+                raise ValueError("all motifs must have the same number of vertices: %d" % k)
             sub_list.append(m._Graph__graph)
 
     if directed_motifs != g.is_directed():
@@ -427,7 +428,7 @@ def _graph_sig(g):
     """return the graph signature, i.e., the in and out degree distribution as
     concatenated as a tuple."""
     bins = list(range(0, g.num_vertices() + 1))
-    in_dist = vertex_hist(g, "in", bins=bins if g.is_directed() else [0],
+    in_dist = vertex_hist(g, "in", bins=bins if g.is_directed() else [0, 1],
                           float_count=False)
     out_dist = vertex_hist(g, "out", bins=bins, float_count=False)
     sig = tuple([(in_dist[1][i], in_dist[0][i]) for \
@@ -439,7 +440,7 @@ def _graph_sig(g):
 
 def motif_significance(g, k, n_shuffles=100, p=1.0, motif_list=None,
                        threshold=0, self_loops=False, parallel_edges=False,
-                       full_output=False, shuffle_model="uncorrelated"):
+                       full_output=False, shuffle_model="configuration"):
     r"""
     Obtain the motif significance profile, for subgraphs with k vertices. A
     tuple with two lists is returned: the list of motifs found, and their
@@ -473,7 +474,7 @@ def motif_significance(g, k, n_shuffles=100, p=1.0, motif_list=None,
         of each motif, the average count of each motif in the shuffled networks,
         and the standard deviation of the average count of each motif in the
         shuffled networks.
-    shuffle_model : string (optional, default: "uncorrelated")
+    shuffle_model : string (optional, default: "configuration")
         Shuffle model to use. See :func:`~graph_tool.generation.random_rewire`
         for details.
 
@@ -521,7 +522,14 @@ def motif_significance(g, k, n_shuffles=100, p=1.0, motif_list=None,
     >>> print(len(motifs))
     11
     >>> print(zscores)
-    [0.04162670293683441, 0.046769781223679897, 0.55888261369689918, 0.82906624302416043, -0.41384527710386287, -0.42070845824900477, -1.3262411347517733, 1.82, -0.13, -0.37, -0.22]
+    [0.22728646681107012, 0.21409572051644973, 0.007022040788902111, 0.5872141967123348, -0.37770179603294357, -0.3484733504783734, 0.8861811801325502, -0.08, -0.2, -0.38, -0.2]
+
+    References
+    ----------
+    .. [wernicke-efficient-2006] S. Wernicke, "Efficient detection of network
+       motifs", IEEE/ACM Transactions on Computational Biology and
+       Bioinformatics (TCBB), Volume 3, Issue 4, Pages 347-359, 2006.
+       :doi:`10.1109/TCBB.2006.51`
     """
 
     s_ms, counts = motifs(g, k, p, motif_list)
