@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@ Summary
    scalar_assortativity
    corr_hist
    combined_corr_hist
-   avg_neighbour_corr
+   avg_neighbor_corr
    avg_combined_corr
 
 Contents
@@ -48,7 +48,8 @@ from .. import _degree, _prop
 from numpy import *
 
 __all__ = ["assortativity", "scalar_assortativity", "corr_hist",
-           "combined_corr_hist", "avg_neighbour_corr", "avg_combined_corr"]
+           "combined_corr_hist", "avg_neighbor_corr", "avg_neighbour_corr",
+           "avg_combined_corr"]
 
 
 def assortativity(g, deg):
@@ -74,7 +75,7 @@ def assortativity(g, deg):
     scalar_assortativity: scalar assortativity coefficient
     corr_hist: vertex-vertex correlation histogram
     combined_corr_hist: combined single-vertex correlation histogram
-    avg_neighbour_corr: average nearest-neighbour correlation
+    avg_neighbor_corr: average nearest-neighbor correlation
     avg_combined_corr: average combined single-vertex correlation
 
     Notes
@@ -103,7 +104,7 @@ def assortativity(g, deg):
        >>> g = gt.collection.data["pgp-strong-2009"]
        >>> g = gt.GraphView(g, directed=False)
        >>> gt.assortativity(g, "out")
-       (0.033159070186109..., 0.00032590407570...)
+       (0.0240578552..., 0.0003272847...)
 
     References
     ----------
@@ -139,7 +140,7 @@ def scalar_assortativity(g, deg):
     scalar_assortativity: scalar assortativity coefficient
     corr_hist: vertex-vertex correlation histogram
     combined_corr_hist: combined single-vertex correlation histogram
-    avg_neighbour_corr: average nearest-neighbour correlation
+    avg_neighbor_corr: average nearest-neighbor correlation
     avg_combined_corr: average combined single-vertex correlation
 
     Notes
@@ -167,7 +168,7 @@ def scalar_assortativity(g, deg):
        >>> g = gt.collection.data["pgp-strong-2009"]
        >>> g = gt.GraphView(g, directed=False)
        >>> gt.scalar_assortativity(g, "out")
-       (0.0263849324180816..., 0.00129447062420...)
+       (0.026384932418..., 0.001213444270...)
 
     References
     ----------
@@ -223,13 +224,13 @@ def corr_hist(g, deg_source, deg_target, bins=[[0, 1], [0, 1]], weight=None,
     scalar_assortativity: scalar assortativity coefficient
     corr_hist: vertex-vertex correlation histogram
     combined_corr_hist: combined single-vertex correlation histogram
-    avg_neighbour_corr: average nearest-neighbour correlation
+    avg_neighbor_corr: average nearest-neighbor correlation
     avg_combined_corr: average combined single-vertex correlation
 
     Notes
     -----
     The correlation histogram counts, for every vertex with degree (or scalar
-    property) 'source_deg', the number of out-neighbours with degree (or scalar
+    property) 'source_deg', the number of out-neighbors with degree (or scalar
     property) 'target_deg'.
 
     If enabled during compilation, this algorithm runs in parallel.
@@ -252,25 +253,26 @@ def corr_hist(g, deg_source, deg_target, bins=[[0, 1], [0, 1]], weight=None,
        ...         accept = np.random.random() < 1.0/k
        ...     return k
        ...
-       >>> g = gt.random_graph(10000, lambda: sample_k(40), model="probabilistic",
-       ...                     vertex_corr=lambda i, j: (sin(i / pi) * sin(j / pi) + 1) / 2,
+       >>> g = gt.random_graph(10000, lambda: sample_k(40),
+       ...                     model="probabilistic-configuration",
+       ...                     edge_probs=lambda i, j: (sin(i / pi) * sin(j / pi) + 1) / 2,
        ...                     directed=False, n_iter=100)
        >>> h = gt.corr_hist(g, "out", "out")
        >>> clf()
        >>> xlabel("Source out-degree")
-       <...>
+       Text(...)
        >>> ylabel("Target out-degree")
-       <...>
+       Text(...)
        >>> imshow(h[0].T, interpolation="nearest", origin="lower")
        <...>
        >>> colorbar()
        <...>
-       >>> savefig("corr.pdf")
+       >>> savefig("corr.svg")
 
     .. testcode:: corr_hist
        :hide:
 
-       savefig("corr.png")
+       savefig("corr.pdf")
 
     .. figure:: corr.*
         :align: center
@@ -329,7 +331,7 @@ def combined_corr_hist(g, deg1, deg2, bins=[[0, 1], [0, 1]], float_count=True):
     scalar_assortativity: scalar assortativity coefficient
     corr_hist: vertex-vertex correlation histogram
     combined_corr_hist: combined single-vertex correlation histogram
-    avg_neighbour_corr: average nearest-neighbour correlation
+    avg_neighbor_corr: average nearest-neighbor correlation
     avg_combined_corr: average combined single-vertex correlation
 
     Examples
@@ -355,19 +357,19 @@ def combined_corr_hist(g, deg1, deg2, bins=[[0, 1], [0, 1]], float_count=True):
        >>> h = gt.combined_corr_hist(g, "in", "out")
        >>> clf()
        >>> xlabel("In-degree")
-       <...>
+       Text(...)
        >>> ylabel("Out-degree")
-       <...>
+       Text(...)
        >>> imshow(h[0].T, interpolation="nearest", origin="lower")
        <...>
        >>> colorbar()
        <...>
-       >>> savefig("combined_corr.pdf")
+       >>> savefig("combined_corr.svg")
 
     .. testcode:: combined_corr_hist
        :hide:
 
-       savefig("combined_corr.png")
+       savefig("combined_corr.pdf")
 
     .. figure:: combined_corr.*
         :align: center
@@ -385,9 +387,9 @@ def combined_corr_hist(g, deg1, deg2, bins=[[0, 1], [0, 1]], float_count=True):
             [ret[1][0], ret[1][1]]]
 
 
-def avg_neighbour_corr(g, deg_source, deg_target, bins=[0, 1], weight=None):
+def avg_neighbor_corr(g, deg_source, deg_target, bins=[0, 1], weight=None):
     r"""
-    Obtain the average neighbour-neighbour correlation for the given graph.
+    Obtain the average neighbor-neighbor correlation for the given graph.
 
     Parameters
     ----------
@@ -423,28 +425,28 @@ def avg_neighbour_corr(g, deg_source, deg_target, bins=[0, 1], weight=None):
     scalar_assortativity: scalar assortativity coefficient
     corr_hist: vertex-vertex correlation histogram
     combined_corr_hist: combined single-vertex correlation histogram
-    avg_neighbour_corr: average nearest-neighbour correlation
+    avg_neighbor_corr: average nearest-neighbor correlation
     avg_combined_corr: average combined single-vertex correlation
 
     Notes
     -----
     The average correlation is the average, for every vertex with degree (or
     scalar property) 'source_deg', the of the 'target_deg' degree (or
-    scalar property) of its neighbours.
+    scalar property) of its neighbors.
 
     If enabled during compilation, this algorithm runs in parallel.
 
     Examples
     --------
 
-    .. testsetup:: avg_neighbour_corr
+    .. testsetup:: avg_neighbor_corr
 
        from pylab import *
        np.random.seed(42)
        gt.seed_rng(42)
 
 
-    .. doctest:: avg_neighbour_corr
+    .. doctest:: avg_neighbor_corr
 
        >>> def sample_k(max):
        ...     accept = False
@@ -453,23 +455,24 @@ def avg_neighbour_corr(g, deg_source, deg_target, bins=[0, 1], weight=None):
        ...         accept = np.random.random() < 1.0 / k
        ...     return k
        ...
-       >>> g = gt.random_graph(10000, lambda: sample_k(40), model="probabilistic",
-       ...                     vertex_corr=lambda i, j: (sin(i / pi) * sin(j / pi) + 1) / 2,
+       >>> g = gt.random_graph(10000, lambda: sample_k(40),
+       ...                     model="probabilistic-configuration",
+       ...                     edge_probs=lambda i, j: (sin(i / pi) * sin(j / pi) + 1) / 2,
        ...                     directed=False, n_iter=100)
-       >>> h = gt.avg_neighbour_corr(g, "out", "out")
+       >>> h = gt.avg_neighbor_corr(g, "out", "out")
        >>> clf()
        >>> xlabel("Source out-degree")
-       <...>
+       Text(...)
        >>> ylabel("Target out-degree")
-       <...>
+       Text(...)
        >>> errorbar(h[2][:-1], h[0], yerr=h[1], fmt="o")
        <...>
-       >>> savefig("avg_corr.pdf")
+       >>> savefig("avg_corr.svg")
 
-    .. testcode:: avg_neighbour_corr
+    .. testcode:: avg_neighbor_corr
        :hide:
 
-       savefig("avg_corr.png")
+       savefig("avg_corr.pdf")
 
     .. figure:: avg_corr.*
         :align: center
@@ -483,6 +486,7 @@ def avg_neighbour_corr(g, deg_source, deg_target, bins=[0, 1], weight=None):
                                  [float(x) for x in bins])
     return [ret[0], ret[1], ret[2][0]]
 
+avg_neighbour_corr = avg_neighbor_corr
 
 def avg_combined_corr(g, deg1, deg2, bins=[0, 1]):
     r"""
@@ -521,7 +525,7 @@ def avg_combined_corr(g, deg1, deg2, bins=[0, 1]):
     scalar_assortativity: scalar assortativity coefficient
     corr_hist: vertex-vertex correlation histogram
     combined_corr_hist: combined single-vertex correlation histogram
-    avg_neighbour_corr: average nearest-neighbour correlation
+    avg_neighbor_corr: average nearest-neighbor correlation
     avg_combined_corr: average combined single-vertex correlation
 
     Examples
@@ -548,17 +552,17 @@ def avg_combined_corr(g, deg1, deg2, bins=[0, 1]):
        >>> h = gt.avg_combined_corr(g, "in", "out")
        >>> clf()
        >>> xlabel("In-degree")
-       <...>
+       Text(...)
        >>> ylabel("Out-degree")
-       <...>
+       Text(...)
        >>> errorbar(h[2][:-1], h[0], yerr=h[1], fmt="o")
        <...>
-       >>> savefig("combined_avg_corr.pdf")
+       >>> savefig("combined_avg_corr.svg")
 
     .. testcode:: avg_combined_corr
        :hide:
 
-       savefig("combined_avg_corr.png")
+       savefig("combined_avg_corr.pdf")
 
     .. figure:: combined_avg_corr.*
         :align: center
