@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,17 +24,21 @@ import sys
 import os.path
 
 try:
-    from DLFCN import RTLD_LAZY, RTLD_GLOBAL
+    from os import RTLD_LAZY, RTLD_GLOBAL
     dl_flags = RTLD_LAZY | RTLD_GLOBAL
 except ImportError:
-    # handle strange python installations, by importing from the deprecated dl
-    # module, otherwise from ctypes
     try:
-        from dl import RTLD_LAZY, RTLD_GLOBAL
+        from DLFCN import RTLD_LAZY, RTLD_GLOBAL
         dl_flags = RTLD_LAZY | RTLD_GLOBAL
     except ImportError:
-        from ctypes import RTLD_GLOBAL
-        dl_flags = RTLD_GLOBAL
+        # handle strange python installations, by importing from the deprecated dl
+        # module, otherwise from ctypes
+        try:
+            from dl import RTLD_LAZY, RTLD_GLOBAL
+            dl_flags = RTLD_LAZY | RTLD_GLOBAL
+        except ImportError:
+            from ctypes import RTLD_GLOBAL
+            dl_flags = RTLD_GLOBAL
 
 __all__ = ["dl_import"]
 
