@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@ Layout algorithms
    fruchterman_reingold_layout
    arf_layout
    radial_tree_layout
+   planar_layout
    random_layout
    get_hierarchy_control_points
 
@@ -67,10 +68,11 @@ Contents
 
 from __future__ import division, absolute_import, print_function
 
-from .. import GraphView, _check_prop_vector, group_vector_property, \
+from .. import Graph, GraphView, _check_prop_vector, group_vector_property, \
      ungroup_vector_property, infect_vertex_property, _prop, _get_rng
 from .. topology import max_cardinality_matching, max_independent_vertex_set, \
-    label_components, pseudo_diameter, shortest_distance
+    label_components, pseudo_diameter, shortest_distance, make_maximal_planar, \
+    is_planar
 from .. stats import label_parallel_edges
 from .. generation import predecessor_tree, condensation_graph
 import numpy.random
@@ -82,9 +84,9 @@ dl_import("from . import libgraph_tool_layout")
 
 
 __all__ = ["graph_draw", "graphviz_draw", "fruchterman_reingold_layout",
-           "arf_layout", "sfdp_layout", "random_layout", "radial_tree_layout",
-           "cairo_draw", "prop_to_size", "get_hierarchy_control_points",
-           "default_cm"]
+           "arf_layout", "sfdp_layout", "planar_layout", "random_layout",
+           "radial_tree_layout", "cairo_draw", "prop_to_size",
+           "get_hierarchy_control_points", "default_cm"]
 
 
 def random_layout(g, shape=None, pos=None, dim=2):
@@ -126,17 +128,17 @@ def random_layout(g, shape=None, pos=None, dim=2):
     >>> shape = [[50, 100], [1, 2], 4]
     >>> pos = gt.random_layout(g, shape=shape, dim=3)
     >>> pos[g.vertex(0)].a
-    array([ 68.72700594,   1.03142919,   2.56812658])
+    array([68.72700594,  1.03142919,  2.56812658])
 
     """
 
-    if pos == None:
+    if pos is None:
         pos = g.new_vertex_property("vector<double>")
     _check_prop_vector(pos, name="pos")
 
     pos = ungroup_vector_property(pos, list(range(0, dim)))
 
-    if shape == None:
+    if shape is None:
         shape = [sqrt(g.num_vertices())] * dim
 
     for i in range(dim):
@@ -156,6 +158,68 @@ def random_layout(g, shape=None, pos=None, dim=2):
     return pos
 
 
+def planar_layout(g, pos=None):
+    r"""Performs a canonical layout of a planar graph.
+
+    Parameters
+    ----------
+    g : :class:`~graph_tool.Graph`
+        Planar graph to be used.
+    pos : :class:`~graph_tool.PropertyMap` (optional, default: ``None``)
+        Vector vertex property maps where the coordinates should be stored.
+
+    Returns
+    -------
+    pos : :class:`~graph_tool.PropertyMap`
+        A vector-valued vertex property map with the coordinates of the
+        vertices.
+
+    Notes
+    -----
+    This algorithm has complexity :math:`O(V + E)`.
+
+    Examples
+    --------
+    >>> g = gt.lattice([10, 10])
+    >>> pos = gt.planar_layout(g)
+    >>> gt.graph_draw(g, pos=pos, output="lattice-planar.pdf")
+    <...>
+
+    .. testcode::
+       :hide:
+
+       gt.graph_draw(g, pos=pos, output="lattice-planar.png")
+
+    .. figure:: lattice-planar.*
+        :align: center
+
+        Straight-line drawing of planar graph (a 2D square lattice).
+
+    References
+    ----------
+    .. [straight-line-boost] http://www.boost.org/doc/libs/release/libs/graph/doc/straight_line_drawing.html
+    .. [chrobak-linear-1995] M. Chrobak, T. Payne, "A Linear-time Algorithm for
+       Drawing a Planar Graph on the Grid", Information Processing Letters 54:
+       241-246, (1995), :doi:`10.1016/0020-0190(95)00020-D`
+    """
+
+    if g.num_vertices() < 3:
+        raise ValueError("Graph must have at least 3 vertices.")
+    if not is_planar(g):
+        raise ValueError("Graph is not planar.")
+    u = Graph(GraphView(g, directed=False, skip_properties=True))
+    make_maximal_planar(u)
+    embed = is_planar(u, embedding=True)[1]
+    if pos is None:
+        pos = u.new_vp("vector<double>")
+    make_maximal_planar(u)
+    libgraph_tool_layout.planar_layout(u._Graph__graph,
+                                       _prop("v", u, embed),
+                                       _prop("v", u, pos))
+    pos = g.own_property(pos)
+    return pos
+
+
 def fruchterman_reingold_layout(g, weight=None, a=None, r=1., scale=None,
                                 circular=False, grid=True, t_range=None,
                                 n_iter=100, pos=None):
@@ -165,7 +229,7 @@ def fruchterman_reingold_layout(g, weight=None, a=None, r=1., scale=None,
     ----------
     g : :class:`~graph_tool.Graph`
         Graph to be used.
-    weight : :class:`PropertyMap` (optional, default: ``None``)
+    weight : :class:`~graph_tool.PropertyMap` (optional, default: ``None``)
         An edge property map with the respective weights.
     a : float (optional, default: :math:`V`)
         Attracting force between adjacent vertices.
@@ -184,7 +248,7 @@ def fruchterman_reingold_layout(g, weight=None, a=None, r=1., scale=None,
         displacement at each iteration.
     n_iter : int (optional, default: ``100``)
         Total number of iterations.
-    pos : :class:`PropertyMap` (optional, default: ``None``)
+    pos : :class:`~graph_tool.PropertyMap` (optional, default: ``None``)
         Vector vertex property maps where the coordinates should be stored. If
         provided, this will also be used as the initial position of the
         vertices.
@@ -231,7 +295,7 @@ def fruchterman_reingold_layout(g, weight=None, a=None, r=1., scale=None,
        (Wiley) 21 (11): 1129-1164. (1991) :doi:`10.1002/spe.4380211102`
     """
 
-    if pos == None:
+    if pos is None:
         pos = random_layout(g, dim=2)
     _check_prop_vector(pos, name="pos", floating=True)
 
@@ -622,6 +686,7 @@ def sfdp_layout(g, vweight=None, eweight=None, pin=None, groups=None, C=0.2,
         pos = random_layout(g, dim=2)
     _check_prop_vector(pos, name="pos", floating=True)
 
+    g_ = g
     g = GraphView(g, directed=False)
 
     if pin is not None:
@@ -660,7 +725,6 @@ def sfdp_layout(g, vweight=None, eweight=None, pin=None, groups=None, C=0.2,
                 print("Positioning level:", count, u.num_vertices(), end=' ')
                 print("with K =", K, "...")
                 count += 1
-            #graph_draw(u, pos)
             pos = sfdp_layout(u, pos=pos,
                               vweight=vcount if weighted_coarse else None,
                               eweight=ecount if weighted_coarse else None,
@@ -675,7 +739,7 @@ def sfdp_layout(g, vweight=None, eweight=None, pin=None, groups=None, C=0.2,
                               #               _avg_edge_distance(u, pos)),
                               multilevel=False,
                               verbose=False)
-            #graph_draw(u, pos)
+        pos = g_.own_property(pos)
         return pos
 
     if g.num_vertices() <= 1:
@@ -700,10 +764,11 @@ def sfdp_layout(g, vweight=None, eweight=None, pin=None, groups=None, C=0.2,
                                      theta, init_step, cooling_step, max_level,
                                      epsilon, max_iter, not adaptive_cooling,
                                      verbose, _get_rng())
+    pos = g_.own_property(pos)
     return pos
 
-def radial_tree_layout(g, root, rel_order=None, weighted=False,
-                       node_weight=None, r=1.):
+def radial_tree_layout(g, root, rel_order=None, rel_order_leaf=False,
+                       weighted=False, node_weight=None, r=1.):
     r"""Computes a radial layout of the graph according to the minimum spanning
     tree centered at the ``root`` vertex.
 
@@ -715,6 +780,9 @@ def radial_tree_layout(g, root, rel_order=None, weighted=False,
         The root of the radial tree.
     rel_order : :class:`~graph_tool.PropertyMap` (optional, default: ``None``)
         Relative order of the nodes at each respective branch.
+    rel_order_leaf : ``bool`` (optional, default: ``False``)
+        If ``True``, the relative order of the leafs will propagate to the
+        root. Otherwise they will propagate in the opposite direction.
     weighted : ``bool`` (optional, default: ``False``)
         If true, the angle between the child branches will be computed according
         to weight of the entire sub-branches.
@@ -732,7 +800,8 @@ def radial_tree_layout(g, root, rel_order=None, weighted=False,
 
     Notes
     -----
-    This algorithm has complexity :math:`O(V + E)`.
+    This algorithm has complexity :math:`O(V + E)`, or :math:`O(V\log V + E)` if
+    ``rel_order`` is given.
 
     Examples
     --------
@@ -771,11 +840,12 @@ def radial_tree_layout(g, root, rel_order=None, weighted=False,
     elif node_weight.value_type() != "double":
         node_weight = node_weight.copy("double")
     libgraph_tool_layout.get_radial(t._Graph__graph,
-                                    _prop("v", g, pos),
-                                    _prop("v", g, levels),
+                                    _prop("v", t, pos),
+                                    _prop("v", t, levels),
                                     _prop("v", g, rel_order),
                                     _prop("v", g, node_weight),
-                                    int(root), weighted, r)
+                                    int(root), weighted, r,
+                                    rel_order_leaf)
     return g.own_property(pos)
 
 def prop_to_size(prop, mi=0, ma=5, log=False, power=0.5):
