@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -224,11 +224,11 @@ def graphviz_draw(g, pos=None, size=(15, 15), pin=False, layout=None,
         Drawing color for edges. If the valued supplied is a property map,
         the values must be scalar types, whose color values are obtained from
         the ``ecmap`` argument.
-    vcmap : :class:`matplotlib.colors.Colormap` (default: :class:`matplotlib.cm.jet`)
+    vcmap : :class:`matplotlib.colors.Colormap` (default: :obj:`matplotlib.cm.jet`)
         Vertex color map.
     vnorm : bool (default: ``True``)
         Normalize vertex color values to the [0,1] range.
-    ecmap : :class:`matplotlib.colors.Colormap` (default: :class:`matplotlib.cm.jet`)
+    ecmap : :class:`matplotlib.colors.Colormap` (default: :obj:`matplotlib.cm.jet`)
         Edge color map.
     enorm : bool (default: ``True``)
         Normalize edge color values to the [0,1] range.
@@ -299,7 +299,7 @@ def graphviz_draw(g, pos=None, size=(15, 15), pin=False, layout=None,
 
         Kamada-Kawai force-directed layout of a Price network with 1500
         nodes. The vertex size and color indicate the degree, and the edge color
-        corresponds to the edge betweeness centrality
+        corresponds to the edge betweenness centrality
 
     References
     ----------
@@ -487,6 +487,9 @@ def graphviz_draw(g, pos=None, size=(15, 15), pin=False, layout=None,
             aset(ge, "arrowsize", "0.3")
             if g.is_directed():
                 aset(ge, "arrowhead", "vee")
+                aset(ge, "dir", "forward")
+            else:
+                aset(ge, "dir", "none")
 
             # apply color
             if isinstance(ecolor, (str, unicode)):
@@ -524,7 +527,7 @@ def graphviz_draw(g, pos=None, size=(15, 15), pin=False, layout=None,
         has_layout = True
         retv = libgv.gvRender(gvc, gvg, "dot".encode("utf8"), None)  # retrieve positions only
 
-        if pos == None:
+        if pos is None:
             pos = (g.new_vertex_property("double"),
                    g.new_vertex_property("double"))
         for v in g.vertices():
