@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -54,8 +54,8 @@ class VertexMatrix(object):
 
     def get_box(self, p, size=None):
         if size is None:
-            return (int(round(p[0] / self.m_res)),
-                    int(round(p[1] / self.m_res)))
+            return (int(round(float(p[0] / self.m_res))),
+                    int(round(float(p[1] / self.m_res))))
         else:
             n = int(np.ceil(size / self.m_res))
             b = self.get_box(p)
@@ -133,7 +133,7 @@ class GraphWidget(Gtk.DrawingArea):
     def __init__(self, g, pos, vprops=None, eprops=None, vorder=None,
                  eorder=None, nodesfirst=False, update_layout=False,
                  layout_K=1., multilevel=False, display_props=None,
-                 display_props_size=11, fit_area=0.95, bg_color=None,
+                 display_props_size=11, fit_view=True, bg_color=None,
                  max_render_time=300, layout_callback=None,
                  key_press_callback=None, highlight_color=None, **kwargs):
         r"""Interactive GTK+ widget displaying a given graph.
@@ -159,7 +159,7 @@ class GraphWidget(Gtk.DrawingArea):
             If provided, defines the relative order in which the edges are drawn.
         nodesfirst : bool (optional, default: ``False``)
             If ``True``, the vertices are drawn first, otherwise the edges are.
-        update_layout : bool (optional, default: ``True``)
+        update_layout : bool (optional, default: ``False``)
             If ``True``, the layout will be updated dynamically.
         layout_K : float (optional, default: ``1.0``)
             Parameter ``K`` passed to :func:`~graph_tool.draw.sfdp_layout`.
@@ -169,8 +169,12 @@ class GraphWidget(Gtk.DrawingArea):
             List of properties to be displayed when the mouse passes over a vertex.
         display_props_size : float (optional, default: ``11.``)
             Font size used to display the vertex properties.
-        fit_area : float  (optional, default: ``.95``)
-            Fraction of the drawing area to fit the graph initially.
+        fit_view : bool, float or tuple (optional, default: ``True``)
+            If ``True``, the layout will be scaled to fit the entire clip region.
+            If a float value is given, it will be interpreted as ``True``, and in
+            addition the viewport will be scaled out by that factor. If a tuple
+            value is given, it should have four values ``(x, y, w, h)`` that
+            specify the view in user coordinates.
         bg_color : str or sequence (optional, default: ``None``)
             Background color. The default is white.
         max_render_time : int (optional, default: ``300``)
@@ -274,7 +278,7 @@ class GraphWidget(Gtk.DrawingArea):
         self.drag_begin = None
         self.moved_picked = False
         self.vertex_matrix = None
-        self.pad = fit_area
+        self.fit_view = fit_view
 
         self.display_prop = g.vertex_index if display_props is None \
                             else display_props
@@ -285,7 +289,7 @@ class GraphWidget(Gtk.DrawingArea):
         self.base_geometry = None
         self.background = None
         self.bg_color = bg_color if bg_color is not None else [1, 1, 1, 1]
-        self.regenerate_offset = 0
+        self.regenerate_generator = None
         self.regenerate_max_time = max_render_time
         self.max_render_time = max_render_time
         self.lazy_regenerate = False
@@ -308,8 +312,12 @@ class GraphWidget(Gtk.DrawingArea):
             self.g = self.cg
             self.pos = self.cpos
             self.layout_step = self.layout_K
+            self.selected = self.g.own_property(self.selected)
+            self.highlight = self.g.own_property(self.highlight)
+            self.sel_edge_filt = self.g.own_property(self.sel_edge_filt)
         else:
             self.cg = None
+
         if update_layout:
             self.reset_layout()
 
@@ -417,6 +425,9 @@ class GraphWidget(Gtk.DrawingArea):
                     self.layout_K *= 0.75
                     self.g = self.cg
                     self.pos = self.cpos
+                    self.selected = self.g.own_property(self.selected)
+                    self.highlight = self.g.own_property(self.highlight)
+                    self.sel_edge_filt = self.g.own_property(self.sel_edge_filt)
                     self.layout_step = max(self.layout_K,
                                            _avg_edge_distance(self.g,
                                                               self.pos) / 10)
@@ -432,6 +443,9 @@ class GraphWidget(Gtk.DrawingArea):
                 except StopIteration:
                     self.g = self.ag
                     self.pos = self.apos
+                    self.selected = self.g.own_property(self.selected)
+                    self.highlight = self.g.own_property(self.highlight)
+                    self.sel_edge_filt = self.g.own_property(self.sel_edge_filt)
                     self.g.copy_property(self.cpos, self.pos)
                     if self.vertex_matrix is not None:
                         self.vertex_matrix = VertexMatrix(self.g, self.pos)
@@ -452,7 +466,8 @@ class GraphWidget(Gtk.DrawingArea):
         r"""Redraw the graph surface."""
 
         if reset:
-            self.regenerate_offset = 0
+            self.regenerate_generator = None
+            self.regen_context = None
 
         geometry = [self.get_allocated_width() * 3,
                     self.get_allocated_height() * 3]
@@ -464,32 +479,42 @@ class GraphWidget(Gtk.DrawingArea):
             w = self.get_window()
             if w is None:
                 return False
+
+            self.regenerate_generator = None
+            self.regen_context = None
+
             self.base = w.create_similar_surface(cairo.CONTENT_COLOR_ALPHA,
                                                  *geometry)
             self.base_geometry = geometry
-            self.regenerate_offset = 0
 
             m = cairo.Matrix()
             m.translate(self.get_allocated_width(),
                         self.get_allocated_height())
-            self.smatrix = self.smatrix * m
-            self.tmatrix = self.tmatrix * self.smatrix
+            self.smatrix = self.smatrix.multiply(m)
+            self.tmatrix = self.tmatrix.multiply(self.smatrix)
             self.smatrix = cairo.Matrix()
             self.smatrix.translate(-self.get_allocated_width(),
                                    -self.get_allocated_height())
 
-        cr = cairo.Context(self.base)
-        if self.regenerate_offset == 0:
+        if self.regenerate_generator is None:
+            cr = cairo.Context(self.base)
             cr.set_source_rgba(*self.bg_color)
             cr.paint()
-        cr.set_matrix(self.tmatrix)
-        mtime = -1 if complete else self.regenerate_max_time
-        res = 5 * self.get_scale_factor()
-        count = cairo_draw(self.g, self.pos, cr, self.vprops, self.eprops,
-                           self.vorder, self.eorder, self.nodesfirst, res=res,
-                           render_offset=self.regenerate_offset,
-                           max_render_time=mtime, **self.kwargs)
-        self.regenerate_offset = count
+            cr.set_matrix(self.tmatrix)
+            mtime = -1 if complete else self.regenerate_max_time
+            res = 5 * self.get_scale_factor()
+            gen = cairo_draw(self.g, self.pos, cr, self.vprops, self.eprops,
+                             self.vorder, self.eorder, self.nodesfirst, res=res,
+                             max_render_time=mtime, **self.kwargs)
+            self.regenerate_generator = gen
+            self.regen_context = cr
+
+        if self.regenerate_generator is not None:
+            try:
+                next(self.regenerate_generator)
+            except StopIteration:
+                self.regenerate_generator = None
+                self.regen_context = None
         self.lazy_regenerate = False
 
     def draw(self, da, cr):
@@ -519,7 +544,7 @@ class GraphWidget(Gtk.DrawingArea):
              ul[1] > 0 or lr[1] < geometry[1]) or
             self.lazy_regenerate):
             self.regenerate_surface(reset=True)
-        elif self.regenerate_offset > 0:
+        elif self.regenerate_generator is not None:
             self.regenerate_surface()
 
         if self.background is None:
@@ -552,7 +577,7 @@ class GraphWidget(Gtk.DrawingArea):
         cr.restore()
 
         if self.picked is not None or self.picked is not False:
-            # draw immediate neighbourhood
+            # draw immediate neighborhood
             if self.selected.fa.sum() == 1:
                 vprops = dict(**self.vprops)
                 vprops["halo"] = self.highlight
@@ -596,7 +621,7 @@ class GraphWidget(Gtk.DrawingArea):
                     eprops["pen_width"] *= 1.1
 
                 cr.save()
-                cr.set_matrix(self.tmatrix * self.smatrix)
+                cr.set_matrix(self.tmatrix.multiply(self.smatrix))
                 cairo_draw(u, self.pos, cr, vprops, eprops, self.vorder,
                            self.eorder, self.nodesfirst)
                 cr.restore()
@@ -611,7 +636,7 @@ class GraphWidget(Gtk.DrawingArea):
                           efilt=self.sel_edge_filt)
 
             cr.save()
-            cr.set_matrix(self.tmatrix * self.smatrix)
+            cr.set_matrix(self.tmatrix.multiply(self.smatrix))
             cairo_draw(u, self.pos, cr, vprops, eprops, self.vorder,
                        self.eorder, self.nodesfirst)
             cr.restore()
@@ -626,7 +651,7 @@ class GraphWidget(Gtk.DrawingArea):
             cr.set_source_rgba(0, 0, 1, 0.3)
             cr.fill()
 
-        if self.regenerate_offset > 0:
+        if self.regenerate_generator is not None:
             icon = self.render_icon(Gtk.STOCK_EXECUTE, Gtk.IconSize.BUTTON)
             Gdk.cairo_set_source_pixbuf(cr, icon, 10, 10)
             cr.paint()
@@ -652,7 +677,7 @@ class GraphWidget(Gtk.DrawingArea):
             cr.set_source_rgba(0, 0, 0, 1.0)
             cr.show_text(txt)
 
-        if self.regenerate_offset > 0:
+        if self.regenerate_generator is not None:
             self.queue_draw()
         return False
 
@@ -666,7 +691,7 @@ class GraphWidget(Gtk.DrawingArea):
             if surface:
                 cr.set_matrix(self.smatrix)
             else:
-                cr.set_matrix(self.tmatrix * self.smatrix)
+                cr.set_matrix(self.tmatrix.multiply(self.smatrix))
         if dist:
             return cr.user_to_device_distance(pos[0], pos[1])
         else:
@@ -681,7 +706,7 @@ class GraphWidget(Gtk.DrawingArea):
             if surface:
                 cr.set_matrix(self.smatrix)
             else:
-                cr.set_matrix(self.tmatrix * self.smatrix)
+                cr.set_matrix(self.tmatrix.multiply(self.smatrix))
         if dist:
             return cr.device_to_user_distance(pos[0], pos[1])
         else:
@@ -690,11 +715,11 @@ class GraphWidget(Gtk.DrawingArea):
     def apply_transform(self):
         r"""Apply current transform matrix to vertex coordinates."""
         zoom = self.pos_from_device((1, 0), dist=True)[0]
-        apply_transforms(self.g, self.pos, self.smatrix * self.tmatrix)
+        apply_transforms(self.g, self.pos, self.smatrix.multiply(self.tmatrix))
         self.tmatrix = cairo.Matrix()
         self.tmatrix.scale(zoom, zoom)
         self.smatrix = cairo.Matrix()
-        apply_transforms(self.g, self.pos, self.smatrix * self.tmatrix)
+        apply_transforms(self.g, self.pos, self.smatrix.multiply(self.tmatrix))
         self.tmatrix = cairo.Matrix()
         self.tmatrix.scale(1. / zoom, 1. / zoom)
         if self.vertex_matrix is not None:
@@ -711,21 +736,30 @@ class GraphWidget(Gtk.DrawingArea):
             g = self.g
         pos = g.own_property(self.pos)
         cr = self.get_window().cairo_create()
-        offset, zoom = fit_to_view(g, pos, geometry,
-                                   self.vprops.get("size", 0),
-                                   self.vprops.get("pen_width", 0),
-                                   self.tmatrix * self.smatrix,
-                                   self.vprops.get("text", None),
-                                   self.vprops.get("font_family",
-                                                   _vdefaults["font_family"]),
-                                   self.vprops.get("font_size",
-                                                   _vdefaults["font_size"]),
-                                   self.pad,
-                                   cr)
+        if self.fit_view != False:
+            try:
+                x, y, w, h = self.fit_view
+                zoom = min(geometry[0] / w, geometry[1] / h)
+                offset = (x * zoom, y * zoom)
+            except TypeError:
+                pad = self.fit_view if self.fit_view != True else 0.95
+                offset, zoom = fit_to_view(g, pos, geometry,
+                                           self.vprops.get("size", 0),
+                                           self.vprops.get("pen_width", 0),
+                                           self.tmatrix.multiply(self.smatrix),
+                                           self.vprops.get("text", None),
+                                           self.vprops.get("font_family",
+                                                           _vdefaults["font_family"]),
+                                           self.vprops.get("font_size",
+                                                           _vdefaults["font_size"]),
+                                           pad,
+                                           cr)
+        else:
+            offset, zoom = (0,0), 1
         m = cairo.Matrix()
         m.translate(offset[0] + ox, offset[1] + oy)
         m.scale(zoom, zoom)
-        self.tmatrix = self.tmatrix * self.smatrix * m
+        self.tmatrix = self.tmatrix.multiply(self.smatrix.multiply(m))
         self.smatrix = cairo.Matrix()
         if ink:
             scale_ink(zoom, self.vprops, self.eprops)
