@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -154,7 +154,7 @@ def edmonds_karp_max_flow(g, source, target, capacity, residual=None):
         :align: center
 
         Edge flows obtained with the Edmonds-Karp algorithm. The source and
-        target are on the lower left and upper right corners, respectively. The
+        target are on the upper left and lower right corners, respectively. The
         edge flows are represented by the edge width.
 
 
@@ -170,7 +170,7 @@ def edmonds_karp_max_flow(g, source, target, capacity, residual=None):
     """
 
     _check_prop_scalar(capacity, "capacity")
-    if residual == None:
+    if residual is None:
         residual = g.new_edge_property(capacity.value_type())
     _check_prop_scalar(residual, "residual")
     _check_prop_writable(residual, "residual")
@@ -236,7 +236,7 @@ def push_relabel_max_flow(g, source, target, capacity, residual=None):
 
 
         Edge flows obtained with the push-relabel algorithm. The source and
-        target are on the lower left and upper right corners, respectively. The
+        target are on the upper left and lower right corners, respectively. The
         edge flows are represented by the edge width.
 
     References
@@ -247,7 +247,7 @@ def push_relabel_max_flow(g, source, target, capacity, residual=None):
     """
 
     _check_prop_scalar(capacity, "capacity")
-    if residual == None:
+    if residual is None:
         residual = g.new_edge_property(capacity.value_type())
     _check_prop_scalar(residual, "residual")
     _check_prop_writable(residual, "residual")
@@ -317,7 +317,7 @@ def boykov_kolmogorov_max_flow(g, source, target, capacity, residual=None):
         :align: center
 
         Edge flows obtained with the Boykov-Kolmogorov algorithm. The source and
-        target are on the lower left and upper right corners, respectively. The
+        target are on the upper left and lower right corners, respectively. The
         edge flows are represented by the edge width.
 
     References
@@ -333,7 +333,7 @@ def boykov_kolmogorov_max_flow(g, source, target, capacity, residual=None):
        :doi:`10.1109/TPAMI.2004.60`
     """
     _check_prop_scalar(capacity, "capacity")
-    if residual == None:
+    if residual is None:
         residual = g.new_edge_property(capacity.value_type())
     _check_prop_scalar(residual, "residual")
     _check_prop_writable(residual, "residual")
