@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,18 +27,19 @@ from .. import Vector_size_t, Vector_double
 
 import numpy
 from . util import *
+from . nested_blockmodel import NestedBlockState
 
-def mcmc_equilibrate(state, wait=10, nbreaks=2, max_niter=numpy.inf,
-                     force_niter=None, epsilon=0, gibbs=False,
-                     block_moves=False, mcmc_args={}, entropy_args={},
-                     history=False, callback=None, verbose=False):
+def mcmc_equilibrate(state, wait=1000, nbreaks=2, max_niter=numpy.inf,
+                     force_niter=None, epsilon=0, gibbs=False, multiflip=False,
+                     mcmc_args={}, entropy_args={}, history=False,
+                     callback=None, verbose=False):
     r"""Equilibrate a MCMC with a given starting state.
 
     Parameters
     ----------
-    state : Any state class (e.g. :class:`~graph_tool.inference.BlockState`)
+    state : Any state class (e.g. :class:`~graph_tool.inference.blockmodel.BlockState`)
         Initial state. This state will be modified during the algorithm.
-    wait : ``int`` (optional, default: ``10``)
+    wait : ``int`` (optional, default: ``1000``)
         Number of iterations to wait for a record-breaking event.
     nbreaks : ``int`` (optional, default: ``2``)
         Number of iteration intervals (of size ``wait``) without record-breaking
@@ -54,14 +55,15 @@ def mcmc_equilibrate(state, wait=10, nbreaks=2, max_niter=numpy.inf,
     gibbs : ``bool`` (optional, default: ``False``)
         If ``True``, each step will call ``state.gibbs_sweep`` instead of
         ``state.mcmc_sweep``.
-    block_moves : ``bool`` (optional, default: ``False``)
-        If ``True``, each iteration will be accompanied by a "block move", where
-        all vertices of the same group are moved simultaneously.
+    multiflip : ``bool`` (optional, default: ``False``)
+        If ``True``, each step will call ``state.multiflip_mcmc_sweep`` instead of
+        ``state.mcmc_sweep``.
     mcmc_args : ``dict`` (optional, default: ``{}``)
         Arguments to be passed to ``state.mcmc_sweep`` (or ``state.gibbs_sweep``).
     history : ``bool`` (optional, default: ``False``)
-        If ``True``, a list of tuples of the form ``(iteration, entropy)`` will
-        be kept and returned.
+        If ``True``, a list of tuples of the form ``(nattempts, nmoves,
+        entropy)`` will be kept and returned, where ``entropy`` is the current
+        entropy and ``nmoves`` is the number of vertices moved.
     callback : ``function`` (optional, default: ``None``)
         If given, this function will be called after each iteration. The
         function must accept the current state as an argument, and its return
@@ -82,19 +84,21 @@ def mcmc_equilibrate(state, wait=10, nbreaks=2, max_niter=numpy.inf,
     event.
 
     This function calls ``state.mcmc_sweep`` (or ``state.gibbs_sweep``) at each
-    iteration (e.g. :meth:`graph_tool.inference.BlockState.mcmc_sweep` and
-    :meth:`graph_tool.inference.BlockState.gibbs_sweep`), and keeps track of
+    iteration (e.g. :meth:`graph_tool.inference.blockmodel.BlockState.mcmc_sweep` and
+    :meth:`graph_tool.inference.blockmodel.BlockState.gibbs_sweep`), and keeps track of
     the value of ``state.entropy(**args)`` with ``args`` corresponding to
     ``mcmc_args["entropy_args"]``.
 
     Returns
     -------
 
-    history : list of tuples of the form ``(iteration, entropy)``
+    history : list of tuples of the form ``(nattempts, nmoves, entropy)``
         Summary of the MCMC run. This is returned only if ``history == True``.
     entropy : ``float``
         Current entropy value after run. This is returned only if ``history ==
         False``.
+    nattempts : ``int``
+        Number of node move attempts.
     nmoves : ``int``
         Number of node moves.
 
@@ -105,40 +109,30 @@ def mcmc_equilibrate(state, wait=10, nbreaks=2, max_niter=numpy.inf,
        greedy heuristic for the inference of stochastic block models", Phys.
        Rev. E 89, 012804 (2014), :doi:`10.1103/PhysRevE.89.012804`,
        :arxiv:`1310.4378`
+
     """
 
     count = 0
     break_count = 0
     niter = 0
     total_nmoves = 0
+    total_nattempts = 0
     S = state.entropy(**mcmc_args.get("entropy_args", {}))
     min_S = max_S = S
     m_eps = 1e-6
     hist = []
     while count < wait:
-        if not gibbs:
-            delta, nmoves = state.mcmc_sweep(**mcmc_args)
+        if gibbs:
+            delta, nattempts, nmoves = state.gibbs_sweep(**mcmc_args)
+        elif multiflip:
+            delta, nattempts, nmoves = state.multiflip_mcmc_sweep(**mcmc_args)
         else:
-            delta, nmoves = state.gibbs_sweep(**mcmc_args)
-
-        if block_moves:
-            bstate = state.get_block_state(vweight=True,
-                                           clabel=state.get_bclabel())
-            if not gibbs:
-                ret = bstate.mcmc_sweep(**mcmc_args)
-            else:
-                ret = bstate.gibbs_sweep(**mcmc_args)
-
-            b = state.b.copy()
-            pmap(b, bstate.b)
-            state.set_blocks(b)
-
-            delta += ret[0]
-            nmoves += ret[1]
+            delta, nattempts, nmoves = state.mcmc_sweep(**mcmc_args)
 
         S += delta
         niter += 1
         total_nmoves += nmoves
+        total_nattempts += nattempts
 
         if force_niter is not None:
             if niter >= force_niter:
@@ -176,7 +170,7 @@ def mcmc_equilibrate(state, wait=10, nbreaks=2, max_niter=numpy.inf,
                     str(extra) if len(extra) > 0 else ""))
 
         if history:
-            hist.append([S, nmoves] + extra)
+            hist.append(tuple([nattempts, nmoves, S] + extra))
 
         if niter >= max_niter:
             break
@@ -184,7 +178,7 @@ def mcmc_equilibrate(state, wait=10, nbreaks=2, max_niter=numpy.inf,
     if history:
         return hist
     else:
-        return (S, total_nmoves)
+        return (S, total_nattempts, total_nmoves)
 
 def mcmc_anneal(state, beta_range=(1., 10.), niter=100, history=False,
                 mcmc_equilibrate_args={}, verbose=False):
@@ -193,7 +187,7 @@ def mcmc_anneal(state, beta_range=(1., 10.), niter=100, history=False,
 
     Parameters
     ----------
-    state : Any state class (e.g. :class:`~graph_tool.inference.BlockState`)
+    state : Any state class (e.g. :class:`~graph_tool.inference.blockmodel.BlockState`)
         Initial state. This state will be modified during the algorithm.
     beta_range : ``tuple`` of two floats (optional, default: ``(1., 10.)``)
         Inverse temperature range.
@@ -201,9 +195,9 @@ def mcmc_anneal(state, beta_range=(1., 10.), niter=100, history=False,
         Number of steps (in logspace) from the starting temperature to the final
         one.
     history : ``bool`` (optional, default: ``False``)
-        If ``True``, a list of tuples of the form ``(iteration, beta, entropy)``
+        If ``True``, a list of tuples of the form ``(nattempts, nmoves, beta, entropy)``
     mcmc_equilibrate_args : ``dict`` (optional, default: ``{}``)
-        Arguments to be passed to :func:`~graph_tool.inference.mcmc_equilibrate`.
+        Arguments to be passed to :func:`~graph_tool.inference.mcmc.mcmc_equilibrate`.
     verbose : ``bool`` or ``tuple`` (optional, default: ``False``)
         If ``True``, progress information will be shown. Optionally, this
         accepts arguments of the type ``tuple`` of the form ``(level, prefix)``
@@ -220,17 +214,19 @@ def mcmc_anneal(state, beta_range=(1., 10.), niter=100, history=False,
     iterations.
 
     At each iteration, the function
-    :func:`~graph_tool.inference.mcmc_equilibrate` is called with the current
+    :func:`~graph_tool.inference.mcmc.mcmc_equilibrate` is called with the current
     value of `beta` (via the ``mcmc_args`` parameter).
 
     Returns
     -------
 
-    history : list of tuples of the form ``(iteration, beta, entropy)``
+    history : list of tuples of the form ``(nattempts, nmoves, beta, entropy)``
         Summary of the MCMC run. This is returned only if ``history == True``.
     entropy : ``float``
         Current entropy value after run. This is returned only if ``history ==
         False``.
+    nattempts : ``int``
+        Number of node move attempts.
     nmoves : ``int``
         Number of node moves.
 
@@ -244,35 +240,38 @@ def mcmc_anneal(state, beta_range=(1., 10.), niter=100, history=False,
     """
 
     beta = beta_range[0]
-    hist = ([], [], [])
+    hist = ([], [], [], [])
+    nattempts = 0
     nmoves = 0
-    speed = exp((log(beta_range[1]) - log(beta_range[0])) / nsteps)
+    speed = exp((log(beta_range[1]) - log(beta_range[0])) / niter)
     mcmc_args = mcmc_equilibrate_args.get("mcmc_args", {})
     while beta < beta_range[1] * speed:
         ret = mcmc_equilibrate(state,
-                               **overlay(mcmc_equilibrate_args,
-                                         mcmc_args=overlay(mcmc_args,
-                                                           beta=beta),
-                                         history=history,
-                                         verbose=verbose_push(verbose,
-                                                              ("β: %#8.6g  " %
-                                                               beta))))
+                               **dict(mcmc_equilibrate_args,
+                                      mcmc_args=dict(mcmc_args,
+                                                     beta=beta),
+                                      history=history,
+                                      verbose=verbose_push(verbose,
+                                                           ("β: %#8.6g  " %
+                                                            beta))))
         if history:
             ret = list(zip(*ret))
-            hist[0].extend([beta] * len(ret[0]))
-            hist[1].extend(ret[0])
-            hist[2].extend(ret[1])
+            hist[0].extend(ret[0])
+            hist[1].extend(ret[1])
+            hist[2].extend([beta] * len(ret[0]))
+            hist[3].extend(ret[2])
             S = ret[0][-1]
         else:
             S = ret[0]
-            nmoves += ret[1]
+            attempts += ret[1]
+            nmoves += ret[2]
 
         beta *= speed
 
     if history:
         return list(zip(hist))
     else:
-        return S, nmoves
+        return S, nattempts, nmoves
 
 
 def mcmc_multilevel(state, B, r=2, b_cache=None, anneal=False,
@@ -284,7 +283,7 @@ def mcmc_multilevel(state, B, r=2, b_cache=None, anneal=False,
 
     Parameters
     ----------
-    state : Any state class (e.g. :class:`~graph_tool.inference.BlockState`)
+    state : Any state class (e.g. :class:`~graph_tool.inference.blockmodel.BlockState`)
         Initial state. This state will **not** be modified during the algorithm.
     B : ``int``
         Desired model order (i.e. number of groups).
@@ -297,15 +296,15 @@ def mcmc_multilevel(state, B, r=2, b_cache=None, anneal=False,
         order. This dictionary will be modified during the algorithm.
     anneal : ``bool`` (optional, default: ``False``)
         If ``True``, the equilibration steps will use simulated annealing, by
-        calling :func:`~graph_tool.inference.mcmc_anneal`, instead of
-        :func:`~graph_tool.inference.mcmc_equilibrate`.
+        calling :func:`~graph_tool.inference.mcmc.mcmc_anneal`, instead of
+        :func:`~graph_tool.inference.mcmc.mcmc_equilibrate`.
     mcmc_equilibrate_args : ``dict`` (optional, default: ``{}``)
-        Arguments to be passed to :func:`~graph_tool.inference.mcmc_equilibrate`.
+        Arguments to be passed to :func:`~graph_tool.inference.mcmc.mcmc_equilibrate`.
     mcmc_anneal_args : ``dict`` (optional, default: ``{}``)
-        Arguments to be passed to :func:`~graph_tool.inference.mcmc_anneal`.
+        Arguments to be passed to :func:`~graph_tool.inference.mcmc.mcmc_anneal`.
     shrink_args : ``dict`` (optional, default: ``{}``)
         Arguments to be passed to ``state.shrink``
-        (e.g. :meth:`graph_tool.inference.BlockState.shrink`).
+        (e.g. :meth:`graph_tool.inference.blockmodel.BlockState.shrink`).
     verbose : ``bool`` or ``tuple`` (optional, default: ``False``)
         If ``True``, progress information will be shown. Optionally, this
         accepts arguments of the type ``tuple`` of the form ``(level, prefix)``
@@ -318,7 +317,7 @@ def mcmc_multilevel(state, B, r=2, b_cache=None, anneal=False,
 
     This algorithm alternates between equilibrating the MCMC state and reducing
     the state order (via calls to ``state.shrink``,
-    e.g. :meth:`graph_tool.inference.BlockState.shrink`).
+    e.g. :meth:`graph_tool.inference.blockmodel.BlockState.shrink`).
 
     This greatly reduces the changes of getting trapped in metastable states if
     the starting point if far away from equilibrium, as discussed in
@@ -342,32 +341,42 @@ def mcmc_multilevel(state, B, r=2, b_cache=None, anneal=False,
     if "mcmc_equilibrate_args" in anneal_args:
         raise ValueError("'mcmc_equilibrate_args' should be passed directly " +
                          "to mcmc_multilevel(), not via 'anneal_args'")
-    while state.B > B:
-        B_next = max(min(int(round(state.B / r)), state.B - 1), B)
+
+    mcmc_args = mcmc_equilibrate_args.get("mcmc_args", {})
+    if not mcmc_equilibrate_args.get("gibbs", False):
+        mcmc_args["d"] = 0
+    else:
+        mcmc_args["allow_new_group"] = False
+    mcmc_equilibrate_args["mcmc_args"] = mcmc_args
+
+    while state.get_nonempty_B() > B:
+        B_next = max(min(int(round(state.get_nonempty_B() / r)),
+                         state.get_nonempty_B() - 1), B)
 
         if b_cache is not None and B_next in b_cache:
-            state = b_cache[B_next]
             if check_verbose(verbose):
                 print(verbose_pad(verbose) +
-                      "shrinking %d -> %d (cached)" % (state.B, B_next))
+                      "shrinking %d -> %d (cached)" % (state.get_nonempty_B(),
+                                                       B_next))
+            state = b_cache[B_next][1]
             continue
 
         if check_verbose(verbose):
             print(verbose_pad(verbose) +
-                  "shrinking %d -> %d" % (state.B, B_next))
+                  "shrinking %d -> %d" % (state.get_nonempty_B(), B_next))
         state = state.shrink(B=B_next, **shrink_args)
         if anneal:
             mcmc_anneal(state,
-                        **overlay(anneal_args,
-                                  mcmc_equilibrate_args=mcmc_equilibrate_args,
-                                  verbose=verbose_push(verbose,
-                                                       "B=%d  " % state.B)))
+                        **dict(anneal_args,
+                               mcmc_equilibrate_args=mcmc_equilibrate_args,
+                               verbose=verbose_push(verbose,
+                                                    "B=%d  " % state.get_nonempty_B())))
         else:
             mcmc_equilibrate(state,
-                             **overlay(mcmc_equilibrate_args,
-                                       verbose=verbose_push(verbose,
-                                                            ("B=%d  " %
-                                                             state.B))))
+                             **dict(mcmc_equilibrate_args,
+                                    verbose=verbose_push(verbose,
+                                                         ("B=%d  " %
+                                                          state.get_nonempty_B()))))
         if b_cache is not None:
             mcmc_args = mcmc_equilibrate_args.get("mcmc_args", {})
             entropy_args = mcmc_args.get("entropy_args", {})
@@ -377,7 +386,7 @@ def mcmc_multilevel(state, B, r=2, b_cache=None, anneal=False,
 
 class MulticanonicalState(object):
     r"""The density of states of a multicanonical Monte Carlo algorithm. It is used
-    by :func:`graph_tool.inference.multicanonical_equilibrate`.
+    by :func:`graph_tool.inference.mcmc.multicanonical_equilibrate`.
 
     Parameters
     ----------
@@ -402,19 +411,15 @@ class MulticanonicalState(object):
         self._hist.resize(nbins)
         self._perm_hist = numpy.zeros(nbins, dtype=self._hist.a.dtype)
         self._f = None
-        self._time = 0
-        self._refine = False
 
     def __getstate__(self):
         state = [self._g, self._S_min, self._S_max,
                  numpy.array(self._density.a), numpy.array(self._hist.a),
-                 numpy.array(self._perm_hist), self._f, self._time,
-                 self._refine]
+                 numpy.array(self._perm_hist), self._f]
         return state
 
     def __setstate__(self, state):
-        g, S_min, S_max, density, hist, phist, self._f, self._time, \
-            self._refine = state
+        g, S_min, S_max, density, hist, phist, self._f = state
         self.__init__(g, S_min, S_max, len(hist))
         self._density.a[:] = density
         self._hist.a[:] = hist
@@ -451,10 +456,13 @@ class MulticanonicalState(object):
 
     def get_entropy(self, S, B=None):
         r = self.get_density(B)
-        dS = (self._S_max - self._S_min) / len(r)
-        j = int(round((S - self._S_min) / dS))
+        j = self.get_bin()
         return r[j]
 
+    def get_bin(self, S):
+        return int(round((len(self._hist) - 1) * ((S - self._S_min) /
+                                                  (self._S_max - self._S_min))))
+
     def get_hist(self):
         "Get energy histogram."
         return numpy.array(self._hist.a)
@@ -463,30 +471,27 @@ class MulticanonicalState(object):
         "Get permanent energy histogram."
         return self._perm_hist
 
-    def get_flatness(self, use_ent=True, h=None, allow_gaps=False):
+    def get_flatness(self, h=None, allow_gaps=True):
         "Get energy histogram flatness."
         if h is None:
             h = self._hist.a
         if h.sum() == 0:
             return 0
         if allow_gaps:
-            h_all = h + self._perm_hist
-            h = array(h[h_all>0], dtype="float")
+            idx = (h + self._perm_hist) > 0
         else:
             Ss = self.get_range()
             S_min, S_max = self.get_allowed_energies()
-            h = array(h[numpy.logical_and(Ss >= S_min, Ss <= S_max)],
-                      dtype="float")
+            idx =numpy.logical_and(Ss >= S_min, Ss <= S_max)
+
+        h = array(h[idx], dtype="float")
+
         if len(h) == 1:
             h = array([1e-6] + list(h))
-        if not use_ent:
-            h_mean = h.mean()
-            return min(h.min() / h_mean,
-                       h_mean / h.max())
-        else:
-            h /= h.sum()
-            S = -(h * log(h)).sum()
-            return exp(S - log(len(h)))
+
+        h_mean = h.mean()
+        return min(h.min() / h_mean,
+                   h_mean / h.max())
 
     def get_posterior(self, N=None):
         "Get posterior probability."
@@ -502,35 +507,26 @@ class MulticanonicalState(object):
         self._perm_hist += self._hist.a
         self._hist.a = 0
 
-def multicanonical_equilibrate(state, m_state, f_range=(1., 1e-6),
-                               f_refine=1e-5, r=2, flatness=.95, use_ent=False,
-                               allow_gaps=False, callback=None,
+def multicanonical_equilibrate(state, m_state, f_range=(1., 1e-6), r=2,
+                               flatness=.95, allow_gaps=True, callback=None,
                                multicanonical_args={}, verbose=False):
     r"""Equilibrate a multicanonical Monte Carlo sampling using the Wang-Landau
-     algorithm.
+    algorithm.
 
     Parameters
     ----------
-    state : Any state class (e.g. :class:`~graph_tool.inference.BlockState`)
+    state : Any state class (e.g. :class:`~graph_tool.inference.blockmodel.BlockState`)
         Initial state. This state will be modified during the algorithm.
-    m_state :  :class:`~graph_tool.inference.MulticanonicalState`
+    m_state :  :class:`~graph_tool.inference.mcmc.MulticanonicalState`
         Initial multicanonical state, where the state density will be stored.
     f_range : ``tuple`` of two floats (optional, default: ``(1., 1e-6)``)
         Range of density updates.
-    f_refine : ``float`` (optional, default: ``1e-5``)
-        If the value of ``f`` decreases below this threshold, the refinement
-        steps described in [belardinelli-wang-2007]_ will be employed, instead
-        of histogram flatness.
     r : ``float`` (optional, default: ``2.``)
         Greediness of convergence. At each iteration, the density updates will
         be reduced by a factor ``r``.
-    flatness : ``float`` (optional, default: ``.99``)
+    flatness : ``float`` (optional, default: ``.95``)
         Sufficient histogram flatness threshold used to continue the algorithm.
-    use_ent : ``bool`` (optional, default: ``False``)
-        If ``True``, the histogram entropy will be used to determine flatness,
-        otherwise the smallest and largest counts relative to the mean will be
-        used.
-    allow_gaps : ``bool`` (optional, default: ``False``)
+    allow_gaps : ``bool`` (optional, default: ``True``)
         If ``True``, gaps in the histogram (regions with zero count) will be
         ignored when computing the flatness.
     callback : ``function`` (optional, default: ``None``)
@@ -538,7 +534,7 @@ def multicanonical_equilibrate(state, m_state, f_range=(1., 1e-6),
         function must accept the current ``state`` and ``m_state`` as arguments.
     multicanonical_args : ``dict`` (optional, default: ``{}``)
         Arguments to be passed to ``state.multicanonical_sweep`` (e.g.
-        :meth:`graph_tool.inference.BlockState.multicanonical_sweep`).
+        :meth:`graph_tool.inference.blockmodel.BlockState.multicanonical_sweep`).
     verbose : ``bool`` or ``tuple`` (optional, default: ``False``)
         If ``True``, progress information will be shown. Optionally, this
         accepts arguments of the type ``tuple`` of the form ``(level, prefix)``
@@ -570,25 +566,172 @@ def multicanonical_equilibrate(state, m_state, f_range=(1., 1e-6),
         m_state._f = f_range[0]
     while m_state._f >= f_range[1]:
         state.multicanonical_sweep(m_state, **multicanonical_args)
-        hf = m_state.get_flatness(use_ent=use_ent, allow_gaps=allow_gaps)
+        hf = m_state.get_flatness(allow_gaps=allow_gaps)
 
         if callback is not None:
             callback(state, m_state)
 
         if check_verbose(verbose):
             print(verbose_pad(verbose) +
-                  "count: %d  time: %#8.8g  f: %#8.8g  flatness: %#8.8g  S: %#8.8g" % \
-                  (count, m_state._time, m_state._f, hf,
-                   state.entropy(multicanonical_args.get("entropy_args", {}))))
+                  "count: %d  f: %#8.8g  flatness: %#8.8g  nonempty bins: %d  S: %#8.8g  B: %d" % \
+                  (count, m_state._f, hf, (m_state._hist.a > 0).sum(),
+                   state.entropy(**multicanonical_args.get("entropy_args", {})),
+                   state.get_nonempty_B()))
+
+        if hf > flatness:
+            m_state._f /= r
+            if m_state._f >= f_range[1]:
+                m_state.reset_hist()
 
-        if not m_state._refine:
-            if hf > flatness:
-                m_state._f /= r
-                if m_state._f >= f_range[1]:
-                    m_state.reset_hist()
-                if m_state._f <= f_refine:
-                    m_state._f = f_refine
-                    m_state._refine = True
         count += 1
 
-    return count
\ No newline at end of file
+    return count
+
+
+class TemperingState(object):
+    """This class aggregates several state classes and corresponding
+    inverse-temperature values to implement `parallel tempering MCMC
+    <https://en.wikipedia.org/wiki/Parallel_tempering>`_.
+
+    This is meant to be used with :func:`~graph_tool.inference.mcmc.mcmc_equilibrate`.
+
+    Parameters
+    ----------
+    states : list of state objects (e.g. :class:`~graph_tool.inference.blockmodel.BlockState`)
+        Initial parallel states.
+    betas : list of floats
+        Inverse temperature values.
+    """
+
+    def __init__(self, states, betas, idx=None):
+        if not (len(states) == len(betas)):
+            raise ValueError("states and betas must be of the same size")
+        self.states = states
+        self.betas = betas
+        if idx is None:
+            self.idx = list(range(len(betas)))
+
+    def entropy(self, **kwargs):
+        """Returns the sum of the entropy of the parallel states. All keyword
+        arguments are propagated to the individual states' `entropy()`
+        method.
+        """
+        return sum(s.entropy(beta_dl=beta, **kwargs) for s, beta in
+                   zip(self.states, self.betas))
+
+    def entropies(self, **kwargs):
+        """Returns the entropies of the parallel states. All keyword
+        arguments are propagated to the individual states' `entropy()`
+        method.
+        """
+        return [s.entropy(beta_dl=beta, **kwargs) for s, beta in
+                zip(self.states, self.betas)]
+
+    def states_swap(self, adjacent=True, **kwargs):
+        """Perform a full sweep of the parallel states, where swaps are attempted. All
+        relevant keyword arguments are propagated to the individual states'
+        `entropy()` method."""
+
+        verbose = kwargs.get("verbose", False)
+        eargs = kwargs.get("entropy_args", {})
+
+        if adjacent:
+            idx = numpy.arange(len(self.states) - 1)
+        else:
+            idx = numpy.arange(len(self.states))
+        numpy.random.shuffle(idx)
+        nswaps = 0
+        dS = 0
+        for i in idx:
+            if adjacent:
+                j = i + 1
+            else:
+                j = i
+                while j == i:
+                    j = numpy.random.randint(0, len(idx))
+
+            s1 = self.states[i]
+            s2 = self.states[j]
+            b1 = self.betas[i]
+            b2 = self.betas[j]
+
+            P1_f = -s1.entropy(beta_dl=b2, **eargs)
+            P2_f = -s2.entropy(beta_dl=b1, **eargs)
+
+            P1_b = -s1.entropy(beta_dl=b1, **eargs)
+            P2_b = -s2.entropy(beta_dl=b2, **eargs)
+
+            ddS = -(P1_f + P2_f - P1_b - P2_b)
+
+            if ddS < 0 or numpy.random.random() < exp(-ddS):
+                self.states[j], self.states[i], self.idx[j], self.idx[i] = \
+                            self.states[i], self.states[j], self.idx[i], self.idx[j]
+                nswaps += 1
+                dS += ddS
+                if check_verbose(verbose):
+                    print(verbose_pad(verbose)
+                          + u"swapped states: %d [β = %g] <-> %d [β = %g], a: %g" % \
+                            (i, b1, j, b2, a))
+        return dS, nswaps
+
+    def states_move(self, sweep_algo, **kwargs):
+        """Perform a full sweep of the parallel states, where state moves are
+        attempted by calling `sweep_algo(state, beta=beta, **kwargs)`."""
+        algo_states = []
+
+        if isinstance(self.states[0], NestedBlockState):
+            ls = list(kwargs.pop("ls", range(len(self.states[0].levels))))
+            if kwargs.pop("ls_shuffle", True):
+                numpy.random.shuffle(ls)
+            kwargs["ls"] = ls
+            kwargs["ls_shuffle"] = False
+
+        for state, beta in zip(self.states, self.betas):
+            entropy_args = dict(kwargs.get("entropy_args", {}))
+            algo_state = sweep_algo[0](state,
+                                       dispatch=False,
+                                       **dict(kwargs,
+                                              entropy_args=dict(entropy_args,
+                                                                beta_dl=beta)))
+            algo_states.append(algo_state)
+
+        return sweep_algo[1](self.states, algo_states)
+
+    def _sweep(self, algo, r=0.1, adjacent=True, **kwargs):
+        if numpy.random.random() < r:
+            return self.states_swap(adjacent=adjacent, **kwargs)
+        else:
+            return self.states_move(algo, **kwargs)
+
+    def mcmc_sweep(self, **kwargs):
+        """Perform a full mcmc sweep of the parallel states, where swap or moves are
+        chosen randomly. It accepts an keyword argument ``r`` (default: ``0.1``)
+        specifying the relative probability with which state swaps is performed
+        with respect to node moves.  All remaining keyword arguments are
+        propagated to the individual states' `mcmc_sweep()` method.
+        """
+        algo = (lambda s, **kw: s.mcmc_sweep(**kw),
+                lambda states, sweeps: type(self.states[0])._mcmc_sweep_parallel_dispatch(states, sweeps))
+        return self._sweep(algo, **kwargs)
+
+    def multiflip_mcmc_sweep(self, **kwargs):
+        """Perform a full mcmc sweep of the parallel states, where swap or moves are
+        chosen randomly. It accepts an keyword argument ``r`` (default: ``0.1``)
+        specifying the relative probability with which state swaps is performed
+        with respect to node moves.  All remaining keyword arguments are
+        propagated to the individual states' `mcmc_sweep()` method.
+        """
+        algo = (lambda s, **kw: s.multiflip_mcmc_sweep(**kw),
+                lambda states, sweeps: type(self.states[0])._multiflip_mcmc_sweep_parallel_dispatch(states, sweeps))
+        return self._sweep(algo, **kwargs)
+
+    def gibbs_sweep(self, **kwargs):
+        """Perform a full Gibbs mcmc sweep of the parallel states, where swap or moves
+        are chosen randomly. It accepts an keyword argument ``r`` (default:
+        ``0.1``) specifying the relative probability with which state swaps is
+        performed with respect to node moves.  All remaining keyword arguments
+        are propagated to the individual states' `gibbs_sweep()` method.
+        """
+        algo = (lambda s, **kw: s.gibbs_sweep(**kw),
+                lambda states, sweeps: type(self.states[0])._gibbs_sweep_parallel_dispatch(states, sweeps))
+        return self._sweep(algo, **kwargs)
