@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -34,29 +34,28 @@ from . nested_blockmodel import *
 
 def default_args(mcmc_args={}, anneal_args={}, mcmc_equilibrate_args={},
                  shrink_args={}, mcmc_multilevel_args={}, overlap=False):
-    mcmc_args = overlay(dict(beta=numpy.inf, c=0, niter=5, allow_empty=False,
-                             entropy_args=dict(dl=True)), **mcmc_args)
+    mcmc_args = dict(dict(beta=numpy.inf, c=0, niter=5, allow_vacate=False,
+                          entropy_args=dict(dl=True)), **mcmc_args)
     if overlap:
-        mcmc_args = overlay(mcmc_args, bundled=True)
-    mcmc_equilibrate_args = overlay(dict(wait=1, nbreaks=1, epsilon=1e-4,
-                                         mcmc_args=mcmc_args),
-                                    **dmask(mcmc_equilibrate_args,
-                                            ["mcmc_args"]))
-    shrink_entropy_args = overlay(overlay(mcmc_args["entropy_args"],
-                                          dl=not overlap),
-                                  **shrink_args.get("entropy_args", {}))
+        mcmc_args = dict(mcmc_args, bundled=True)
+    mcmc_equilibrate_args = dict(dict(wait=1, nbreaks=1, epsilon=1e-4,
+                                      mcmc_args=mcmc_args),
+                                 **dmask(mcmc_equilibrate_args,
+                                         ["mcmc_args"]))
+    shrink_entropy_args = dict(dict(mcmc_args["entropy_args"]),
+                               **shrink_args.get("entropy_args", {}))
     if not shrink_entropy_args.get("dense", False):
          shrink_entropy_args["multigraph"] = False
-    shrink_args = overlay(dict(entropy_args=shrink_entropy_args, niter=10),
-                          **dmask(shrink_args, ["entropy_args"]))
+    shrink_args = dict(dict(entropy_args=shrink_entropy_args, niter=20),
+                       **dmask(shrink_args, ["entropy_args"]))
     mcmc_multilevel_args = \
-            overlay(dict(r=1.5, anneal=False,
-                         shrink_args=shrink_args,
-                         mcmc_equilibrate_args=mcmc_equilibrate_args,
-                         anneal_args=anneal_args),
-                    **dmask(mcmc_multilevel_args,
-                            ["shrink_args", "mcmc_equilibrate_args",
-                             "anneal_args"]))
+            dict(dict(r=1.3, anneal=False,
+                      shrink_args=shrink_args,
+                      mcmc_equilibrate_args=mcmc_equilibrate_args,
+                      anneal_args=anneal_args),
+                 **dmask(mcmc_multilevel_args,
+                         ["shrink_args", "mcmc_equilibrate_args",
+                          "anneal_args"]))
     return mcmc_multilevel_args
 
 
@@ -83,7 +82,7 @@ def get_states(g, B_min=None, B_max=None, b_min=None, b_max=None, deg_corr=True,
 
     if layers:
         State = LayeredBlockState
-        state_args = overlay(state_args, overlap=overlap)
+        state_args = dict(state_args, overlap=overlap)
     elif overlap:
         State = OverlapBlockState
     else:
@@ -96,7 +95,7 @@ def get_states(g, B_min=None, B_max=None, b_min=None, b_max=None, deg_corr=True,
         max_state = State(g, B=_B_max, deg_corr=deg_corr, clabel=clabel,
                           **dmask(state_args,["B", "b", "deg_corr", "clabel"]))
 
-    if max_state.B < B_max:
+    if max_state.get_nonempty_B() > B_max:
         max_state = mcmc_multilevel(max_state, B=B_max, **mcmc_multilevel_args)
 
     if b_min is not None:
@@ -108,7 +107,7 @@ def get_states(g, B_min=None, B_max=None, b_min=None, b_max=None, deg_corr=True,
     else:
         min_state = max_state.copy()
 
-    if min_state.B > B_min:
+    if min_state.get_nonempty_B() > B_min:
         min_state = mcmc_multilevel(min_state, B=B_min, **mcmc_multilevel_args)
 
     return min_state, max_state
@@ -120,7 +119,8 @@ def minimize_blockmodel_dl(g, B_min=None, B_max=None, b_min=None, b_max=None,
                            mcmc_args={}, anneal_args={},
                            mcmc_equilibrate_args={}, shrink_args={},
                            mcmc_multilevel_args={}, verbose=False):
-    """Fit the stochastic block model.
+    """Fit the stochastic block model, by minimizing its description length using an
+    agglomerative heuristic.
 
     Parameters
     ----------
@@ -136,32 +136,32 @@ def minimize_blockmodel_dl(g, B_min=None, B_max=None, b_min=None, b_max=None,
         The partition to be used with the maximum number of blocks.
     deg_corr : ``bool`` (optional, default: ``True``)
         If ``True``, the degree-corrected version of the model will be used.
-    overlap : ``bool`` (optional, default: ``True``)
+    overlap : ``bool`` (optional, default: ``False``)
         If ``True``, the overlapping version of the model will be used.
     nonoverlap_init : ``bool`` (optional, default: ``True``)
         If ``True``, and ``overlap == True`` a non-overlapping initial state
         will be used.
-    layers : ``bool`` (optional, default: ``True``)
+    layers : ``bool`` (optional, default: ``False``)
         If ``True``, the layered version of the model will be used.
     state_args : ``dict`` (optional, default: ``{}``)
         Arguments to be passed to appropriate state constructor (e.g.
-        :class:`~graph_tool.inference.BlockState`,
-        :class:`~graph_tool.inference.OverlapBlockState` or
-        :class:`~graph_tool.inference.LayeredBlockState`)
+        :class:`~graph_tool.inference.blockmodel.BlockState`,
+        :class:`~graph_tool.inference.overlap_blockmodel.OverlapBlockState` or
+        :class:`~graph_tool.inference.layered_blockmodel.LayeredBlockState`)
     bisection_args : ``dict`` (optional, default: ``{}``)
-        Arguments to be passed to :func:`~graph_tool.inference.bisection_minimize`.
+        Arguments to be passed to :func:`~graph_tool.inference.bisection.bisection_minimize`.
     mcmc_args : ``dict`` (optional, default: ``{}``)
-        Arguments to be passed to :meth:`graph_tool.inference.BlockState.mcmc_sweep`,
-        :meth:`graph_tool.inference.OverlapBlockState.mcmc_sweep` or
-        :meth:`graph_tool.inference.LayeredBlockState.mcmc_sweep`.
+        Arguments to be passed to :meth:`graph_tool.inference.blockmodel.BlockState.mcmc_sweep`,
+        :meth:`graph_tool.inference.overlap_blockmodel.OverlapBlockState.mcmc_sweep` or
+        :meth:`graph_tool.inference.layered_blockmodel.LayeredBlockState.mcmc_sweep`.
     mcmc_equilibrate_args : ``dict`` (optional, default: ``{}``)
-        Arguments to be passed to :func:`~graph_tool.inference.mcmc_equilibrate`.
+        Arguments to be passed to :func:`~graph_tool.inference.mcmc.mcmc_equilibrate`.
     shrink_args : ``dict`` (optional, default: ``{}``)
-        Arguments to be passed to :meth:`graph_tool.inference.BlockState.shrink`,
-        :meth:`graph_tool.inference.OverlapBlockState.shrink` or
-        :meth:`graph_tool.inference.LayeredBlockState.shrink`.
+        Arguments to be passed to :meth:`graph_tool.inference.blockmodel.BlockState.shrink`,
+        :meth:`graph_tool.inference.overlap_blockmodel.OverlapBlockState.shrink` or
+        :meth:`graph_tool.inference.layered_blockmodel.LayeredBlockState.shrink`.
     mcmc_multilevel_args : ``dict`` (optional, default: ``{}``)
-        Arguments to be passed to :func:`~graph_tool.inference.mcmc_multilevel`.
+        Arguments to be passed to :func:`~graph_tool.inference.mcmc.mcmc_multilevel`.
     verbose : ``bool`` or ``tuple`` (optional, default: ``False``)
         If ``True``, progress information will be shown. Optionally, this
         accepts arguments of the type ``tuple`` of the form ``(level, prefix)``
@@ -171,14 +171,14 @@ def minimize_blockmodel_dl(g, B_min=None, B_max=None, b_min=None, b_max=None,
 
     Returns
     -------
-    min_state : :class:`~graph_tool.inference.BlockState` or  :class:`~graph_tool.inference.OverlapBlockState` or  :class:`~graph_tool.inference.LayeredBlockState`
+    min_state : :class:`~graph_tool.inference.blockmodel.BlockState` or  :class:`~graph_tool.inference.overlap_blockmodel.OverlapBlockState` or  :class:`~graph_tool.inference.layered_blockmodel.LayeredBlockState`
         State with minimal description length.
 
     Notes
     -----
 
     This function is a convenience wrapper around
-    :func:`~graph_tool.inference.bisection_minimize`.
+    :func:`~graph_tool.inference.bisection.bisection_minimize`.
 
     See [peixoto-efficient-2014]_ for details on the algorithm.
 
@@ -186,12 +186,12 @@ def minimize_blockmodel_dl(g, B_min=None, B_max=None, b_min=None, b_max=None,
     the number of nodes in the network.
 
     Examples
-    -----
+    --------
 
     .. testsetup:: mdl
 
-       gt.seed_rng(42)
-       np.random.seed(42)
+       gt.seed_rng(43)
+       np.random.seed(43)
 
     .. doctest:: mdl
 
@@ -277,13 +277,13 @@ def minimize_blockmodel_dl(g, B_min=None, B_max=None, b_min=None, b_max=None,
                      anneal_args=anneal_args,
                      mcmc_equilibrate_args=mcmc_equilibrate_args,
                      shrink_args=shrink_args,
-                     mcmc_multilevel_args=overlay(mcmc_multilevel_args,
-                                                  b_cache=b_cache),
+                     mcmc_multilevel_args=dict(mcmc_multilevel_args,
+                                               b_cache=b_cache),
                      overlap=overlap)
 
-    bisection_args = overlay(dict(mcmc_multilevel_args=mcmc_multilevel_args,
-                                  random_bisection=False),
-                             **bisection_args)
+    bisection_args = dict(dict(mcmc_multilevel_args=mcmc_multilevel_args,
+                               random_bisection=False),
+                          **bisection_args)
 
     clabel = state_args.get("clabel", None)
     if clabel is None:
@@ -320,7 +320,8 @@ def minimize_nested_blockmodel_dl(g, B_min=None, B_max=None, b_min=None,
                                   mcmc_args={}, anneal_args={},
                                   mcmc_equilibrate_args={}, shrink_args={},
                                   mcmc_multilevel_args={}, verbose=False):
-    """Fit the nested stochastic block model.
+    """Fit the nested stochastic block model, by minimizing its description length
+    using an agglomerative heuristic.
 
     Parameters
     ----------
@@ -340,34 +341,34 @@ def minimize_nested_blockmodel_dl(g, B_min=None, B_max=None, b_min=None,
         If provided, it will correspond to the initial hierarchical partition.
     deg_corr : ``bool`` (optional, default: ``True``)
         If ``True``, the degree-corrected version of the model will be used.
-    overlap : ``bool`` (optional, default: ``True``)
+    overlap : ``bool`` (optional, default: ``False``)
         If ``True``, the overlapping version of the model will be used.
     nonoverlap_init : ``bool`` (optional, default: ``True``)
         If ``True``, and ``overlap == True`` a non-overlapping initial state
         will be used.
-    layers : ``bool`` (optional, default: ``True``)
+    layers : ``bool`` (optional, default: ``False``)
         If ``True``, the layered version of the model will be used.
     hierarchy_minimize_args : ``dict`` (optional, default: ``{}``)
-        Arguments to be passed to :func:`~graph_tool.inference.hierarchy_minimize`.
+        Arguments to be passed to :func:`~graph_tool.inference.nested_blockmodel.hierarchy_minimize`.
     state_args : ``dict`` (optional, default: ``{}``)
         Arguments to be passed to appropriate state constructor (e.g.
-        :class:`~graph_tool.inference.BlockState`,
-        :class:`~graph_tool.inference.OverlapBlockState` or
-        :class:`~graph_tool.inference.LayeredBlockState`)
+        :class:`~graph_tool.inference.blockmodel.BlockState`,
+        :class:`~graph_tool.inference.overlap_blockmodel.OverlapBlockState` or
+        :class:`~graph_tool.inference.layered_blockmodel.LayeredBlockState`)
     bisection_args : ``dict`` (optional, default: ``{}``)
-        Arguments to be passed to :func:`~graph_tool.inference.bisection_minimize`.
+        Arguments to be passed to :func:`~graph_tool.inference.bisection.bisection_minimize`.
     mcmc_args : ``dict`` (optional, default: ``{}``)
-        Arguments to be passed to :meth:`graph_tool.inference.BlockState.mcmc_sweep`,
-        :meth:`graph_tool.inference.OverlapBlockState.mcmc_sweep` or
-        :meth:`graph_tool.inference.LayeredBlockState.mcmc_sweep`.
+        Arguments to be passed to :meth:`graph_tool.inference.blockmodel.BlockState.mcmc_sweep`,
+        :meth:`graph_tool.inference.overlap_blockmodel.OverlapBlockState.mcmc_sweep` or
+        :meth:`graph_tool.inference.layered_blockmodel.LayeredBlockState.mcmc_sweep`.
     mcmc_equilibrate_args : ``dict`` (optional, default: ``{}``)
-        Arguments to be passed to :func:`~graph_tool.inference.mcmc_equilibrate`.
+        Arguments to be passed to :func:`~graph_tool.inference.mcmc.mcmc_equilibrate`.
     shrink_args : ``dict`` (optional, default: ``{}``)
-        Arguments to be passed to :meth:`graph_tool.inference.BlockState.shrink`,
-        :meth:`graph_tool.inference.OverlapBlockState.shrink` or
-        :meth:`graph_tool.inference.LayeredBlockState.shrink`.
+        Arguments to be passed to :meth:`graph_tool.inference.blockmodel.BlockState.shrink`,
+        :meth:`graph_tool.inference.overlap_blockmodel.OverlapBlockState.shrink` or
+        :meth:`graph_tool.inference.layered_blockmodel.LayeredBlockState.shrink`.
     mcmc_multilevel_args : ``dict`` (optional, default: ``{}``)
-        Arguments to be passed to :func:`~graph_tool.inference.mcmc_multilevel`.
+        Arguments to be passed to :func:`~graph_tool.inference.mcmc.mcmc_multilevel`.
     verbose : ``bool`` or ``tuple`` (optional, default: ``False``)
         If ``True``, progress information will be shown. Optionally, this
         accepts arguments of the type ``tuple`` of the form ``(level, prefix)``
@@ -377,14 +378,14 @@ def minimize_nested_blockmodel_dl(g, B_min=None, B_max=None, b_min=None,
 
     Returns
     -------
-    min_state : :class:`~graph_tool.inference.NestedBlockState`
+    min_state : :class:`~graph_tool.inference.nested_blockmodel.NestedBlockState`
         Nested state with minimal description length.
 
     Notes
     -----
 
     This function is a convenience wrapper around
-    :func:`~graph_tool.inference.hierarchy_minimize`.
+    :func:`~graph_tool.inference.nested_blockmodel.hierarchy_minimize`.
 
     See [peixoto-hierarchical-2014]_ for details on the algorithm.
 
@@ -395,8 +396,8 @@ def minimize_nested_blockmodel_dl(g, B_min=None, B_max=None, b_min=None,
     --------
     .. testsetup:: nested_mdl
 
-       gt.seed_rng(42)
-       np.random.seed(42)
+       gt.seed_rng(43)
+       np.random.seed(43)
 
     .. doctest:: nested_mdl
 
@@ -474,7 +475,9 @@ def minimize_nested_blockmodel_dl(g, B_min=None, B_max=None, b_min=None,
                                           deg_corr=deg_corr, overlap=overlap,
                                           nonoverlap_init=nonoverlap_init,
                                           layers=layers, clabel=clabel,
-                                          state_args=state_args,
+                                          state_args=dmask(state_args,
+                                                           ["hstate_args",
+                                                            "hentropy_args"]),
                                           mcmc_multilevel_args=mcmc_multilevel_args)
         if b_max is None:
             b_max = max_state.b.fa
@@ -493,18 +496,27 @@ def minimize_nested_blockmodel_dl(g, B_min=None, B_max=None, b_min=None,
                                          **mcmc_multilevel_args)
                 bs.append(bstate.b.a)
                 bstate = bstate.get_block_state()
+        State = type(min_state)
+    else:
+        if layers:
+            State = LayeredBlockState
+            state_args = dict(state_args, overlap=overlap)
+        elif overlap:
+            State = OverlapBlockState
+        else:
+            State = BlockState
 
     if layers:
-        state_args = overlay(state_args, overlap=overlap)
+        state_args = dict(state_args, overlap=overlap)
 
     state = NestedBlockState(g, bs=bs,
-                             base_type=type(min_state),
+                             base_type=State,
                              deg_corr=deg_corr,
                              **dmask(state_args, ["deg_corr"]))
 
-    bisection_args = overlay(dict(mcmc_multilevel_args=mcmc_multilevel_args,
-                                  random_bisection=False),
-                             **bisection_args)
+    bisection_args = dict(dict(mcmc_multilevel_args=mcmc_multilevel_args,
+                               random_bisection=False),
+                          **bisection_args)
 
     hierarchy_minimize(state, B_max=B_max, B_min=B_min, b_max=b_max,
                        b_min=b_min, bisection_args=bisection_args,
