@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@ from . blockmodel import *
 from . blockmodel import _bm_test
 
 class OverlapBlockState(BlockState):
-    r"""The stochastic block model state of a given graph.
+    r"""The overlapping stochastic block model state of a given graph.
 
     Parameters
     ----------
@@ -59,32 +59,46 @@ class OverlapBlockState(BlockState):
     B : ``int`` (optional, default: ``None``)
         Number of blocks (or vertex groups). If not supplied it will be obtained
         from the parameter ``b``.
+    recs : list of :class:`~graph_tool.PropertyMap` instances (optional, default: ``[]``)
+        List of real or discrete-valued edge covariates.
+    rec_types : list of edge covariate types (optional, default: ``[]``)
+        List of types of edge covariates. The possible types are:
+        ``"real-exponential"``, ``"real-normal"``, ``"discrete-geometric"``,
+        ``"discrete-poisson"`` or ``"discrete-binomial"``.
+    rec_params : list of ``dict`` (optional, default: ``[]``)
+        Model hyperparameters for edge covariates. This should a list of
+        ``dict`` instances. See :class:`~graph_tool.inference.blockmodel.BlockState` for
+        more details.
     clabel : :class:`~graph_tool.PropertyMap` (optional, default: ``None``)
         Constraint labels on the vertices. If supplied, vertices with different
         label values will not be clustered in the same group.
     deg_corr : ``bool`` (optional, default: ``True``)
         If ``True``, the degree-corrected version of the blockmodel ensemble will
         be assumed, otherwise the traditional variant will be used.
+    allow_empty : ``bool`` (optional, default: ``False``)
+        If ``True``, partition description length computed will allow for empty
+        groups.
     max_BE : ``int`` (optional, default: ``1000``)
         If the number of blocks exceeds this number, a sparse representation of
         the block graph is used, which is slightly less efficient, but uses less
         memory,
     """
 
-    def __init__(self, g, b=None, B=None, clabel=None, pclabel=None,
-                 deg_corr=True, max_BE=1000, **kwargs):
+    def __init__(self, g, b=None, B=None, recs=[], rec_types=[], rec_params=[],
+                 clabel=None, pclabel=None, deg_corr=True, allow_empty=False,
+                 max_BE=1000, **kwargs):
 
         kwargs = kwargs.copy()
 
         # determine if there is a base graph, and overlapping structure
-        self.base_g = extract_arg(kwargs, "base_g", None)
+        self.base_g = kwargs.pop("base_g", None)
 
         # overlapping information
-        node_index = extract_arg(kwargs, "node_index", None)
-        node_in_degs = extract_arg(kwargs, "node_in_degs", None)
-        node_out_degs = extract_arg(kwargs, "node_out_degs", None)
-        half_edges = extract_arg(kwargs, "half_edges", None)
-        eindex = extract_arg(kwargs, "eindex", None)
+        node_index = kwargs.pop("node_index", None)
+        node_in_degs = kwargs.pop("node_in_degs", None)
+        node_out_degs = kwargs.pop("node_out_degs", None)
+        half_edges = kwargs.pop("half_edges", None)
+        eindex = kwargs.pop("eindex", None)
 
         if node_index is not None and self.base_g is None:
             raise ValueError("Must specify base graph if node_index is specified...")
@@ -94,8 +108,18 @@ class OverlapBlockState(BlockState):
             # keep base graph
             self.base_g = g
 
+            if len(recs) == 0:
+                rec = self.base_g.new_ep("vector<double>")
+            else:
+                recs = [x.copy("double") for x in recs]
+                rec = group_vector_property(recs)
+
             # substitute provided graph by its half-edge graph
-            g, b, node_index, half_edges, eindex = half_edge_graph(g, b, B)
+            g, b, node_index, half_edges, eindex, rec = \
+                                                half_edge_graph(g, b, B, rec)
+
+            if len(recs) > 0:
+                recs = ungroup_vector_property(rec, range(len(recs)))
 
         # create half edges set if absent
         if half_edges is None:
@@ -112,9 +136,6 @@ class OverlapBlockState(BlockState):
         # configure the main graph and block model parameters
         self.g = g
 
-        self.E = self.g.num_edges()
-        self.N = self.base_g.num_vertices()
-
         self.deg_corr = deg_corr
 
         self.is_edge_weighted = False
@@ -123,8 +144,6 @@ class OverlapBlockState(BlockState):
 
         if b is None:
             # create a random partition into B blocks.
-            if B is None:
-                B = get_max_B(self.N, self.E, directed=g.is_directed())
             B = min(B, self.g.num_vertices())
             ba = random.randint(0, B, self.g.num_vertices())
             ba[:B] = arange(B)        # avoid empty blocks
@@ -151,16 +170,38 @@ class OverlapBlockState(BlockState):
         if self.b.fa.max() >= B:
             raise ValueError("Maximum value of b is larger or equal to B!")
 
+        self.rec = [self.g.own_property(p) for p in recs]
+        for i in range(len(self.rec)):
+            if self.rec[i].value_type() != "double":
+                self.rec[i] = self.rec[i].copy("double")
+        self.drec = kwargs.pop("drec", None)
+        if self.drec is None:
+            self.drec = []
+            for rec in self.rec:
+                self.drec.append(self.g.new_ep("double", rec.fa ** 2))
+        else:
+            self.drec = [self.g.own_property(p) for p in self.drec]
+
+        rec_types = list(rec_types)
+        rec_params = list(rec_params)
+
+        # if len(rec_params) < len(rec_types):
+        #     rec_params += [{} for i in range((len(rec_types) -
+        #                                       len(rec_params)))]
+
+        if len(self.rec) > 0 and rec_types[0] != libinference.rec_type.count:
+            rec_types.insert(0, libinference.rec_type.count)
+            rec_params.insert(0, {})
+            self.rec.insert(0, self.g.new_ep("double", 1))
+            self.drec.insert(0, self.g.new_ep("double"))
+
         # Construct block-graph
-        self.bg = get_block_graph(g, B, self.b)
+        self.bg = get_block_graph(g, B, self.b, rec=self.rec, drec=self.drec)
         self.bg.set_fast_edge_removal()
 
         self.mrs = self.bg.ep["count"]
         self.wr = self.bg.vp["count"]
 
-        del self.bg.ep["count"]
-        del self.bg.vp["count"]
-
         self.mrp = self.bg.degree_property_map("out", weight=self.mrs)
 
         if g.is_directed():
@@ -170,8 +211,8 @@ class OverlapBlockState(BlockState):
 
         self.B = B
 
-        self.block_list = Vector_size_t()
-        self.block_list.extend(arange(self.B, dtype="int"))
+        self.candidate_blocks = Vector_size_t()
+        self.candidate_blocks.extend(arange(self.B, dtype="int"))
 
         if pclabel is not None:
             if isinstance(pclabel, PropertyMap):
@@ -194,24 +235,63 @@ class OverlapBlockState(BlockState):
             self.clabel = self.g.new_vp("int")
 
         self.bclabel = self.get_bclabel()
+        self.hclabel = self.bg.new_vp("int")
+
+        BlockState._init_recs(self, self.rec, rec_types, rec_params)
+        self.recdx = libcore.Vector_double(len(self.rec))
+        self.Lrecdx = kwargs.pop("Lrecdx", None)
+        if self.Lrecdx is None:
+            self.Lrecdx = libcore.Vector_double(len(self.rec)+1)
+            self.Lrecdx[0] = -1
+        self.Lrecdx.resize(len(self.rec)+1)
+        self.epsilon = kwargs.pop("epsilon", None)
+        if self.epsilon is None:
+            self.epsilon = libcore.Vector_double(len(self.rec))
+            for i in range(len(self.rec)):
+                idx = self.rec[i].a != 0
+                if numpy.any(idx):
+                    self.epsilon[i] = abs(self.rec[i].a[idx]).min() / 10
 
         self.max_BE = max_BE
 
-        if self.B > self.max_BE:
-            self.use_hash = libinference.true_type()
-        else:
-            self.use_hash = libinference.false_type()
+        self.use_hash = self.B > self.max_BE
+
+        self.allow_empty = True
 
         self._abg = self.bg._get_any()
         self._state = libinference.make_overlap_block_state(self, _get_rng())
 
+        if deg_corr:
+            init_q_cache(max(self.get_E(), self.get_N()) + 1)
+
+        self._entropy_args = dict(adjacency=True, deg_entropy=True, dl=True,
+                                  partition_dl=True, degree_dl=True,
+                                  degree_dl_kind="distributed", edges_dl=True,
+                                  dense=False, multigraph=True, exact=True,
+                                  recs=True, recs_dl=True, beta_dl=1.)
+
+        self._coupled_state = None
+
+        vweight = kwargs.pop("vweight", "unity")
+        eweight = kwargs.pop("eweight", "unity")
+
+        if vweight != "unity":
+            kwargs["vweight"] = vweight
+        if eweight != "unity":
+            kwargs["eweight"] = eweight
+
+        self.ignore_degrees = kwargs.pop("ignore_degrees", self.g.new_vp("bool"))
+
         if len(kwargs) > 0:
-            raise ValueError("unrecognized keyword arguments: " +
-                             str(list(kwargs.keys())))
+            warnings.warn("unrecognized keyword arguments: " +
+                          str(list(kwargs.keys())))
 
     def __repr__(self):
-        return "<OverlapBlockState object with %d blocks,%s for graph %s, at 0x%x>" % \
+        return "<OverlapBlockState object with %d blocks,%s%s for graph %s, at 0x%x>" % \
             (self.B, " degree corrected," if self.deg_corr else "",
+             ((" with %d edge covariate%s," % (len(self.rec_types) - 1,
+                                               "s" if len(self.rec_types) > 2 else ""))
+              if len(self.rec_types) > 0 else ""),
              str(self.base_g), id(self))
 
     def __copy__(self):
@@ -226,11 +306,11 @@ class OverlapBlockState(BlockState):
         return self.copy(g=g, node_index=node_index, eindex=eindex,
                          half_edges=half_edges, base_g=base_g)
 
-    def copy(self, g=None, eweight=None, vweight=None, b=None, B=None,
-             deg_corr=None, clabel=None, pclabel=None, **kwargs):
+    def copy(self, g=None, b=None, B=None, deg_corr=None, clabel=None,
+             pclabel=None, **kwargs):
         r"""Copies the block state. The parameters override the state properties, and
          have the same meaning as in the constructor. If ``overlap=False`` an
-         instance of :class:`~graph_tool.community.BlockState` is returned. This
+         instance of :class:`~graph_tool.inference.blockmodel.BlockState` is returned. This
          is by default a shallow copy."""
 
         state = OverlapBlockState(self.g if g is None else g,
@@ -239,14 +319,31 @@ class OverlapBlockState(BlockState):
                                   clabel=self.clabel.fa if clabel is None else clabel,
                                   pclabel=self.pclabel if pclabel is None else pclabel,
                                   deg_corr=self.deg_corr if deg_corr is None else deg_corr,
+                                  recs=kwargs.pop("recs", self.rec),
+                                  drec=kwargs.pop("drec", self.drec),
+                                  rec_types=kwargs.pop("rec_types", self.rec_types),
+                                  rec_params=kwargs.pop("rec_params",
+                                                        self.rec_params),
                                   half_edges=kwargs.get("half_edges", self.half_edges),
                                   node_index=kwargs.get("node_index", self.node_index),
                                   eindex=kwargs.get("eindex", self.eindex),
                                   max_BE=kwargs.get("max_BE", self.max_BE),
                                   base_g=kwargs.get("base_g", self.base_g),
+                                  Lrecdx=kwargs.pop("Lrecdx", self.Lrecdx.copy()),
+                                  epsilon=kwargs.pop("epsilon",
+                                                     self.epsilon.copy()),
+                                  allow_empty=kwargs.pop("allow_empty",
+                                                         self.allow_empty),
                                   **dmask(kwargs, ["half_edges", "node_index",
-                                                   "eindex", "base_g",
+                                                   "eindex", "base_g", "drec",
                                                    "max_BE"]))
+        if self._coupled_state is not None:
+            state._couple_state(state.get_block_state(b=state.get_bclabel(),
+                                                      vweight="nonempty",
+                                                      copy_bg=False,
+                                                      Lrecdx=state.Lrecdx,
+                                                      allow_empty=False),
+                                self._coupled_state[1])
         return state
 
     def __getstate__(self):
@@ -255,6 +352,11 @@ class OverlapBlockState(BlockState):
                      B=self.B,
                      clabel=array(self.clabel.fa),
                      deg_corr=self.deg_corr,
+                     recs=ungroup_vector_property(self.rec,
+                                                  range(len(self.recs))),
+                     drec=self.drec,
+                     rec_types=list(self.rec_types),
+                     rec_params=self.rec_params,
                      half_edges=self.half_edges,
                      node_index=self.node_index,
                      eindex=self.eindex,
@@ -266,41 +368,21 @@ class OverlapBlockState(BlockState):
         conv_pickle_state(state)
         self.__init__(**state)
 
-    def get_block_state(self, b=None, vweight=False, overlap=False,
-                        deg_corr=False, **kwargs):
-        r"""Returns a :class:`~graph_tool.community.BlockState` (or
-        :class:`~graph_tool.community.OverlapBlockState` if ``overlap == True``)
-        corresponding to the block graph. The parameters have the same meaning
-        as the in the constructor."""
+    def get_E(self):
+        r"Returns the total number of edges."
+        return self.g.num_edges()
 
-        if not overlap:
-            bg = self.bg.copy()
-            mrs = bg.own_property(self.mrs.copy())
-            wr = bg.own_property(self.wr.copy())
-            state = BlockState(bg, eweight=mrs,
-                               vweight=wr if vweight else None,
-                               b=bg.vertex_index.copy("int") if b is None else b,
-                               deg_corr=deg_corr,
-                               max_BE=self.max_BE, **kwargs)
-        else:
-            ## FIXME: Move this to C++
-            bg = Graph(directed=self.g.is_directed())
-            bg.add_vertex(self.B)
-            for e in self.g.edges():
-                r, s = tuple(e)
-                r = self.b[r]
-                s = self.b[s]
-                bg.add_edge(bg.vertex(r), bg.vertex(s))
+    def get_N(self):
+        r"Returns the total number of nodes."
+        return self.base_g.num_vertices()
 
-            state = OverlapBlockState(self.g,
-                                      b=self.b.a if b is None else b.copy(),
-                                      deg_corr=deg_corr,
-                                      base_g=bg,
-                                      node_index=self.b,
-                                      eindex=self.eindex,
-                                      max_BE=self.max_BE,
-                                      **kwargs)
-        return state
+    def get_B(self):
+        r"Returns the total number of blocks."
+        return self.bg.num_vertices()
+
+    def get_nonempty_B(self):
+        r"Returns the total number of nonempty blocks."
+        return int((self.wr.a > 0).sum())
 
     def get_edge_blocks(self):
         r"""Returns an edge property map which contains the block labels pairs for each
@@ -364,184 +446,231 @@ class OverlapBlockState(BlockState):
                                      _prop("v", self.base_g, b))
         return b
 
-    def entropy(self, dl=True, partition_dl=True, degree_dl=True,
-                edges_dl=True, dense=False, multigraph=True, dl_ent=False,
-                deg_entropy=True, **kwargs):
+    def entropy(self, adjacency=True, dl=True, partition_dl=True,
+                degree_dl=True, degree_dl_kind="distributed", edges_dl=True,
+                dense=False, multigraph=True, deg_entropy=True, recs=True,
+                recs_dl=True, beta_dl=1., exact=True, **kwargs):
         r"""Calculate the entropy associated with the current block partition.
 
         Parameters
         ----------
-        dl : ``bool`` (optional, default: ``False``)
-            If ``True``, the full description length will be returned.
+        adjacency : ``bool`` (optional, default: ``True``)
+            If ``True``, the adjacency term of the description length will be
+            included.
+        dl : ``bool`` (optional, default: ``True``)
+            If ``True``, the description length for the parameters will be
+            included.
         partition_dl : ``bool`` (optional, default: ``True``)
             If ``True``, and ``dl == True`` the partition description length
-            will be considered.
+            will be included.
         degree_dl : ``bool`` (optional, default: ``True``)
             If ``True``, and ``dl == True`` the degree sequence description
-            length will be considered.
+            length will be included (for degree-corrected models).
+        degree_dl_kind : ``str`` (optional, default: ``"distributed"``)
+            This specifies the prior used for the degree sequence. It must be
+            one of: ``"uniform"``, ``"distributed"`` (default) or ``"entropy"``.
         edges_dl : ``bool`` (optional, default: ``True``)
             If ``True``, and ``dl == True`` the edge matrix description length
-            will be considered.
+            will be included.
         dense : ``bool`` (optional, default: ``False``)
             If ``True``, the "dense" variant of the entropy will be computed.
-        multigraph : ``bool`` (optional, default: ``False``)
+        multigraph : ``bool`` (optional, default: ``True``)
             If ``True``, the multigraph entropy will be used.
-        dl_ent : ``bool`` (optional, default: ``False``)
-            If ``True``, the description length of the degree sequence will be
-            approximated by its entropy.
         deg_entropy : ``bool`` (optional, default: ``True``)
             If ``True``, the degree entropy term that is independent of the
-            network partition will be returned.
+            network partition will be included (for degree-corrected models).
+        recs : ``bool`` (optional, default: ``True``)
+            If ``True``, the likelihood for real or discrete-valued edge
+            covariates is computed.
+        recs_dl : ``bool`` (optional, default: ``True``)
+            If ``True``, and ``dl == True`` the edge covariate description
+            length will be included.
+        beta_dl : ``double`` (optional, default: ``1.``)
+            Prior inverse temperature.
+        exact : ``bool`` (optional, default: ``True``)
+            If ``True``, the exact expressions will be used. Otherwise,
+            Stirling's factorial approximation will be used for some terms.
 
         Notes
         -----
-        For the traditional blockmodel (``deg_corr == False``), the entropy is
-        given by
+
+        The "entropy" of the state is minus the log-likelihood of the
+        microcanonical SBM, that includes the generated graph
+        :math:`\boldsymbol{A}` and the model parameters :math:`\boldsymbol{\theta}`,
 
         .. math::
 
-          \mathcal{S}_t &\cong E - \frac{1}{2} \sum_{rs}e_{rs}\ln\left(\frac{e_{rs}}{n_rn_s}\right), \\
-          \mathcal{S}^d_t &\cong E - \sum_{rs}e_{rs}\ln\left(\frac{e_{rs}}{n_rn_s}\right),
+           \mathcal{S} &= - \ln P(\boldsymbol{A},\boldsymbol{\theta}) \\
+                       &= - \ln P(\boldsymbol{A}|\boldsymbol{\theta}) - \ln P(\boldsymbol{\theta}).
 
-        for undirected and directed graphs, respectively, where :math:`e_{rs}`
-        is the number of edges from block :math:`r` to :math:`s` (or the number
-        of half-edges for the undirected case when :math:`r=s`), and :math:`n_r
-        = \sum_ib_i^r` is the number of vertices in block :math:`r`, with
-        :math:`\vec{b}_i` being a binary vector with :math:`B` entries
-        describing the mixed membership of node :math:`i`. Note that because of
-        the possible overlaps, we have :math:`\sum_rn_r \ge N`.
+        This value is also called the `description length
+        <https://en.wikipedia.org/wiki/Minimum_description_length>`_ of the data,
+        and it corresponds to the amount of information required to describe it
+        (in `nats <https://en.wikipedia.org/wiki/Nat_(unit)>`_).
 
-        For the degree-corrected variant with "hard" degree constraints the
-        equivalent expressions are
+        For the traditional blockmodel (``deg_corr == False``), the model
+        parameters are :math:`\boldsymbol{\theta} = \{\boldsymbol{e},
+        \boldsymbol{b}\}`, where :math:`\boldsymbol{e}` is the matrix of edge
+        counts between blocks, and :math:`\boldsymbol{b}` is the `overlapping`
+        partition of the nodes into blocks. For the degree-corrected blockmodel
+        (``deg_corr == True``), we have an additional set of parameters, namely
+        the `labelled` degree sequence :math:`\boldsymbol{k}`.
+
+        The model likelihood :math:`P(\boldsymbol{A}|\theta)` is given
+        analogously to the non-overlapping case, as described in
+        :meth:`graph_tool.inference.blockmodel.BlockState.entropy`.
+
+        If ``dl == True``, the description length :math:`\mathcal{L} = -\ln
+        P(\boldsymbol{\theta})` of the model will be returned as well. The
+        edge-count prior :math:`P(\boldsymbol{e})` is described in described in
+        :func:`~graph_tool.inference.blockmodel.model_entropy`. For the
+        overlapping partition :math:`P(\boldsymbol{b})`, we have
 
         .. math::
 
-            \mathcal{S}_c &\cong -E -\sum_{ir}\ln k_i^r! - \frac{1}{2} \sum_{rs}e_{rs}\ln\left(\frac{e_{rs}}{e_re_s}\right), \\
-            \mathcal{S}^d_c &\cong -E -\sum_{ir}\left(\ln {k^+}_i^r! + \ln {k^-}_i^r! \right) - \sum_{rs}e_{rs}\ln\left(\frac{e_{rs}}{e^+_re^-_s}\right),
-
-        where :math:`e_r = \sum_se_{rs}` is the number of half-edges incident on
-        block :math:`r`, :math:`e^+_r = \sum_se_{rs}` and :math:`e^-_r =
-        \sum_se_{sr}` are the numbers of out- and in-edges adjacent to block
-        :math:`r`, respectively, and :math:`k_i^r` is the degree of node
-        :math:`i` of type :math:`r`, whereas :math:`{k^+}_i^r` and
-        :math:`{k^-}_i^r` are the labeled out- and in-degrees, respectively.
-
-        If ``multigraph == True``, the entropy used will be of the "Poisson"
-        model, with the additional term:
-
-        .. math::
-
-            {\mathcal{S}_{cm}^{(d)}} = \mathcal{S}_c^{(d)} + \sum_{i>j}\sum_{rs} \ln A^{rs}_{ij}! + \sum_i\sum_{rs} \ln A^{rs}_{ii}!!
-
-
-        If ``dl == True``, the description length :math:`\mathcal{L}_t` of the
-        model will be returned as well:
-
-        .. math::
-
-           \mathcal{L}_t = \ln\Omega_m + \ln\left(\!\!{D \choose N}\!\!\right) + \sum_d \ln {\left(\!\!{{B\choose d}\choose n_d}\!\!\right)} + \ln N! - \sum_{\vec{b}}\ln n_{\vec{b}}!,
+           -\ln P(\boldsymbol{b}) = \ln\left(\!\!{D \choose N}\!\!\right) + \sum_d \ln {\left(\!\!{{B\choose d}\choose n_d}\!\!\right)} + \ln N! - \sum_{\vec{b}}\ln n_{\vec{b}}!,
 
         where :math:`d \equiv |\vec{b}|_1 = \sum_rb_r` is the mixture
         size, :math:`n_d` is the number of nodes in a mixture of size :math:`d`,
         :math:`D` is the maximum value of :math:`d`, :math:`n_{\vec{b}}` is the
-        number of nodes in mixture :math:`\vec{b}`, and
+        number of nodes in mixture :math:`\vec{b}`.
+
+
+        For the degree-corrected model we need to specify the prior
+        :math:`P(\boldsymbol{k})` for the `labelled` degree sequence as well:
 
         .. math::
 
-            \Omega_m = \left(\!\!{\left(\!{B \choose 2}\!\right) \choose E}\!\!\right)
+            -\ln P(\boldsymbol{k}) = \sum_r\ln\left(\!\!{m_r \choose e_r}\!\!\right) - \sum_{\vec{b}}\ln P(\boldsymbol{k}|{\vec{b}}),
 
-        is the number of different :math:`e_{rs}` matrices for undirected
-        graphs, whereas for directed graphs we have
+        where :math:`m_r` is the number of non-empty mixtures which contain type
+        :math:`r`, and :math:`P(\boldsymbol{k}|{\vec{b}})` is the likelihood of
+        the labelled degree sequence inside mixture :math:`\vec{b}`. For this
+        term we have three options:
 
-        .. math::
+        1. ``degree_dl_kind == "uniform"``
 
-            \Omega_m = \left(\!\!{B^2 \choose E}\!\!\right).
+            .. math::
+
+                P(\boldsymbol{k}|\vec{b}) = \prod_r\left(\!\!{n_{\vec{b}}\choose e^r_{\vec{b}}}\!\!\right)^{-1}.
+
+        2. ``degree_dl_kind == "distributed"``
+
+            .. math::
+
+                P(\boldsymbol{k}|\vec{b}) = \prod_{\vec{b}}\frac{\prod_{\vec{k}}\eta_{\vec{k}}^{\vec{b}}!}{n_{\vec{b}}!} \prod_r q(e_{\vec{b}}^r - n_{\vec{b}}, n_{\vec{b}})
+
+            where :math:`n^{\vec{b}}_{\vec{k}}` is the number of nodes in
+            mixture :math:`\vec{b}` with labelled degree :math:`\vec{k}`, and
+            :math:`q(n,m)` is the number of `partitions
+            <https://en.wikipedia.org/wiki/Partition_(number_theory)>`_ of
+            integer :math:`n` into at most :math:`m` parts.
+
+        3. ``degree_dl_kind == "entropy"``
+
+            .. math::
+
+                P(\boldsymbol{k}|\vec{b}) = \prod_{\vec{b}}\exp\left(-n_{\vec{b}}H(\boldsymbol{k}_{\vec{b}})\right)
+
+            where :math:`H(\boldsymbol{k}_{\vec{b}}) =
+            -\sum_{\vec{k}}p_{\vec{b}}(\vec{k})\ln p_{\vec{b}}(\vec{k})` is the
+            entropy of the labelled degree distribution inside mixture
+            :math:`\vec{b}`.
+
+            Note that, differently from the other two choices, this represents
+            only an approximation of the description length. It is meant to be
+            used only for comparison purposes, and should be avoided in practice.
 
 
-        Note that for the degree-corrected version the description length is
-
-        .. math::
-
-            \mathcal{L}_c = \mathcal{L}_t + \sum_r\ln\left(\!\!{m_r \choose e_r}\!\!\right) + \sum_{\vec{b}}\min\left(\mathcal{L}^{(1)}_{\vec{b}}, \mathcal{L}^{(2)}_{\vec{b}}\right),
-
-        where :math:`m_r` is the number of non-empty mixtures which contain type :math:`r`, and
-
-        .. math::
-
-            \mathcal{L}^{(1)}_{\vec{b}} &= \sum_r\ln{\left(\!\!{n_{\vec{b}}\choose e^r_{\vec{b}}}\!\!\right)}, \\
-            \mathcal{L}^{(2)}_{\vec{b}} &= \sum_rb_r\ln\Xi_{\vec{b}}^r + \ln n_{\vec{b}}! - \sum_{\vec{k}} \ln n^{\vec{b}}_{\vec{k}}!,
-
-        and :math:`\ln\Xi_{\vec{b}}^r \simeq 2\sqrt{\zeta(2)e_{\vec{b}}^r}`,
-        where :math:`\zeta(x)` is the `Riemann zeta function
-        <https://en.wikipedia.org/wiki/Riemann_zeta_function>`_, and
-        :math:`n^{\vec{b}}_{\vec{k}}` is the number of nodes in mixture
-        :math:`\vec{b}` with labelled degree :math:`\vec{k}`. For directed
-        graphs we have instead :math:`\vec{k} \to (\vec{k}^-, \vec{k}^+)`, and
-        :math:`\ln\Xi_{\vec{b}}^r \to \ln{\Xi^+}_{\vec{b}}^r +
-        \ln{\Xi^-}_{\vec{b}}^r`.
-
-        If ``dl_ent=True`` is passed, this will be approximated instead by
-
-        .. math::
-
-            \mathcal{L}_c \simeq \mathcal{L}_t - \sum_{\vec{b}}n_{\vec{b}}\sum_{\vec{k}}p^{\vec{b}}_{\vec{k}}\ln p^{\vec{b}}_{\vec{k}},
-
-        where :math:`p^{\vec{b}}_{\vec{k}} = n^{\vec{b}}_{\vec{k}} / n_{\vec{b}}`.
+        For the directed case, the above expressions are duplicated for the in-
+        and out-degrees.
 
         """
 
-        return BlockState.entropy(self, dl=dl, partition_dl=partition_dl,
-                                  degree_dl=degree_dl, edges_dl=edges_dl,
-                                  dense=dense, multigraph=multigraph,
-                                  dl_ent=dl_ent, deg_entropy=deg_entropy,
+        return BlockState.entropy(self, adjacency=adjacency, dl=dl,
+                                  partition_dl=partition_dl,
+                                  degree_dl=degree_dl,
+                                  degree_dl_kind=degree_dl_kind,
+                                  edges_dl=edges_dl, dense=dense,
+                                  multigraph=multigraph,
+                                  deg_entropy=deg_entropy, recs=recs,
+                                  recs_dl=recs_dl, beta_dl=beta_dl, exact=exact,
                                   **kwargs)
 
+    def _clear_egroups(self):
+        self._state.clear_egroups()
 
     def _mcmc_sweep_dispatch(self, mcmc_state):
-        dS, nmoves = libinference.overlap_mcmc_sweep(mcmc_state, self._state,
-                                                     _get_rng())
+        dS, nattempts, nmoves = \
+                    libinference.overlap_mcmc_sweep(mcmc_state, self._state,
+                                                    _get_rng())
         if self.__bundled:
             ret = libinference.overlap_mcmc_bundled_sweep(mcmc_state,
-                                                           self._state,
+                                                          self._state,
                                                           _get_rng())
             dS += ret[0]
-            nmoves += ret[1]
+            nattempts += ret[1]
+            nmoves += ret[2]
         del self.__bundled
-        return dS, nmoves
+        return dS, nattempts, nmoves
+
+    def _mcmc_sweep_parallel_dispatch(states, mcmc_states):
+        return libinference.overlap_mcmc_sweep_parallel(mcmc_states,
+                                                        [s._state for s in states],
+                                                        _get_rng())
 
     def mcmc_sweep(self, bundled=False, **kwargs):
         r"""Perform sweeps of a Metropolis-Hastings rejection sampling MCMC to sample
         network partitions. If ``bundled == True``, the half-edges incident of
         the same node that belong to the same group are moved together. All
         remaining parameters are passed to
-        :meth:`graph_tool.inference.BlockState.mcmc_sweep`."""
+        :meth:`graph_tool.inference.blockmodel.BlockState.mcmc_sweep`."""
         self.__bundled = bundled
         return BlockState.mcmc_sweep(self, **kwargs)
 
-    def _multicanonical_sweep_dispatch(self, multicanonical_state):
-        return libinference.multicanonical_overlap_sweep(multicanonical_state,
+    def _multiflip_mcmc_sweep_dispatch(self, mcmc_state):
+        return libinference.overlap_multiflip_mcmc_sweep(mcmc_state,
                                                          self._state,
                                                          _get_rng())
 
+    def _multiflip_mcmc_sweep_parallel_dispatch(states, mcmc_states):
+        return libinference.overlap_multiflip_mcmc_sweep_parallel(mcmc_states,
+                                                                  [s._state for s in states],
+                                                                  _get_rng())
+
+    def _multicanonical_sweep_dispatch(self, multicanonical_state):
+        if multicanonical_state.multiflip:
+            return libinference.overlap_multicanonical_sweep(multicanonical_state,
+                                                             self._state,
+                                                             _get_rng())
+        else:
+            return libinference.overlap_multicanonical_multiflip_sweep(multicanonical_state,
+                                                                       self._state,
+                                                                       _get_rng())
+
     def _exhaustive_sweep_dispatch(self, exhaustive_state, callback, hist):
         if callback is not None:
-            return libinference.exhaustive_overlap_sweep(exhaustive_state,
+            return libinference.overlap_exhaustive_sweep(exhaustive_state,
                                                          self._state, callback)
         else:
             if hist is None:
-                return libinference.exhaustive_overlap_sweep_iter(exhaustive_state,
+                return libinference.overlap_exhaustive_sweep_iter(exhaustive_state,
                                                                   self._state)
             else:
-                return libinference.exhaustive_overlap_dens(exhaustive_state,
+                return libinference.overlap_exhaustive_dens(exhaustive_state,
                                                             self._state,
                                                             hist[0], hist[1],
                                                             hist[2])
 
     def _gibbs_sweep_dispatch(self, gibbs_state):
-        return libinference.gibbs_overlap_sweep(multicanonical_state,
-                                                self._state,
+        return libinference.gibbs_overlap_sweep(gibbs_state, self._state,
                                                 _get_rng())
 
+    def _gibbs_sweep_parallel_dispatch(states, gibbs_states):
+        return libinference.overlap_gibbs_sweep_parallel(gibbs_states,
+                                                         [s._state for s in states],
+                                                         _get_rng())
+
     def _merge_sweep_dispatch(self, merge_state):
         return libinference.vacate_overlap_sweep(merge_state, self._state,
                                                  _get_rng())
@@ -549,21 +678,37 @@ class OverlapBlockState(BlockState):
     def shrink(self, B, **kwargs):
         """Reduces the order of current state by progressively merging groups,
         until only ``B`` are left. All remaining keyword arguments are passed to
-        :meth:`graph_tool.inference.BlockState.merge_sweep`.
+        :meth:`graph_tool.inference.blockmodel.BlockState.merge_sweep`.
 
         This function leaves the current state untouched and returns instead a
         copy with the new partition.
         """
 
-        bstate = self.copy()
-        while (bstate.wr.a > 0).sum() > B:
-            bstate.merge_sweep((bstate.wr.a > 0).sum() - B, **kwargs)
+        b = self.b.copy()
+        continuous_map(b)
+        bstate = self.copy(b=b)
+
+        assert self.get_nonempty_B() == bstate.get_nonempty_B(), \
+            "Error: inconsistent number of groups after copying (%d, %d)" % \
+            (self.get_nonempty_B(), bstate.get_nonempty_B())
+
+        if bstate.get_nonempty_B() < B:
+            raise ValueError("cannot shrink state to a larger number" +
+                             " of groups: %d -> %d (total: %d)" %
+                             (bstate.get_nonempty_B(), B, self.B))
+
+        while bstate.get_nonempty_B() > B:
+            bstate.merge_sweep(bstate.get_nonempty_B() - B, **kwargs)
+
         continuous_map(bstate.b)
-        bstate = self.copy(b=bstate.b.a)
+        bstate = self.copy(b=bstate.b.a, Lrecdx=bstate.Lrecdx)
+
         if _bm_test():
-            assert bstate.B == B, \
-                "wrong number of groups after shrink: %d, %d" % (bstate.B, B)
+            assert bstate.get_nonempty_B() == B, \
+                "wrong number of groups after shrink: %d, %d" % \
+                (bstate.get_nonempty_B(), B)
             assert bstate.wr.a.min() > 0, "empty group after shrink!"
+
         return bstate
 
     def draw(self, **kwargs):
@@ -595,7 +740,7 @@ class OverlapBlockState(BlockState):
                                            "edge_gradient"]))
 
 
-def half_edge_graph(g, b=None, B=None):
+def half_edge_graph(g, b=None, B=None, rec=None):
     r"""Generate a half-edge graph, where each half-edge is represented by a node,
     and an edge connects the half-edges like in the original graph."""
 
@@ -645,6 +790,12 @@ def half_edge_graph(g, b=None, B=None):
     half_edges = g.new_vertex_property("vector<int64_t>")
     be = eg.new_vertex_property("int")
     eindex = eg.new_edge_property("int64_t")
+    erec = eg.new_edge_property("vector<double>")
+
+    if rec is None:
+        rec_ = g.new_edge_property("vector<double>")
+    else:
+        rec_ = g.own_property(rec)
 
     # create half-edge graph
     libinference.get_eg_overlap(g._Graph__graph,
@@ -653,12 +804,17 @@ def half_edge_graph(g, b=None, B=None):
                                 _prop("v", eg, be),
                                 _prop("v", eg, node_index),
                                 _prop("v", g, half_edges),
-                                _prop("e", eg, eindex))
+                                _prop("e", eg, eindex),
+                                _prop("e", g, rec_),
+                                _prop("e", eg, erec))
 
     if b_array is not None:
         be.a = b_array
 
-    return eg, be, node_index, half_edges, eindex
+    if rec is None:
+        erec = None
+
+    return eg, be, node_index, half_edges, eindex, erec
 
 def augmented_graph(g, b, node_index, eweight=None):
     r"""Generates an augmented graph from the half-edge graph ``g`` partitioned
