@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -92,7 +92,7 @@ if sys.version_info < (3,):
 from .. dl_import import dl_import
 dl_import("from . import libgraph_tool_search")
 
-from .. import _prop, _python_type
+from .. import _prop, _python_type, _get_null_vertex
 import weakref
 import numpy
 
@@ -157,15 +157,17 @@ class BFSVisitor(object):
         return
 
 
-def bfs_search(g, source, visitor=BFSVisitor()):
+def bfs_search(g, source=None, visitor=BFSVisitor()):
     r"""Breadth-first traversal of a directed or undirected graph.
 
     Parameters
     ----------
     g : :class:`~graph_tool.Graph`
         Graph to be used.
-    source : :class:`~graph_tool.Vertex`
-        Source vertex.
+    source : :class:`~graph_tool.Vertex` (optional, default: ``None``)
+        Source vertex. If unspecified, all vertices will be traversed, by
+        iterating over starting vertices according to their index in increasing
+        order.
     visitor : :class:`~graph_tool.search.BFSVisitor` (optional, default: ``BFSVisitor()``)
         A visitor object that is invoked at the event points inside the
         algorithm. This should be a subclass of
@@ -256,14 +258,14 @@ def bfs_search(g, source, visitor=BFSVisitor()):
     --> Carlos has been discovered!
     --> Isaac has been discovered!
     Eve has been examined...
-    --> Carol has been discovered!
     --> Imothep has been discovered!
+    --> Carol has been discovered!
     Chuck has been examined...
     Carlos has been examined...
     --> Alice has been discovered!
     Isaac has been examined...
-    Carol has been examined...
     Imothep has been examined...
+    Carol has been examined...
     Alice has been examined...
     --> Oscar has been discovered!
     --> Dave has been discovered!
@@ -284,12 +286,16 @@ def bfs_search(g, source, visitor=BFSVisitor()):
     """
 
     try:
+        if source is None:
+            source = _get_null_vertex()
+        else:
+            source = int(source)
         libgraph_tool_search.bfs_search(g._Graph__graph,
-                                        int(source), visitor)
+                                        source, visitor)
     except StopSearch:
         pass
 
-def bfs_iterator(g, source):
+def bfs_iterator(g, source=None, array=False):
     r"""Return an iterator of the edges corresponding to a breath-first traversal of
     the graph.
 
@@ -297,12 +303,20 @@ def bfs_iterator(g, source):
     ----------
     g : :class:`~graph_tool.Graph`
         Graph to be used.
-    source : :class:`~graph_tool.Vertex`
-        Source vertex.
+    source : :class:`~graph_tool.Vertex` (optional, default: ``None``)
+        Source vertex. If unspecified, all vertices will be traversed, by
+        iterating over starting vertices according to their index in increasing
+        order.
+    array : ``bool`` (optional, default: ``False``)
+        If ``True``, a :class:`numpy.ndarray` will the edge endpoints be
+        returned instead.
 
     Returns
     -------
-    bfs_iterator : An iterator over the edges in breath-first order.
+    bfs_iterator : Iterator or :class:`numpy.ndarray`
+        An iterator over the edges in breath-first order. If ``array == True``,
+        this will be a :class:`numpy.ndarray` instead, of shape ``(E,2)``,
+        containing the edge endpoints.
 
     See Also
     --------
@@ -328,8 +342,8 @@ def bfs_iterator(g, source):
     Bob -> Chuck
     Bob -> Carlos
     Bob -> Isaac
-    Eve -> Carol
     Eve -> Imothep
+    Eve -> Carol
     Carlos -> Alice
     Alice -> Oscar
     Alice -> Dave
@@ -340,9 +354,16 @@ def bfs_iterator(g, source):
              Symposium on the Theory of Switching, 1959
     .. [bfs-bgl] http://www.boost.org/doc/libs/release/libs/graph/doc/breadth_first_search.html
     .. [bfs-wikipedia] http://en.wikipedia.org/wiki/Breadth-first_search
-    """
 
-    return libgraph_tool_search.bfs_search_generator(g._Graph__graph, int(source))
+    """
+    if source is None:
+        source = _get_null_vertex()
+    else:
+        source = int(source)
+    if not array:
+        return libgraph_tool_search.bfs_search_generator(g._Graph__graph, source)
+    else:
+        return libgraph_tool_search.bfs_search_array(g._Graph__graph, source)
 
 
 class DFSVisitor(object):
@@ -389,7 +410,7 @@ class DFSVisitor(object):
         there is some ambiguity between tree edges and back edges since the edge
         (u,v) and (v,u) are the same edge, but both the
         :meth:`~graph_tool.search.DFSVisitor.tree_edge` and
-        :meth:`~graph_tool.search..DFSVisitor.back_edge` functions will be
+        :meth:`~graph_tool.search.DFSVisitor.back_edge` functions will be
         invoked. One way to resolve this ambiguity is to record the tree edges,
         and then disregard the back-edges that are already marked as tree
         edges. An easy way to record tree edges is to record predecessors at the
@@ -409,21 +430,23 @@ class DFSVisitor(object):
         This is invoked on vertex u after finish_vertex has been called for all
         the vertices in the DFS-tree rooted at vertex u. If vertex u is a leaf
         in the DFS-tree, then the
-        :meth:`~graph_tool..search.DFSVisitor.finish_vertex` function is called
+        :meth:`~graph_tool.search.DFSVisitor.finish_vertex` function is called
         on u after all the out-edges of u have been examined.
         """
         return
 
 
-def dfs_search(g, source, visitor=DFSVisitor()):
+def dfs_search(g, source=None, visitor=DFSVisitor()):
     r"""Depth-first traversal of a directed or undirected graph.
 
     Parameters
     ----------
     g : :class:`~graph_tool.Graph`
         Graph to be used.
-    source : :class:`~graph_tool.Vertex`
-        Source vertex.
+    source : :class:`~graph_tool.Vertex` (optional, default: ``None``)
+        Source vertex. If unspecified, all vertices will be traversed, by
+        iterating over starting vertices according to their index in increasing
+        order.
     visitor : :class:`~graph_tool.search.DFSVisitor` (optional, default: ``DFSVisitor()``)
         A visitor object that is invoked at the event points inside the
         algorithm. This should be a subclass of
@@ -513,15 +536,23 @@ def dfs_search(g, source, visitor=DFSVisitor()):
     --> Bob has been discovered!
     edge (Bob, Eve) has been examined...
     --> Eve has been discovered!
-    edge (Eve, Bob) has been examined...
-    edge (Eve, Carol) has been examined...
+    edge (Eve, Isaac) has been examined...
+    --> Isaac has been discovered!
+    edge (Isaac, Bob) has been examined...
+    edge (Isaac, Chuck) has been examined...
+    --> Chuck has been discovered!
+    edge (Chuck, Eve) has been examined...
+    edge (Chuck, Isaac) has been examined...
+    edge (Chuck, Imothep) has been examined...
+    --> Imothep has been discovered!
+    edge (Imothep, Carol) has been examined...
     --> Carol has been discovered!
     edge (Carol, Eve) has been examined...
     edge (Carol, Imothep) has been examined...
-    --> Imothep has been discovered!
-    edge (Imothep, Carol) has been examined...
     edge (Imothep, Carlos) has been examined...
     --> Carlos has been discovered!
+    edge (Carlos, Eve) has been examined...
+    edge (Carlos, Imothep) has been examined...
     edge (Carlos, Bob) has been examined...
     edge (Carlos, Alice) has been examined...
     --> Alice has been discovered!
@@ -530,48 +561,45 @@ def dfs_search(g, source, visitor=DFSVisitor()):
     edge (Oscar, Alice) has been examined...
     edge (Oscar, Dave) has been examined...
     --> Dave has been discovered!
-    edge (Dave, Alice) has been examined...
     edge (Dave, Oscar) has been examined...
+    edge (Dave, Alice) has been examined...
     edge (Alice, Dave) has been examined...
     edge (Alice, Carlos) has been examined...
-    edge (Carlos, Eve) has been examined...
-    edge (Carlos, Imothep) has been examined...
     edge (Imothep, Chuck) has been examined...
-    --> Chuck has been discovered!
-    edge (Chuck, Bob) has been examined...
-    edge (Chuck, Eve) has been examined...
-    edge (Chuck, Isaac) has been examined...
-    --> Isaac has been discovered!
-    edge (Isaac, Bob) has been examined...
-    edge (Isaac, Chuck) has been examined...
-    edge (Isaac, Eve) has been examined...
-    edge (Chuck, Imothep) has been examined...
     edge (Imothep, Eve) has been examined...
+    edge (Chuck, Bob) has been examined...
+    edge (Isaac, Eve) has been examined...
+    edge (Eve, Imothep) has been examined...
     edge (Eve, Carlos) has been examined...
     edge (Eve, Chuck) has been examined...
-    edge (Eve, Isaac) has been examined...
-    edge (Eve, Imothep) has been examined...
+    edge (Eve, Bob) has been examined...
+    edge (Eve, Carol) has been examined...
     edge (Bob, Chuck) has been examined...
     edge (Bob, Carlos) has been examined...
     edge (Bob, Isaac) has been examined...
     >>> print(time.a)
-    [0 5 2 4 8 7 1 9 6 3]
+    [0 7 5 6 3 9 1 2 8 4]
     >>> print(pred.a)
-    [0 3 6 9 9 8 0 4 1 2]
+    [0 3 9 9 7 8 0 6 1 4]
 
     References
     ----------
     .. [dfs-bgl] http://www.boost.org/doc/libs/release/libs/graph/doc/depth_first_search.html
     .. [dfs-wikipedia] http://en.wikipedia.org/wiki/Depth-first_search
+
     """
 
     try:
+        if source is None:
+            source = _get_null_vertex()
+        else:
+            source = int(source)
         libgraph_tool_search.dfs_search(g._Graph__graph,
-                                        int(source), visitor)
+                                        source, visitor)
     except StopSearch:
         pass
 
-def dfs_iterator(g, source):
+def dfs_iterator(g, source=None, array=False):
     r"""Return an iterator of the edges corresponding to a depth-first traversal of
     the graph.
 
@@ -579,12 +607,20 @@ def dfs_iterator(g, source):
     ----------
     g : :class:`~graph_tool.Graph`
         Graph to be used.
-    source : :class:`~graph_tool.Vertex`
-        Source vertex.
+    source : :class:`~graph_tool.Vertex` (optional, default: ``None``)
+        Source vertex. If unspecified, all vertices will be traversed, by
+        iterating over starting vertices according to their index in increasing
+        order.
+    array : ``bool`` (optional, default: ``False``)
+        If ``True``, a :class:`numpy.ndarray` will the edge endpoints be
+        returned instead.
 
     Returns
     -------
-    dfs_iterator : An iterator over the edges in detpth-first order.
+    dfs_iterator : Iterator or :class:`numpy.ndarray`
+        An iterator over the edges in depth-first order. If ``array == True``,
+        this will be a :class:`numpy.ndarray` instead, of shape ``(E,2)``,
+        containing the edge endpoints.
 
     See Also
     --------
@@ -607,22 +643,30 @@ def dfs_iterator(g, source):
     >>> for e in gt.dfs_iterator(g, g.vertex(0)):
     ...    print(name[e.source()], "->", name[e.target()])
     Bob -> Eve
-    Eve -> Carol
-    Carol -> Imothep
+    Eve -> Isaac
+    Isaac -> Chuck
+    Chuck -> Imothep
+    Imothep -> Carol
     Imothep -> Carlos
     Carlos -> Alice
     Alice -> Oscar
     Oscar -> Dave
-    Imothep -> Chuck
-    Chuck -> Isaac
 
     References
     ----------
     .. [dfs-bgl] http://www.boost.org/doc/libs/release/libs/graph/doc/depth_first_search.html
     .. [dfs-wikipedia] http://en.wikipedia.org/wiki/Depth-first_search
+
     """
 
-    return libgraph_tool_search.dfs_search_generator(g._Graph__graph, int(source))
+    if source is None:
+        source = _get_null_vertex()
+    else:
+        source = int(source)
+    if not array:
+        return libgraph_tool_search.dfs_search_generator(g._Graph__graph, source)
+    else:
+        return libgraph_tool_search.dfs_search_array(g._Graph__graph, source)
 
 class DijkstraVisitor(object):
     r"""A visitor object that is invoked at the event-points inside the
@@ -641,8 +685,9 @@ class DijkstraVisitor(object):
     def examine_vertex(self, u):
         """
         This is invoked on a vertex as it is popped from the queue. This happens
-        immediately before :meth:`~graph_tool.DijsktraVisitor.examine_edge` is
-        invoked on each of the out-edges of vertex u.
+        immediately before
+        :meth:`~graph_tool.search.DijkstraVisitor.examine_edge` is invoked on
+        each of the out-edges of vertex u.
         """
         return
 
@@ -672,7 +717,7 @@ class DijkstraVisitor(object):
     def edge_not_relaxed(self, e):
         """
         Upon examination, if the edge is not relaxed (see
-        :meth:`~graph_tool.search.DijsktraVisitor.edge_relaxed`) then this
+        :meth:`~graph_tool.search.DijkstraVisitor.edge_relaxed`) then this
         method is invoked.
         """
         return
@@ -686,19 +731,21 @@ class DijkstraVisitor(object):
         return
 
 
-def dijkstra_search(g, source, weight, visitor=DijkstraVisitor(), dist_map=None,
+def dijkstra_search(g, weight, source=None, visitor=DijkstraVisitor(), dist_map=None,
                     pred_map=None, combine=lambda a, b: a + b,
                     compare=lambda a, b: a < b, zero=0, infinity=numpy.inf):
-    r"""Dijsktra traversal of a directed or undirected graph, with non-negative weights.
+    r"""Dijkstra traversal of a directed or undirected graph, with non-negative weights.
 
     Parameters
     ----------
     g : :class:`~graph_tool.Graph`
         Graph to be used.
-    source : :class:`~graph_tool.Vertex`
-        Source vertex.
     weight : :class:`~graph_tool.PropertyMap`
         Edge property map with weight values.
+    source : :class:`~graph_tool.Vertex` (optional, default: ``None``)
+        Source vertex. If unspecified, all vertices will be traversed, by
+        iterating over starting vertices according to their index in increasing
+        order.
     visitor : :class:`~graph_tool.search.DijkstraVisitor` (optional, default: ``DijkstraVisitor()``)
         A visitor object that is invoked at the event points inside the
         algorithm. This should be a subclass of
@@ -815,7 +862,7 @@ def dijkstra_search(g, source, weight, visitor=DijkstraVisitor(), dist_map=None,
     With the above class defined, we can perform the Dijkstra search as follows.
 
     >>> time = g.new_vertex_property("int")
-    >>> dist, pred = gt.dijkstra_search(g, g.vertex(0), weight, VisitorExample(name, time))
+    >>> dist, pred = gt.dijkstra_search(g, weight, g.vertex(0), VisitorExample(name, time))
     --> Bob has been discovered!
     edge (Bob, Eve) has been examined...
     edge (Bob, Eve) has been relaxed...
@@ -829,29 +876,29 @@ def dijkstra_search(g, source, weight, visitor=DijkstraVisitor(), dist_map=None,
     edge (Bob, Isaac) has been examined...
     edge (Bob, Isaac) has been relaxed...
     --> Isaac has been discovered!
-    edge (Eve, Bob) has been examined...
-    edge (Eve, Carol) has been examined...
-    edge (Eve, Carol) has been relaxed...
-    --> Carol has been discovered!
-    edge (Eve, Carlos) has been examined...
-    edge (Eve, Chuck) has been examined...
     edge (Eve, Isaac) has been examined...
     edge (Eve, Imothep) has been examined...
     edge (Eve, Imothep) has been relaxed...
     --> Imothep has been discovered!
+    edge (Eve, Carlos) has been examined...
+    edge (Eve, Chuck) has been examined...
+    edge (Eve, Bob) has been examined...
+    edge (Eve, Carol) has been examined...
+    edge (Eve, Carol) has been relaxed...
+    --> Carol has been discovered!
     edge (Isaac, Bob) has been examined...
     edge (Isaac, Chuck) has been examined...
     edge (Isaac, Eve) has been examined...
-    edge (Chuck, Bob) has been examined...
     edge (Chuck, Eve) has been examined...
     edge (Chuck, Isaac) has been examined...
     edge (Chuck, Imothep) has been examined...
+    edge (Chuck, Bob) has been examined...
+    edge (Carlos, Eve) has been examined...
+    edge (Carlos, Imothep) has been examined...
     edge (Carlos, Bob) has been examined...
     edge (Carlos, Alice) has been examined...
     edge (Carlos, Alice) has been relaxed...
     --> Alice has been discovered!
-    edge (Carlos, Eve) has been examined...
-    edge (Carlos, Imothep) has been examined...
     edge (Imothep, Carol) has been examined...
     edge (Imothep, Carlos) has been examined...
     edge (Imothep, Chuck) has been examined...
@@ -867,15 +914,15 @@ def dijkstra_search(g, source, weight, visitor=DijkstraVisitor(), dist_map=None,
     edge (Carol, Imothep) has been examined...
     edge (Oscar, Alice) has been examined...
     edge (Oscar, Dave) has been examined...
-    edge (Dave, Alice) has been examined...
     edge (Dave, Oscar) has been examined...
+    edge (Dave, Alice) has been examined...
     >>> print(time.a)
-    [0 7 5 3 2 9 1 4 8 6]
+    [0 7 6 3 2 9 1 4 8 5]
     >>> print(pred.a)
     [0 3 6 0 0 1 0 0 1 6]
     >>> print(dist.a)
-    [  0.           8.91915887   9.27141329   4.29277116   4.02118246
-      12.23513866   3.23790211   3.45487436  11.04391549   7.74858396]
+    [ 0.          8.91915887  9.27141329  4.29277116  4.02118246 12.23513866
+      3.23790211  3.45487436 11.04391549  7.74858396]
 
     References
     ----------
@@ -910,8 +957,12 @@ def dijkstra_search(g, source, weight, visitor=DijkstraVisitor(), dist_map=None,
         infinity = _python_type(dist_map.value_type())(infinity)
 
     try:
+        if source is None:
+            source = _get_null_vertex()
+        else:
+            source = int(source)
         libgraph_tool_search.dijkstra_search(g._Graph__graph,
-                                             int(source),
+                                             source,
                                              _prop("v", g, dist_map),
                                              _prop("v", g, pred_map),
                                              _prop("e", g, weight), visitor,
@@ -921,8 +972,8 @@ def dijkstra_search(g, source, weight, visitor=DijkstraVisitor(), dist_map=None,
 
     return dist_map, pred_map
 
-def dijkstra_iterator(g, source, weight, dist_map=None, combine=None,
-                      compare=None, zero=0, infinity=numpy.inf):
+def dijkstra_iterator(g, weight, source=None, dist_map=None, combine=None,
+                      compare=None, zero=0, infinity=numpy.inf, array=False):
     r"""Return an iterator of the edges corresponding to a Dijkstra traversal of
     the graph.
 
@@ -930,10 +981,12 @@ def dijkstra_iterator(g, source, weight, dist_map=None, combine=None,
     ----------
     g : :class:`~graph_tool.Graph`
         Graph to be used.
-    source : :class:`~graph_tool.Vertex`
-        Source vertex.
     weight : :class:`~graph_tool.PropertyMap`
         Edge property map with weight values.
+    source : :class:`~graph_tool.Vertex` (optional, default: ``None``)
+        Source vertex. If unspecified, all vertices will be traversed, by
+        iterating over starting vertices according to their index in increasing
+        order.
     dist_map : :class:`~graph_tool.PropertyMap` (optional, default: ``None``)
         A vertex property map where the distances from the source will be
         stored.
@@ -947,12 +1000,18 @@ def dijkstra_iterator(g, source, weight, dist_map=None, combine=None,
          Value assumed to correspond to a distance of zero by the combine and
          compare functions.
     infinity : int or float (optional, default: ``numpy.inf``)
-         Value assumed to correspond to a distance of infinity by the combine and
-         compare functions.
+         Value assumed to correspond to a distance of infinity by the combine
+         and compare functions.
+    array : ``bool`` (optional, default: ``False``)
+        If ``True``, a :class:`numpy.ndarray` will the edge endpoints be
+        returned instead.
 
     Returns
     -------
-    djk_iterator : An iterator over the edges in Dijkstra order.
+    dfs_iterator : Iterator or :class:`numpy.ndarray`
+        An iterator over the edges in Dijkstra order. If ``array == True``,
+        this will be a :class:`numpy.ndarray` instead, of shape ``(E,2)``,
+        containing the edge endpoints.
 
     See Also
     --------
@@ -972,25 +1031,25 @@ def dijkstra_iterator(g, source, weight, dist_map=None, combine=None,
     Examples
     --------
 
-    >>> for e in gt.dijkstra_iterator(g, g.vertex(0), weight):
+    >>> for e in gt.dijkstra_iterator(g, weight, g.vertex(0)):
     ...    print(name[e.source()], "->", name[e.target()])
     Bob -> Eve
     Bob -> Chuck
     Bob -> Carlos
     Bob -> Isaac
-    Eve -> Carol
     Eve -> Imothep
+    Eve -> Carol
     Carlos -> Alice
     Alice -> Oscar
     Alice -> Dave
 
-
     References
     ----------
     .. [dijkstra] E. Dijkstra, "A note on two problems in connexion with
         graphs", Numerische Mathematik, 1:269-271, 1959.
     .. [dijkstra-bgl] http://www.boost.org/doc/libs/release/libs/graph/doc/dijkstra_shortest_paths_no_color_map.html
     .. [dijkstra-wikipedia] http://en.wikipedia.org/wiki/Dijkstra's_algorithm
+
     """
 
     if dist_map is None:
@@ -1010,9 +1069,20 @@ def dijkstra_iterator(g, source, weight, dist_map=None, combine=None,
         infinity = (weight.a.max() + 1) * g.num_vertices()
         infinity = _python_type(dist_map.value_type())(infinity)
 
+    if source is None:
+        source = _get_null_vertex()
+    else:
+        source = int(source)
     if compare is None and combine is None:
-        return libgraph_tool_search.dijkstra_generator_fast(g._Graph__graph,
-                                                            int(source),
+        if not array:
+            return libgraph_tool_search.dijkstra_generator_fast(g._Graph__graph,
+                                                                source,
+                                                                _prop("v", g, dist_map),
+                                                                _prop("e", g, weight),
+                                                                zero, infinity)
+        else:
+            return libgraph_tool_search.dijkstra_array_fast(g._Graph__graph,
+                                                            source,
                                                             _prop("v", g, dist_map),
                                                             _prop("e", g, weight),
                                                             zero, infinity)
@@ -1021,8 +1091,16 @@ def dijkstra_iterator(g, source, weight, dist_map=None, combine=None,
             compare = lambda a, b: a < b
         if combine is None:
             combine = lambda a, b: a + b
-        return libgraph_tool_search.dijkstra_generator(g._Graph__graph,
-                                                       int(source),
+        if not array:
+            return libgraph_tool_search.dijkstra_generator(g._Graph__graph,
+                                                           source,
+                                                           _prop("v", g, dist_map),
+                                                           _prop("e", g, weight),
+                                                           compare, combine,
+                                                           zero, infinity)
+        else:
+            return libgraph_tool_search.dijkstra_array(g._Graph__graph,
+                                                       source,
                                                        _prop("v", g, dist_map),
                                                        _prop("e", g, weight),
                                                        compare, combine,
@@ -1128,7 +1206,7 @@ def bellman_ford_search(g, source, weight, visitor=BellmanFordVisitor(),
     --------
     bfs_search: Breadth-first search
     dfs_search: Depth-first search
-    dijsktra_search: Dijkstra search
+    dijkstra_search: Dijkstra search
     astar_search: :math:`A^*` heuristic search
 
     Notes
@@ -1765,7 +1843,8 @@ def astar_search(g, source, weight, visitor=AStarVisitor(),
 
 
 def astar_iterator(g, source, weight, heuristic=lambda v: 1, dist_map=None,
-                   combine=None, compare=None, zero=0, infinity=numpy.inf):
+                   combine=None, compare=None, zero=0, infinity=numpy.inf,
+                   array=False):
     r"""Return an iterator of the edges corresponding to an :math:`A^*` traversal of
     the graph.
 
@@ -1794,12 +1873,18 @@ def astar_iterator(g, source, weight, heuristic=lambda v: 1, dist_map=None,
          Value assumed to correspond to a distance of zero by the combine and
          compare functions.
     infinity : int or float (optional, default: ``numpy.inf``)
-         Value assumed to correspond to a distance of infinity by the combine and
-         compare functions.
+         Value assumed to correspond to a distance of infinity by the combine
+         and compare functions.
+    array : ``bool`` (optional, default: ``False``)
+        If ``True``, a :class:`numpy.ndarray` will the edge endpoints be
+        returned instead.
 
     Returns
     -------
-    astar_iterator : An iterator over the edges in :math:`A^*` order.
+    astar_iterator : Iterator or :class:`numpy.ndarray`
+        An iterator over the edges in :math:`A^*` order. If ``array == True``,
+        this will be a :class:`numpy.ndarray` instead, of shape ``(E,2)``,
+        containing the edge endpoints.
 
     See Also
     --------
@@ -1828,8 +1913,8 @@ def astar_iterator(g, source, weight, heuristic=lambda v: 1, dist_map=None,
     Bob -> Chuck
     Bob -> Carlos
     Bob -> Isaac
-    Eve -> Carol
     Eve -> Imothep
+    Eve -> Carol
     Carlos -> Alice
     Alice -> Oscar
     Alice -> Dave
@@ -1842,6 +1927,7 @@ def astar_iterator(g, source, weight, heuristic=lambda v: 1, dist_map=None,
        :doi:`10.1109/TSSC.1968.300136`
     .. [astar-bgl] http://www.boost.org/doc/libs/release/libs/graph/doc/astar_search.html
     .. [astar-wikipedia] http://en.wikipedia.org/wiki/A*_search_algorithm
+
     """
 
     if dist_map is None:
@@ -1862,7 +1948,14 @@ def astar_iterator(g, source, weight, heuristic=lambda v: 1, dist_map=None,
         infinity = _python_type(dist_map.value_type())(infinity)
 
     if compare is None and combine is None:
-        return libgraph_tool_search.astar_generator_fast(g._Graph__graph,
+        if not array:
+            return libgraph_tool_search.astar_generator_fast(g._Graph__graph,
+                                                             int(source),
+                                                             _prop("v", g, dist_map),
+                                                             _prop("e", g, weight),
+                                                             zero, infinity, heuristic)
+        else:
+            return libgraph_tool_search.astar_array_fast(g._Graph__graph,
                                                          int(source),
                                                          _prop("v", g, dist_map),
                                                          _prop("e", g, weight),
@@ -1872,15 +1965,21 @@ def astar_iterator(g, source, weight, heuristic=lambda v: 1, dist_map=None,
             compare = lambda a, b: a < b
         if combine is None:
             combine = lambda a, b: a + b
-        return libgraph_tool_search.astar_generator(g._Graph__graph,
+        if not array:
+            return libgraph_tool_search.astar_generator(g._Graph__graph,
+                                                        int(source),
+                                                        _prop("v", g, dist_map),
+                                                        _prop("e", g, weight),
+                                                        compare, combine,
+                                                        zero, infinity, heuristic)
+        else:
+            return libgraph_tool_search.astar_array(g._Graph__graph,
                                                     int(source),
                                                     _prop("v", g, dist_map),
                                                     _prop("e", g, weight),
                                                     compare, combine,
                                                     zero, infinity, heuristic)
 
-
-
 class StopSearch(Exception):
     """If this exception is raised from inside any search visitor object, the search is aborted."""
     pass
