@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -114,9 +114,9 @@ def adjacency(g, weight=None, index=None):
     >>> scatter(real(ew), imag(ew), c=sqrt(abs(ew)), linewidths=0, alpha=0.6)
     <...>
     >>> xlabel(r"$\operatorname{Re}(\lambda)$")
-    <...>
+    Text(...)
     >>> ylabel(r"$\operatorname{Im}(\lambda)$")
-    <...>
+    Text(...)
     >>> tight_layout()
     >>> savefig("adjacency-spectrum.pdf")
 
@@ -136,7 +136,7 @@ def adjacency(g, weight=None, index=None):
     """
 
     if index is None:
-        if g.get_vertex_filter()[0] != None:
+        if g.get_vertex_filter()[0] is not None:
             index = g.new_vertex_property("int64_t")
             index.fa = numpy.arange(g.num_vertices())
         else:
@@ -239,9 +239,9 @@ def laplacian(g, deg="total", normalized=False, weight=None, index=None):
     >>> scatter(real(ew), imag(ew), c=sqrt(abs(ew)), linewidths=0, alpha=0.6)
     <...>
     >>> xlabel(r"$\operatorname{Re}(\lambda)$")
-    <...>
+    Text(...)
     >>> ylabel(r"$\operatorname{Im}(\lambda)$")
-    <...>
+    Text(...)
     >>> tight_layout()
     >>> savefig("laplacian-spectrum.pdf")
 
@@ -263,9 +263,9 @@ def laplacian(g, deg="total", normalized=False, weight=None, index=None):
     >>> scatter(real(ew), imag(ew), c=sqrt(abs(ew)), linewidths=0, alpha=0.6)
     <...>
     >>> xlabel(r"$\operatorname{Re}(\lambda)$")
-    <...>
+    Text(...)
     >>> ylabel(r"$\operatorname{Im}(\lambda)$")
-    <...>
+    Text(...)
     >>> tight_layout()
     >>> savefig("norm-laplacian-spectrum.pdf")
 
@@ -285,14 +285,14 @@ def laplacian(g, deg="total", normalized=False, weight=None, index=None):
     """
 
     if index is None:
-        if g.get_vertex_filter()[0] != None:
+        if g.get_vertex_filter()[0] is not None:
             index = g.new_vertex_property("int64_t")
             index.fa = numpy.arange(g.num_vertices())
         else:
             index = g.vertex_index
 
     V = g.num_vertices()
-    nself = label_self_loops(g, mark_only=True).a.sum()
+    nself = int(label_self_loops(g, mark_only=True).a.sum())
     E = g.num_edges() - nself
     if not g.is_directed():
         E *= 2
@@ -369,13 +369,13 @@ def incidence(g, vindex=None, eindex=None):
     >>> g = gt.random_graph(100, lambda: (2,2))
     >>> m = gt.incidence(g)
     >>> print(m.todense())
-    [[-1. -1.  0. ...,  0.  0.  0.]
-     [ 0.  0.  0. ...,  0.  0.  0.]
-     [ 0.  0.  0. ...,  0.  0.  0.]
-     ..., 
-     [ 0.  0. -1. ...,  0.  0.  0.]
-     [ 0.  0.  0. ...,  0.  0.  0.]
-     [ 0.  0.  0. ...,  0.  0.  0.]]
+    [[-1. -1.  0. ...  0.  0.  0.]
+     [ 0.  0.  0. ...  0.  0.  0.]
+     [ 0.  0.  0. ...  0.  0.  0.]
+     ...
+     [ 0.  0. -1. ...  0.  0.  0.]
+     [ 0.  0.  0. ...  0.  0.  0.]
+     [ 0.  0.  0. ...  0.  0.  0.]]
 
     References
     ----------
@@ -383,14 +383,14 @@ def incidence(g, vindex=None, eindex=None):
     """
 
     if vindex is None:
-        if g.get_edge_filter()[0] != None:
+        if g.get_edge_filter()[0] is not None:
             vindex = g.new_vertex_property("int64_t")
             vindex.fa = numpy.arange(g.num_vertices())
         else:
             vindex = g.vertex_index
 
     if eindex is None:
-        if g.get_edge_filter()[0] != None:
+        if g.get_edge_filter()[0] is not None:
             eindex = g.new_edge_property("int64_t")
             eindex.fa = numpy.arange(g.num_edges())
         else:
@@ -467,9 +467,9 @@ def transition(g, weight=None, index=None):
     >>> scatter(real(ew), imag(ew), c=sqrt(abs(ew)), linewidths=0, alpha=0.6)
     <...>
     >>> xlabel(r"$\operatorname{Re}(\lambda)$")
-    <...>
+    Text(...)
     >>> ylabel(r"$\operatorname{Im}(\lambda)$")
-    <...>
+    Text(...)
     >>> tight_layout()
     >>> savefig("transition-spectrum.pdf")
 
@@ -489,7 +489,7 @@ def transition(g, weight=None, index=None):
     """
 
     if index is None:
-        if g.get_vertex_filter()[0] != None:
+        if g.get_vertex_filter()[0] is not None:
             index = g.new_vertex_property("int64_t")
             index.fa = numpy.arange(g.num_vertices())
         else:
@@ -564,9 +564,9 @@ def modularity_matrix(g, weight=None, index=None):
     >>> scatter(real(ew), imag(ew), c=sqrt(abs(ew)), linewidths=0, alpha=0.6)
     <...>
     >>> xlabel(r"$\operatorname{Re}(\lambda)$")
-    <...>
+    Text(...)
     >>> ylabel(r"$\operatorname{Im}(\lambda)$")
-    <...>
+    Text(...)
     >>> tight_layout()
     >>> savefig("modularity-spectrum.pdf")
 
