@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2016 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2018 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -113,8 +113,9 @@ def vertex_hist(g, deg, bins=[0, 1], float_count=True):
     >>> from numpy.random import poisson
     >>> g = gt.random_graph(1000, lambda: (poisson(5), poisson(5)))
     >>> print(gt.vertex_hist(g, "out"))
-    [array([   7.,   33.,   91.,  145.,  165.,  164.,  152.,  115.,   62.,
-             29.,   28.,    6.,    1.,    1.,    0.,    1.]), array([ 0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15, 16], dtype=uint...)]
+    [array([  7.,  33.,  91., 145., 165., 164., 152., 115.,  62.,  29.,  28.,
+             6.,   1.,   1.,   0.,   1.]), array([ 0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15, 16],
+          dtype=uint64)]
     """
 
     ret = libgraph_tool_stats.\
@@ -177,7 +178,7 @@ def edge_hist(g, eprop, bins=[0, 1], float_count=True):
     >>> eprop = g.new_edge_property("double")
     >>> eprop.get_array()[:] = random(g.num_edges())
     >>> print(gt.edge_hist(g, eprop, linspace(0, 1, 11)))
-    [array([ 501.,  441.,  478.,  480.,  506.,  494.,  507.,  535.,  499.,  559.]), array([ 0. ,  0.1,  0.2,  0.3,  0.4,  0.5,  0.6,  0.7,  0.8,  0.9,  1. ])]
+    [array([501., 441., 478., 480., 506., 494., 507., 535., 499., 559.]), array([0. , 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1. ])]
 
     """
 
@@ -231,7 +232,7 @@ def vertex_average(g, deg):
     >>> from numpy.random import poisson
     >>> g = gt.random_graph(1000, lambda: (poisson(5), poisson(5)))
     >>> print(gt.vertex_average(g, "in"))
-    (4.975, 0.068675869124460318)
+    (4.96, 0.0679882342762334)
     """
 
     if isinstance(deg, PropertyMap) and "string" in deg.value_type():
@@ -293,7 +294,7 @@ def edge_average(g, eprop):
     >>> eprop = g.new_edge_property("double")
     >>> eprop.get_array()[:] = random(g.num_edges())
     >>> print(gt.edge_average(g, eprop))
-    (0.4989741369720412, 0.004101065927783254)
+    (0.49888156584192045, 0.004096739923418754)
     """
 
     if "string" in eprop.value_type():
@@ -374,21 +375,21 @@ def distance_histogram(g, weight=None, bins=[0, 1], samples=None,
 
     Parameters
     ----------
-    g : :class:`Graph`
+    g : :class:`~graph_tool.Graph`
         Graph to be used.
     weight : :class:`~graph_tool.PropertyMap` (optional, default: None)
         Edge weights.
-    bins : list of bins (optional, default: [0, 1])
+    bins : list of bins (optional, default: `[0, 1]`)
         List of bins to be used for the histogram. The values given represent
         the edges of the bins (i.e. lower and upper bounds). If the list
         contains two values, this will be used to automatically create an
         appropriate bin range, with a constant width given by the second value,
         and starting from the first value.
-    samples : int (optional, default: None)
+    samples : int (optional, default: `None`)
         If supplied, the distances will be randomly sampled from a number of
-        source vertices given by this parameter. It `samples == None` (default),
+        source vertices given by this parameter. It `samples is None` (default),
         all pairs are used.
-    float_count : bool (optional, default: True)
+    float_count : bool (optional, default: `True`)
         If True, the counts in each histogram bin will be returned as floats. If
         False, they will be returned as integers.
 
@@ -409,7 +410,7 @@ def distance_histogram(g, weight=None, bins=[0, 1], samples=None,
     Notes
     -----
     The algorithm runs in :math:`O(V^2)` time, or :math:`O(V^2\log V)` if
-    `weight != None`. If `samples` is supplied, the complexities are
+    `weight is not None`. If `samples` is supplied, the complexities are
     :math:`O(\text{samples}\times V)`  and
     :math:`O(\text{samples}\times V\log V)`, respectively.
 
@@ -426,10 +427,10 @@ def distance_histogram(g, weight=None, bins=[0, 1], samples=None,
     >>> g = gt.random_graph(100, lambda: (3, 3))
     >>> hist = gt.distance_histogram(g)
     >>> print(hist)
-    [array([    0.,   300.,   865.,  2214.,  3857.,  2480.,   184.]), array([0, 1, 2, 3, 4, 5, 6, 7], dtype=uint...)]
+    [array([   0.,  300.,  880., 2269., 3974., 2358.,  119.]), array([0, 1, 2, 3, 4, 5, 6, 7], dtype=uint64)]
     >>> hist = gt.distance_histogram(g, samples=10)
     >>> print(hist)
-    [array([   0.,   30.,   88.,  226.,  391.,  240.,   15.]), array([0, 1, 2, 3, 4, 5, 6, 7], dtype=uint...)]
+    [array([  0.,  30.,  87., 223., 394., 239.,  17.]), array([0, 1, 2, 3, 4, 5, 6, 7], dtype=uint64)]
     """
 
     if samples is not None:
